@@ -38,12 +38,14 @@ namespace KDevelop {
             
             Q_SCRIPTABLE virtual bool automaticCompletionEnabled() const = 0;
             
-            Q_SCRIPTABLE virtual bool semanticHighlightingEnabled() const = 0;
+            Q_SCRIPTABLE virtual int localColorizationLevel() const = 0;
             
-            Q_SCRIPTABLE virtual int localVariableColorizationLevel() const = 0;
+            Q_SCRIPTABLE virtual int globalColorizationLevel() const = 0;
             
             Q_SCRIPTABLE virtual bool highlightSemanticProblems() const = 0;
             
+            Q_SCRIPTABLE virtual bool showMultiLineSelectionInformation() const = 0;
+            
         Q_SIGNALS:
             void settingsChanged(ICompletionSettings*);
     };
