@@ -48,6 +48,8 @@ class IRunController;
 class ISourceFormatterController;
 class ISession;
 class ISelectionController;
+class IDocumentationController;
+class IDebugController;
 
 /**
  * ICore is the container class for all the various objects in use by
@@ -105,6 +107,12 @@ public:
 
     /** @return the selection controller */
     Q_SCRIPTABLE virtual KDevelop::ISelectionController* selectionController() = 0;
+    
+    /** @return the documetation controller */
+    Q_SCRIPTABLE virtual KDevelop::IDocumentationController* documentationController() = 0;
+
+    /** @return the debug controller */
+    Q_SCRIPTABLE virtual KDevelop::IDebugController* debugController() = 0;
 
     /** @return the component data of the framework, different from the main component which is created by the application */
     virtual KComponentData componentData() const = 0;
