@@ -1,6 +1,6 @@
 /* This file is part of KDevelop
 Copyright 2006 Adam Treat <treat@kde.org>
-Copyright 2007 Anreas Pakulat <apaku@gmx.de>
+Copyright 2007 Andreas Pakulat <apaku@gmx.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -57,7 +57,7 @@ public:
 
     /**
      * Provides access to the selection model of the project view.
-     * This way its possible to fetch the list of selected items in
+     * This way it is possible to fetch the list of selected items in
      * the project view and use that to prefill some thing.
      * @returns the selection model
      */
@@ -86,7 +86,14 @@ public:
      */
     Q_SCRIPTABLE virtual bool isProjectNameUsed( const QString& name ) const = 0;
 
+    virtual KUrl projectsBaseDirectory() const = 0;
+
 public Q_SLOTS:
+    /**
+     * Tries finding a project-file for the given source-url and opens it.
+     * If no .kdev4 project file is found, the user is asked to import a project.
+     */
+        virtual void openProjectForUrl( const KUrl &sourceUrl ) = 0;
     /**
      * open the project from the given kdev4 project file. This only reads
      * the file and starts creating the project model from it. The opening process
@@ -94,16 +101,16 @@ public Q_SLOTS:
      * @param url a kdev4 project file top open
      * @returns true if the given project could be opened, false otherwise
      */
-    virtual bool openProject( const KUrl & url = KUrl() ) = 0;
+    virtual void openProject( const KUrl & url = KUrl() ) = 0;
     /**
      * close the given project. Closing the project is done in steps and
      * the @ref projectClosing and @ref projectClosed signals are emitted. Only when
-     * the latter signal is emitted its guaranteed that the project has been closed.
+     * the latter signal is emitted it is guaranteed that the project has been closed.
      * The @ref IProject object will be deleted after the closing has finished.
      * @returns true if the project could be closed, false otherwise
      */
-    virtual bool closeProject( IProject* ) = 0;
-    virtual bool configureProject( IProject* ) = 0;
+    virtual void closeProject( IProject* ) = 0;
+    virtual void configureProject( IProject* ) = 0;
 //     virtual void changeCurrentProject( KDevelop::ProjectBaseItem* ) = 0;
 
 Q_SIGNALS:
@@ -125,6 +132,13 @@ Q_SIGNALS:
      * @param project the project that has been closed.
      */
     void projectClosed( KDevelop::IProject* project );
+
+    /**
+     * emitted whenever the project configuration dialog accepted 
+     * changes
+     * @param project the project whose configuration has changed
+     */
+    void projectConfigurationChanged( KDevelop::IProject* project );
 };
 
 }
