@@ -22,7 +22,7 @@
  */
 
 #include "backgroundparser.h"
- 
+
 #include <QList>
 #include <QFile>
 #include <QTimer>
@@ -56,7 +56,7 @@
 #include <editor/modificationrevisionset.h>
 
 //@todo Enable this again once languageForUrl is thread-safe
-const bool separateThreadForHighPriority = false;
+const bool separateThreadForHighPriority = true;
 
 namespace KDevelop
 {
@@ -65,10 +65,10 @@ class BackgroundParserPrivate
 {
 public:
     BackgroundParserPrivate(BackgroundParser *parser, ILanguageController *languageController)
-        :m_parser(parser), m_languageController(languageController)
+        :m_parser(parser), m_languageController(languageController), m_mutex(QMutex::Recursive)
     {
         parser->d = this; //Set this so we can safely call back BackgroundParser from within loadSettings()
-        
+
         m_timer.setSingleShot(true);
         m_delay = 500;
         m_threads = 1;
@@ -82,11 +82,11 @@ public:
 
         loadSettings(); // Start the weaver
     }
-    
+
     void startTimerThreadSafe() {
         QMetaObject::invokeMethod(m_parser, "startTimer", Qt::QueuedConnection);
     }
-    
+
     ~BackgroundParserPrivate()
     {
         suspend();
@@ -112,15 +112,15 @@ public:
         // Create delayed jobs, that is, jobs for documents which have been changed
         // by the user.
         QList<ParseJob*> jobs;
-        
+
         for(QHash<KUrl, DocumentChangeTracker*>::iterator it = m_delayedParseJobs.begin(); it != m_delayedParseJobs.end(); ) {
             KUrl url(it.key());
-            
+
             if(m_parseJobs.contains(url)) {
                 kDebug() << "already parsing" << url << ", delaying the parse-job";
                 ++it; //Add the delayed job later
             }else{
-                ParseJob* job = createParseJob(url, 
+                ParseJob* job = createParseJob(url,
                                             TopDUContext::AllDeclarationsContextsAndUses,
                                             QList<QPointer<QObject> >());
                 if (job) {
@@ -140,31 +140,31 @@ public:
         {
             if(it1.key() > m_neededPriority)
                 break; //The priority is not good enough to be processed right now
-            
+
             for(QSet<KUrl>::Iterator it = it1.value().begin(); it != it1.value().end();) {
                 //Only create parse-jobs for up to thread-count * 2 documents, so we don't fill the memory unnecessarily
                 if(m_parseJobs.count() >= m_threads+1 || (m_parseJobs.count() >= m_threads && !separateThreadForHighPriority) )
                     break;
-                
+
                 if(m_parseJobs.count() >= m_threads && it1.key() > BackgroundParser::NormalPriority && !specialParseJob)
                     break; //The additional parsing thread is reserved for higher priority parsing
-                
+
                 // When a document is scheduled for parsing while it is being parsed, it will be parsed
                 // again once the job finished, but not now.
                 if (m_parseJobs.contains(*it) ) {
                     ++it;
                     continue;
                 }
-                
+
                 kDebug(9505) << "creating parse-job" << *it << "new count of active parse-jobs:" << m_parseJobs.count() + 1;
                 ParseJob* job = createParseJob(*it, m_documents[*it].features(), m_documents[*it].notifyWhenReady());
-                
+
                 if(m_parseJobs.count() == m_threads+1 && !specialParseJob)
                     specialParseJob = job; //This parse-job is allocated into the reserved thread
-                
+
                 if(job)
                     jobs.append(job);
-                
+
                 m_documents.remove(*it);
                 it = it1.value().erase(it);
                 --m_maxParseJobs; //We have added one when putting the document into m_documents
@@ -187,6 +187,14 @@ public:
     {
         QList<ILanguage*> languages = m_languageController->languagesForUrl(url);
         foreach (ILanguage* language, languages) {
+            if(!language) {
+                kWarning(9504) << "got zero language for" << url;
+                continue;
+            }
+            if(!language->languageSupport()) {
+                kWarning(9504) << "language has no language support assigned:" << language->name();
+                continue;
+            }
             ParseJob* job = language->languageSupport()->createParseJob(url);
             if (!job) {
                 continue; // Language part did not produce a valid ParseJob.
@@ -213,6 +221,14 @@ public:
 
         if(languages.isEmpty())
             kWarning(9504) << "found no languages for url" << url;
+        else
+            kWarning(9504) << "could not create parse-job for url" << url;
+
+        //Notify that we failed
+        typedef QPointer<QObject> Notify;
+        foreach(Notify n, notifyWhenReady)
+            if(n)
+                QMetaObject::invokeMethod(n, "updateReady", Qt::DirectConnection, Q_ARG(KDevelop::IndexedString, IndexedString(url)), Q_ARG(KDevelop::ReferencedTopDUContext, ReferencedTopDUContext()));
 
         return 0;
     }
@@ -226,7 +242,7 @@ public:
         m_delay = config.readEntry("Delay", 500);
         m_timer.setInterval(m_delay);
         m_threads = 0;
-        m_parser->setThreadCount(config.readEntry("Number of Threads Internal", 1)); ///@todo Change back to "Numer of Threads" once languageForUrl is thread-safe
+        m_parser->setThreadCount(config.readEntry("Number of Threads", 1));
 
         if (config.readEntry("Enabled", true)) {
             resume();
@@ -266,7 +282,7 @@ public:
 
     //Current parse-job that is executed in the additional thread
     QPointer<ParseJob> specialParseJob;
-    
+
     QTimer m_timer;
     int m_delay;
     int m_threads;
@@ -274,11 +290,14 @@ public:
         QPointer<QObject> notifyWhenReady;
         int priority;
         TopDUContext::Features features;
+        bool operator==(const DocumentParseTarget& rhs) const {
+            return notifyWhenReady == rhs.notifyWhenReady && priority == rhs.priority && features == rhs.features;
+        }
     };
 
     struct DocumentParsePlan {
         QList<DocumentParseTarget> targets;
-        
+
         int priority() const {
             //Pick the best priority
             int ret = BackgroundParser::WorstPriority;
@@ -287,7 +306,7 @@ public:
                     ret = target.priority;
             return ret;
         }
-        
+
         TopDUContext::Features features() const {
             //Pick the best features
             TopDUContext::Features ret = (TopDUContext::Features)0;
@@ -295,14 +314,14 @@ public:
                 ret = (TopDUContext::Features) (ret | target.features);
             return ret;
         }
-        
+
         QList<QPointer<QObject> > notifyWhenReady() {
             QList<QPointer<QObject> > ret;
-            
+
             foreach(const DocumentParseTarget &target, targets)
                 if(target.notifyWhenReady)
                     ret << target.notifyWhenReady;
-            
+
             return ret;
         }
     };
@@ -359,6 +378,7 @@ void BackgroundParser::loadSettings(bool projectIsLoaded)
 {
     Q_UNUSED(projectIsLoaded)
 
+
     d->loadSettings();
 }
 
@@ -384,27 +404,27 @@ void BackgroundParser::parseProgress(KDevelop::ParseJob* job, float value, QStri
 void BackgroundParser::revertAllRequests(QObject* notifyWhenReady)
 {
     QPointer<QObject> p(notifyWhenReady);
-    
+
     QMutexLocker lock(&d->m_mutex);
     for(QMap<KUrl, BackgroundParserPrivate::DocumentParsePlan >::iterator it = d->m_documents.begin(); it != d->m_documents.end(); ) {
-        
+
         d->m_documentsForPriority[it.value().priority()].remove(it.key());
-        
+
         int index = -1;
         for(int a = 0; a < (*it).targets.size(); ++a)
             if((*it).targets[a].notifyWhenReady == notifyWhenReady)
                 index = a;
-        
+
         if(index != -1) {
             (*it).targets.removeAt(index);
             if((*it).targets.isEmpty()) {
                 it = d->m_documents.erase(it);
                 --d->m_maxParseJobs;
-                
+
                 continue;
             }
         }
-        
+
         d->m_documentsForPriority[it.value().priority()].insert(it.key());
         ++it;
     }
@@ -416,29 +436,25 @@ void BackgroundParser::addDocument(const KUrl& url, TopDUContext::Features featu
     QMutexLocker lock(&d->m_mutex);
     {
         Q_ASSERT(url.isValid());
-        
+
         BackgroundParserPrivate::DocumentParseTarget target;
         target.priority = priority;
         target.features = features;
         target.notifyWhenReady = QPointer<QObject>(notifyWhenReady);
-        
+
         QMap<KUrl, BackgroundParserPrivate::DocumentParsePlan >::iterator it = d->m_documents.find(url);
-        
+
         if (it != d->m_documents.end()) {
             //Update the stored plan
-            
+
             d->m_documentsForPriority[it.value().priority()].remove(url);
             it.value().targets << target;
             d->m_documentsForPriority[it.value().priority()].insert(url);
-        }
-        
-        if (it == d->m_documents.end()) {
+        }else{
 //             kDebug(9505) << "BackgroundParser::addDocument: queuing" << url;
             d->m_documents[url].targets << target;
             d->m_documentsForPriority[d->m_documents[url].priority()].insert(url);
             ++d->m_maxParseJobs; //So the progress-bar waits for this document
-        } else {
-//             kDebug(9505) << "BackgroundParser::addDocument: is already queued:" << url;
         }
 
         d->startTimerThreadSafe();
@@ -451,16 +467,27 @@ void BackgroundParser::addDocumentList(const KUrl::List &urls, TopDUContext::Fea
         addDocument(url, features, priority);
 }
 
-void BackgroundParser::removeDocument(const KUrl &url)
+void BackgroundParser::removeDocument(const KUrl &url, QObject* notifyWhenReady)
 {
     QMutexLocker lock(&d->m_mutex);
 
     Q_ASSERT(url.isValid());
 
     if(d->m_documents.contains(url)) {
+        
         d->m_documentsForPriority[d->m_documents[url].priority()].remove(url);
-        d->m_documents.remove(url);
-        --d->m_maxParseJobs;
+        
+        foreach(BackgroundParserPrivate::DocumentParseTarget target, d->m_documents[url].targets)
+            if(target.notifyWhenReady == notifyWhenReady)
+                d->m_documents[url].targets.removeAll(target);
+
+        if(d->m_documents[url].targets.isEmpty()) {
+            d->m_documents.remove(url);
+            --d->m_maxParseJobs;
+        }else{
+            //Insert with an eventually different priority
+            d->m_documentsForPriority[d->m_documents[url].priority()].insert(url);
+        }
     }
 }
 
@@ -473,22 +500,27 @@ void BackgroundParser::parseDocuments()
 
 void BackgroundParser::parseComplete(ThreadWeaver::Job* job)
 {
-    QMutexLocker lock(&d->m_mutex);
-
     if (ParseJob* parseJob = qobject_cast<ParseJob*>(job)) {
 
         emit parseJobFinished(parseJob);
 
-        d->m_parseJobs.remove(parseJob->document().str());
-
-        d->m_jobProgress.remove(parseJob);
-        
-        parseJob->setBackgroundParser(0);
-
-		delete parseJob;
-
-        ++d->m_doneParseJobs;
-        updateProgressBar();
+        {
+            {
+                QMutexLocker lock(&d->m_mutex);
+    
+                d->m_parseJobs.remove(parseJob->document().str());
+    
+                d->m_jobProgress.remove(parseJob);
+    
+                parseJob->setBackgroundParser(0);
+    
+                ++d->m_doneParseJobs;
+                updateProgressBar();
+            }
+            //Unlock the mutex before deleting the parse-job, because the parse-job
+            //has a virtual destructor that may lock the duchain, leading to deadlocks
+            delete parseJob;
+        }
 
         //Continue creating more parse-jobs
         QMetaObject::invokeMethod(this, "parseDocuments", Qt::QueuedConnection);
@@ -505,6 +537,11 @@ void BackgroundParser::enableProcessing()
     setNeededPriority(WorstPriority);
 }
 
+bool BackgroundParser::isQueued(KUrl url) const {
+    QMutexLocker lock(&d->m_mutex);
+    return d->m_documents.contains(url) || d->m_delayedParseJobs.contains(url);
+}
+
 void BackgroundParser::setNeededPriority(int priority)
 {
     QMutexLocker lock(&d->m_mutex);
@@ -537,11 +574,11 @@ void BackgroundParser::updateProgressBar()
         float additionalProgress = 0;
         for(QMap<KDevelop::ParseJob*, float>::const_iterator it = d->m_jobProgress.constBegin(); it != d->m_jobProgress.constEnd(); ++it)
             additionalProgress += *it;
-        
+
         emit showProgress(this, 0, d->m_maxParseJobs*1000, (additionalProgress + d->m_doneParseJobs)*1000);
     }
 }
- 
+
 ParserDependencyPolicy* BackgroundParser::dependencyPolicy() const
 {
     return &d->m_dependencyPolicy;
@@ -574,16 +611,24 @@ void BackgroundParser::setDelay(int miliseconds)
     }
 }
 
+QList< KUrl > BackgroundParser::managedDocuments()
+{
+    QMutexLocker l(&d->m_mutex);
+    return d->m_managedRanges.values();
+}
+
 void BackgroundParser::addManagedTopRange(const KUrl& document, KTextEditor::SmartRange* range)
 {
     range->addWatcher(this);
+    QMutexLocker l(&d->m_mutex);
     d->m_managedRanges.insert(range, document);
 }
 
 void BackgroundParser::removeManagedTopRange(KTextEditor::SmartRange* range)
 {
-    range->removeWatcher(this);
-    d->m_managedRanges.remove(range);
+   range->removeWatcher(this);
+   QMutexLocker l(&d->m_mutex);
+   d->m_managedRanges.remove(range);
 }
 
 void BackgroundParser::rangeContentsChanged(KTextEditor::SmartRange* range, KTextEditor::SmartRange* mostSpecificChild)
@@ -616,7 +661,7 @@ void BackgroundParser::rangeContentsChanged(KTextEditor::SmartRange* range, KTex
     }
 
     newTracker->addChangedRange( mostSpecificChild );
-    
+
     d->startTimerThreadSafe();
 }
 
