@@ -65,6 +65,21 @@ void CompletionTreeElement::setParent(CompletionTreeElement* parent) {
     }
 }
 
+void CompletionTreeNode::appendChildren(QList< KSharedPtr< KDevelop::CompletionTreeElement > > children) {
+  for(QList< KSharedPtr< KDevelop::CompletionTreeElement > >::const_iterator it = children.begin(); it != children.end(); ++it)
+    appendChild(*it);
+}
+
+void CompletionTreeNode::appendChildren(QList< KSharedPtr< KDevelop::CompletionTreeItem > > children) {
+  for(QList< KSharedPtr< KDevelop::CompletionTreeItem > >::iterator it = children.begin(); it != children.end(); ++it)
+    appendChild(KSharedPtr< KDevelop::CompletionTreeElement >((*it).data()));
+}
+
+void CompletionTreeNode::appendChild(KSharedPtr< KDevelop::CompletionTreeElement > child) {
+  child->setParent(this);
+  children << child;
+}
+
 int CompletionTreeElement::columnInParent() const {
   return 0;
 }
@@ -123,12 +138,30 @@ int CompletionTreeItem::argumentHintDepth() const
   return 0;
 }
 
+KTextEditor::CodeCompletionModel::CompletionProperties CompletionTreeItem::completionProperties() const {
+  Declaration* dec = declaration().data();
+  if(!dec)
+    return (KTextEditor::CodeCompletionModel::CompletionProperties)0;
+
+  return DUChainUtils::completionProperties(dec);
+}
+
 DeclarationPointer CompletionTreeItem::declaration() const {
   return DeclarationPointer();
 }
 
-IndexedType CompletionTreeItem::typeForArgumentMatching() const {
-  return IndexedType();
+QList<IndexedType> CompletionTreeItem::typeForArgumentMatching() const {
+  return QList<IndexedType>();
+}
+
+CompletionCustomGroupNode::CompletionCustomGroupNode(QString groupName, int _inheritanceDepth) {
+  role = (KTextEditor::CodeCompletionModel::ExtraItemDataRoles)Qt::DisplayRole;
+  roleValue = groupName;
+  inheritanceDepth = _inheritanceDepth;
+}
+
+bool CompletionTreeItem::dataChangedWithInput() const {
+  return false;
 }
 
 }
