@@ -51,63 +51,166 @@
 
 #include "codecompletionworker.h"
 #include "codecompletioncontext.h"
+#include <duchain/specializationstore.h>
 
 using namespace KTextEditor;
 
+//Multi-threaded completion creates some multi-threading related crashes, and sometimes shows the completions in the wrong position if the cursor was moved
+// #define SINGLE_THREADED_COMPLETION
+
 namespace KDevelop {
 
+struct CompletionWorkerThread : public QThread {
+   CompletionWorkerThread(CodeCompletionModel* model) : QThread(model), m_model(model), m_worker(0) {
+   }
+   ~CompletionWorkerThread() {
+     delete m_worker;
+   }
+   
+   virtual void run () {
+     if(!m_worker)
+      m_worker = m_model->createCompletionWorker();
+     
+     //We connect directly, so we can do the pre-grouping within the background thread
+     connect(m_worker, SIGNAL(foundDeclarationsReal(QList<KSharedPtr<CompletionTreeElement> >, KSharedPtr<CodeCompletionContext>)), m_model, SLOT(foundDeclarations(QList<KSharedPtr<CompletionTreeElement> >, KSharedPtr<CodeCompletionContext>)), Qt::QueuedConnection);
+
+     connect(m_model, SIGNAL(completionsNeeded(KDevelop::DUContextPointer, const KTextEditor::Cursor&, KTextEditor::View*)), m_worker, SLOT(computeCompletions(KDevelop::DUContextPointer, const KTextEditor::Cursor&, KTextEditor::View*)), Qt::QueuedConnection);
+     connect(m_model, SIGNAL(doSpecialProcessingInBackground(uint)), m_worker, SLOT(doSpecialProcessing(uint)));
+     exec();
+   }
+   
+   CodeCompletionModel* m_model;
+   CodeCompletionWorker* m_worker;
+};
+
+bool CodeCompletionModel::forceWaitForModel()
+{
+  return m_forceWaitForModel;
+}
+
+void CodeCompletionModel::setForceWaitForModel(bool wait)
+{
+  m_forceWaitForModel = wait;
+}
+
+
 CodeCompletionModel::CodeCompletionModel( QObject * parent )
   : CodeCompletionModel2(parent)
+  , m_forceWaitForModel(false)
   , m_fullCompletion(true)
   , m_mutex(new QMutex)
-  , m_worker(0)
+  , m_thread(0)
 {
   qRegisterMetaType<QList<CompletionTreeElement> >("QList<KSharedPtr<CompletionTreeElement> >");
+  qRegisterMetaType<KSharedPtr<CodeCompletionContext> >("KSharedPtr<CodeCompletionContext>");
   qRegisterMetaType<KTextEditor::Cursor>("KTextEditor::Cursor");
 }
 
+void CodeCompletionModel::initialize() {
+  if(!m_thread) {
+    m_thread = new CompletionWorkerThread(this);
+#ifdef SINGLE_THREADED_COMPLETION
+    m_thread->m_worker = createCompletionWorker();
+#endif
+    m_thread->start();
+  }
+}
+
 CodeCompletionModel::~CodeCompletionModel()
 {
-  // Let it leak...??
-  if (m_worker) {
-    m_worker->setParent(0L);
-    m_worker->quit();
-  }
-
+  if(m_thread->m_worker)
+    m_thread->m_worker->abortCurrentCompletion();
+  m_thread->quit();
+  m_thread->wait();
+  
+  delete m_thread;
   delete m_mutex;
 }
 
-void CodeCompletionModel::setCompletionWorker(CodeCompletionWorker* worker)
-{
-  if (m_worker) {
-    kWarning() << "Already have a current code completion worker!";
-    return;
-  }
-
-  m_worker = worker;
-
-  //We connect directly, so we can do the pre-grouping within the background thread
-  connect(m_worker, SIGNAL(foundDeclarations(QList<KSharedPtr<CompletionTreeElement> >, void*)), this, SLOT(foundDeclarations(QList<KSharedPtr<CompletionTreeElement> >, void*)), Qt::QueuedConnection);
-
-  connect(this, SIGNAL(completionsNeeded(KDevelop::DUContextPointer, const KTextEditor::Cursor&, KTextEditor::View*)), m_worker, SLOT(computeCompletions(KDevelop::DUContextPointer, const KTextEditor::Cursor&, KTextEditor::View*)), Qt::QueuedConnection);
-
-  m_worker->start();
-}
-
 void CodeCompletionModel::addNavigationWidget(const CompletionTreeElement* element, QWidget* widget) const
 {
   Q_ASSERT(dynamic_cast<QuickOpenEmbeddedWidgetInterface*>(widget));
   m_navigationWidgets[element] = widget;
 }
 
+
 bool CodeCompletionModel::fullCompletion() const
 {
   return m_fullCompletion;
 }
 
+KDevelop::CodeCompletionWorker* CodeCompletionModel::worker() const {
+  return m_thread->m_worker;
+}
+
+void CodeCompletionModel::clear() {
+  m_completionItems.clear();
+  m_navigationWidgets.clear();
+  m_completionContext.clear();
+  reset();
+}
+
+void CodeCompletionModel::completionInvokedInternal(KTextEditor::View* view, const KTextEditor::Range& range, InvocationType invocationType, const KUrl& url)
+{
+  Q_UNUSED(invocationType)
+
+  DUChainReadLocker lock(DUChain::lock(), 400);
+  if( !lock.locked() ) {
+    kDebug() << "could not lock du-chain in time";
+    return;
+  }
+
+  TopDUContext* top = DUChainUtils::standardContextForUrl( url );
+  if(!top) {
+    return;
+  }
+  setCurrentTopContext(TopDUContextPointer(top));
+
+  if (top) {
+    kDebug() << "completion invoked for context" << (DUContext*)top;
+
+    if( top->parsingEnvironmentFile() && top->parsingEnvironmentFile()->modificationRevision() != ModificationRevision::revisionForFile(IndexedString(url.pathOrUrl())) ) {
+      kDebug() << "Found context is not current. Its revision is " /*<< top->parsingEnvironmentFile()->modificationRevision() << " while the document-revision is " << ModificationRevision::revisionForFile(IndexedString(url.pathOrUrl()))*/;
+    }
+
+    DUContextPointer thisContext;
+    {
+      thisContext = SpecializationStore::self().applySpecialization(top->findContextAt(SimpleCursor(range.start())), top);
+      if(!thisContext)
+        thisContext = top;
+
+       kDebug() << "context is set to" << thisContext.data();
+        if( thisContext ) {
+/*          kDebug() << "================== duchain for the context =======================";
+          DumpChain dump;
+          dump.dump(thisContext.data());*/
+        } else {
+          kDebug() << "================== NO CONTEXT FOUND =======================";
+          m_completionItems.clear();
+          m_navigationWidgets.clear();
+          reset();
+          return;
+        }
+    }
+
+    lock.unlock();
+
+#if KDE_IS_VERSION(4,2,85)
+    if(m_forceWaitForModel)
+      emit waitForReset();
+#endif
+    
+    emit completionsNeeded(thisContext, range.start(), view);
+  } else {
+    kDebug() << "Completion invoked for unknown context. Document:" << url << ", Known documents:" << DUChain::self()->documents();
+  }
+}
+
 
 void CodeCompletionModel::completionInvoked(KTextEditor::View* view, const KTextEditor::Range& range, InvocationType invocationType)
 {
+  //If this triggers, initialize() has not been called after creation.
+  Q_ASSERT(m_thread);
   
   KDevelop::ICompletionSettings::CompletionLevel level = KDevelop::ICore::self()->languageController()->completionSettings()->completionLevel();
   if(level == KDevelop::ICompletionSettings::AlwaysFull || (invocationType != AutomaticInvocation && level == KDevelop::ICompletionSettings::MinimalWhenAutomatic))
@@ -120,7 +223,7 @@ void CodeCompletionModel::completionInvoked(KTextEditor::View* view, const KText
   
   Q_UNUSED(invocationType)
 
-  if (!m_worker)
+  if (!worker())
     kWarning() << "Completion invoked on a completion model which has no code completion worker assigned!";
 
   m_navigationWidgets.clear();
@@ -128,18 +231,30 @@ void CodeCompletionModel::completionInvoked(KTextEditor::View* view, const KText
 
   reset();
 
-  m_worker->abortCurrentCompletion();
-  m_worker->setFullCompletion(m_fullCompletion);
+  worker()->abortCurrentCompletion();
+  worker()->setFullCompletion(m_fullCompletion);
 
   KUrl url = view->document()->url();
 
   completionInvokedInternal(view, range, invocationType, url);
 }
 
-void CodeCompletionModel::foundDeclarations(QList<KSharedPtr<CompletionTreeElement> > items, void* completionContext)
+void CodeCompletionModel::foundDeclarations(QList<KSharedPtr<CompletionTreeElement> > items, KSharedPtr<CodeCompletionContext> completionContext)
 {
+  m_completionContext = completionContext;
+  
+  if(m_completionItems.isEmpty() && items.isEmpty()) {
+    if(m_forceWaitForModel)
+      reset(); //If we need to reset the model, reset it
+    return; //We don't need to reset, which is bad for target model
+  }
+  
   m_completionItems = items;
-  m_completionContext = KSharedPtr<CodeCompletionContext>((CodeCompletionContext*)completionContext);
+  
+  
+  if(m_completionContext)
+    kDebug() << "got completion-context with " << m_completionContext->ungroupedElements().size() << "ungrouped elements";
+
   reset();
 
 /*  if (completionContext == m_completionContext.data()) {
@@ -156,6 +271,9 @@ void CodeCompletionModel::setCompletionContext(KSharedPtr<CodeCompletionContext>
 {
   QMutexLocker lock(m_mutex);
   m_completionContext = completionContext;
+
+  if(m_completionContext)
+    kDebug() << "got completion-context with " << m_completionContext->ungroupedElements().size() << "ungrouped elements";
 }
 
 KSharedPtr<CodeCompletionContext> CodeCompletionModel::completionContext() const
@@ -164,15 +282,9 @@ KSharedPtr<CodeCompletionContext> CodeCompletionModel::completionContext() const
   return m_completionContext;
 }
 
-///@todo move into subclass
 void CodeCompletionModel::executeCompletionItem2(Document* document, const Range& word, const QModelIndex& index) const
 {
-  DUChainReadLocker lock(DUChain::lock(), 3000);
-  if(!lock.locked()) {
-    kDebug(9007) << "Failed to lock the du-chain for completion-item execution"; //Probably we prevented a deadlock
-    return;
-  }
-
+  //We must not lock the duchain at this place, because the items might rely on that
   CompletionTreeElement* element = (CompletionTreeElement*)index.internalPointer();
   if( !element || !element->asItem() )
     return;
@@ -180,6 +292,11 @@ void CodeCompletionModel::executeCompletionItem2(Document* document, const Range
   element->asItem()->execute(document, word);
 }
 
+KSharedPtr< KDevelop::CompletionTreeElement > CodeCompletionModel::itemForIndex(QModelIndex index) const {
+  CompletionTreeElement* element = (CompletionTreeElement*)index.internalPointer();
+  return KSharedPtr< KDevelop::CompletionTreeElement >(element);
+}
+
 QVariant CodeCompletionModel::data(const QModelIndex& index, int role) const
 {
   CompletionTreeElement* element = (CompletionTreeElement*)index.internalPointer();
@@ -192,22 +309,26 @@ QVariant CodeCompletionModel::data(const QModelIndex& index, int role) const
     if( treeElement.asNode() ) {
       return QVariant(treeElement.asNode()->role);
     }else {
-      kDebug(9007) << "Requested group-role from leaf tree element";
+      kDebug() << "Requested group-role from leaf tree element";
       return QVariant();
     }
   }else{
     if( treeElement.asNode() ) {
+      if( role == CodeCompletionModel::InheritanceDepth ) {
+        CompletionCustomGroupNode* customGroupNode = dynamic_cast<CompletionCustomGroupNode*>(element);
+        if(customGroupNode)
+          return QVariant(customGroupNode->inheritanceDepth);
+      }
       if( role == treeElement.asNode()->role ) {
         return treeElement.asNode()->roleValue;
       } else {
-        kDebug(9007) << "Requested wrong role from non-leaf tree element";
         return QVariant();
       }
     }
   }
 
   if(!treeElement.asItem()) {
-    kWarning(9007) << "Error in completion model";
+    kWarning() << "Error in completion model";
     return QVariant();
   }
 
@@ -234,7 +355,7 @@ QVariant CodeCompletionModel::data(const QModelIndex& index, int role) const
   QVariant ret = treeElement.asItem()->data(index, role, this);
 
   //In reduced completion mode, don't show information text with the selected items
-  if(!m_fullCompletion && role == ItemSelected)
+  if(role == ItemSelected && (!m_fullCompletion || !ICore::self()->languageController()->completionSettings()->showMultiLineSelectionInformation()))
     return QVariant();
   
   return ret;
@@ -258,7 +379,7 @@ QModelIndex CodeCompletionModel::index(int row, int column, const QModelIndex& p
     CompletionTreeNode* node = element->asNode();
 
     if( !node ) {
-      kDebug(9007) << "Requested sub-index of leaf node";
+      kDebug() << "Requested sub-index of leaf node";
       return QModelIndex();
     }
 
@@ -305,6 +426,12 @@ int CodeCompletionModel::rowCount ( const QModelIndex & parent ) const
   }
 }
 
+QString CodeCompletionModel::filterString(KTextEditor::View* view, const KTextEditor::SmartRange& range, const KTextEditor::Cursor& position)
+{
+  m_filterString = KTextEditor::CodeCompletionModelControllerInterface::filterString(view, range, position);
+  return m_filterString;
+}
+
 }
 
 #include "codecompletionmodel.moc"
