@@ -37,8 +37,9 @@ class KDEVPLATFORMLANGUAGE_EXPORT CodeRepresentation {
     virtual ~CodeRepresentation() {
     }
     virtual QString line(int line) const = 0;
+    virtual int lines() const = 0;
     virtual QString text() const = 0;
-    //Overwrites the text in the file with the new given one
+    ///Overwrites the text in the file with the new given one
     ///@return true on success
     virtual bool setText(QString) = 0;
 };
@@ -47,7 +48,7 @@ class KDEVPLATFORMLANGUAGE_EXPORT DynamicCodeRepresentation : public CodeReprese
   public:
       ///Used to group edit-history together. Call this before a bunch of replace(), and endEdit in the end.
       virtual void startEdit() = 0;
-      virtual bool replace(const KTextEditor::Range& range, QString oldText, QString newText) = 0;
+      virtual bool replace(const KTextEditor::Range& range, QString oldText, QString newText, bool ignoreOldText = false) = 0;
       virtual QString rangeText(KTextEditor::Range range) const = 0;
       ///Must be called exactly once per startEdit()
       virtual void endEdit() = 0;
