@@ -18,14 +18,23 @@
 
 #include "createclass.h"
 
+#include <QDirIterator>
+#include <QFile>
+
 #include <KListWidget>
 #include <KLineEdit>
+#include <kdebug.h>
+#include <kmessagebox.h>
 
 #include "ui_newclass.h"
 #include "ui_licensechooser.h"
 #include "ui_outputlocation.h"
 
 #include "overridespage.h"
+#include <kfiledialog.h>
+#include <kstandarddirs.h>
+#include <kcomponentdata.h>
+#include <kconfig.h>
 
 using namespace KDevelop;
 
@@ -67,6 +76,7 @@ void CreateClass::setup()
 
 KUrl CreateClass::headerUrlFromBase(QString className, KUrl baseUrl)
 {
+    Q_UNUSED(baseUrl);
     KUrl url;
     url.addPath(className);
     return url;
@@ -74,6 +84,8 @@ KUrl CreateClass::headerUrlFromBase(QString className, KUrl baseUrl)
 
 KUrl CreateClass::implementationUrlFromBase(QString className, KUrl baseUrl)
 {
+    Q_UNUSED(baseUrl);
+    Q_UNUSED(className);
     return KUrl();
 }
 
@@ -215,30 +227,26 @@ QStringList ClassIdentifierPage::inheritanceList() const
     return ret;
 }
 
-class KDevelop::LicensePagePrivate
+struct KDevelop::LicensePagePrivate
 {
-public:
+    struct LicenseInfo
+    {
+        QString name;
+        QString path;
+        QString contents;
+    };
+    typedef QList<LicenseInfo> LicenseList;
+
+
     LicensePagePrivate()
         : license(0)
     {
     }
 
     Ui::LicenseChooserDialog* license;
+    LicenseList availableLicenses;
 };
 
-static const char* s_lgpl =
-"   This library is free software; you can redistribute it and/or\n"
-"   modify it under the terms of the GNU Library General Public\n"
-"   License version 2 as published by the Free Software Foundation.\n\n"
-"   This library is distributed in the hope that it will be useful,\n"
-"   but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-"   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
-"   Library General Public License for more details.\n\n"
-"   You should have received a copy of the GNU Library General Public License\n"
-"   along with this library; see the file COPYING.LIB.  If not, write to\n"
-"   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,\n"
-"   Boston, MA 02110-1301, USA.\n";
-
 LicensePage::LicensePage(QWizard* parent)
     : QWizardPage(parent)
     , d(new LicensePagePrivate)
@@ -250,20 +258,141 @@ LicensePage::LicensePage(QWizard* parent)
     d->license->setupUi(this);
 
     connect(d->license->licenseComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(licenseComboChanged(int)));
+    connect(d->license->saveLicense, SIGNAL(clicked(bool)), d->license->licenseName, SLOT(setEnabled(bool)));
 
-    d->license->licenseTextEdit->setText(s_lgpl);
+    // Read all the available licenses from the standard dirs
+    initializeLicenses();
+    
+    //Set the license selection to the previous one
+    KConfigGroup config(KSharedConfig::openConfig(), "CodeGeneration");
+    d->license->licenseComboBox->setCurrentIndex(config.readEntry( "LastSelectedLicense", 0 ));
+    //Needed to avoid a bug where licenseComboChanged doesn't get called by QComboBox if the past selection was 0
+    licenseComboChanged(d->license->licenseComboBox->currentIndex());
 
     registerField("license", d->license->licenseTextEdit);
 }
 
-LicensePage::~LicensePage()
+LicensePage::~LicensePage(void)
 {
+    KConfigGroup config(KSharedConfig::openConfig(), "CodeGeneration");
+    config.writeEntry("LastSelectedLicense", d->license->licenseComboBox->currentIndex());
+    config.config()->sync();
+    
     delete d;
 }
 
-void LicensePage::licenseComboChanged(int license)
+// If the user entered a custom license that they want to save, save it
+bool LicensePage::validatePage(void)
 {
-    //TODO
+    if(d->license->licenseComboBox->currentIndex() == (d->availableLicenses.size() - 1) && 
+        d->license->saveLicense->isChecked())
+        return saveLicense();
+    else
+        return true;
+}
+
+//! Read all the license files in the global and local config dirs
+void LicensePage::initializeLicenses(void)
+{
+    kDebug() << "Searching for available licenses";
+    KStandardDirs * dirs = KGlobal::dirs();
+    QStringList licenseDirs = dirs->findDirs("data", "kdevcodegen/licenses");
+    
+    //Iterate through the possible directories that contain licenses, and load their names
+    foreach(QString currentDir, licenseDirs)
+    {
+        QDirIterator it(currentDir, QDir::Files | QDir::Readable);
+        while(it.hasNext())
+        {
+            LicensePagePrivate::LicenseInfo newLicense;
+            newLicense.path = it.next();
+            newLicense.name = it.fileName();
+            
+            kDebug() << "Found License: " << newLicense.name;
+            
+            d->availableLicenses.push_back(newLicense);
+            d->license->licenseComboBox->addItem(newLicense.name);
+        }
+    }
+    
+    //Finally add the option other for user specified licenses
+    LicensePagePrivate::LicenseInfo license;
+    d->availableLicenses.push_back(license);
+    d->license->licenseComboBox->addItem("Other");
+}
+
+// Read a license index, if it is not loaded, open it from the file
+QString & LicensePage::readLicense(int licenseIndex)
+{
+    //If the license is not loaded into memory, read it in
+    if(d->availableLicenses[licenseIndex].contents.isNull())
+    {
+        QString licenseText("");
+        //If we are dealing with the last option "other" just return a new empty string
+        if(licenseIndex != (d->availableLicenses.size() - 1))
+        {
+            kDebug() << "Reading license: " << d->availableLicenses[licenseIndex].name ;
+            QFile newLicense(d->availableLicenses[licenseIndex].path);
+            
+            if(newLicense.open(QIODevice::ReadOnly))
+            {
+                licenseText = newLicense.readAll();
+                newLicense.close();
+            }
+            else
+                licenseText = "Error, could not open license file.\n Was it deleted?";
+        }
+        
+        d->availableLicenses[licenseIndex].contents = licenseText;
+    }
+    
+    return d->availableLicenses[licenseIndex].contents;
+}
+
+// ---Slots---
+
+void LicensePage::licenseComboChanged(int selectedLicense)
+{
+    //If the last slot is selected enable the save license combobox
+    if(selectedLicense == (d->availableLicenses.size() - 1))
+    {
+        d->license->licenseTextEdit->clear();
+        d->license->licenseTextEdit->setReadOnly(false);
+        d->license->saveLicense->setEnabled(true);
+    }
+    else
+    {
+        d->license->saveLicense->setEnabled(false);
+        d->license->licenseTextEdit->setReadOnly(true);
+    }
+    
+    d->license->licenseTextEdit->setText(readLicense(selectedLicense));
+}
+
+bool LicensePage::saveLicense(void)
+{
+    kDebug() << "Attempting to save custom license: " << d->license->licenseName->text();
+    
+    QString localDataDir = KStandardDirs::locateLocal("data", "kdevcodegen/licenses/", KGlobal::activeComponent());
+    QFile newFile(localDataDir + d->license->licenseName->text());
+    
+    if(newFile.exists())
+    {
+        KMessageBox::sorry(this, i18n("The specified license already exists. Please provide a different name."));
+        return false;
+    }
+    
+    newFile.open(QIODevice::WriteOnly);
+    qint64 result = newFile.write(d->license->licenseTextEdit->toPlainText().toUtf8());
+    newFile.close();
+    
+    if(result == -1)
+    {
+        KMessageBox::sorry(this, i18n("There was an error writing the file."));
+        return false;
+    }
+
+    return true;
 }
 
 class KDevelop::OutputPagePrivate
@@ -288,6 +417,10 @@ OutputPage::OutputPage(CreateClass* parent)
 
     d->output = new Ui::OutputLocationDialog;
     d->output->setupUi(this);
+    d->output->headerUrl->setMode( KFile::File | KFile::LocalOnly );
+    d->output->headerUrl->fileDialog()->setOperationMode( KFileDialog::Saving );
+    d->output->implementationUrl->setMode( KFile::File | KFile::LocalOnly );
+    d->output->implementationUrl->fileDialog()->setOperationMode( KFileDialog::Saving );
 
     registerField("headerUrl*", d->output->headerUrl);
     registerField("implementationUrl*", d->output->implementationUrl);
