@@ -19,10 +19,10 @@
 #ifndef KDEV_CREATECLASS_H
 #define KDEV_CREATECLASS_H
 
-#include <QWizard>
+#include <QtGui/QWizard>
+#include <KDE/KUrl>
 
 #include "../languageexport.h"
-#include <kurl.h>
 
 class KLineEdit;
 class KUrl;
@@ -30,6 +30,7 @@ class KUrl;
 namespace KDevelop {
 
 class OverridesPage;
+class IndexedDeclaration;
 
 class KDEVPLATFORMLANGUAGE_EXPORT ClassIdentifierPage : public QWizardPage
 {
@@ -85,19 +86,28 @@ private:
     class ClassIdentifierPagePrivate* const d;
 };
 
+//!@todo Add the name of the Author at the top of the license
 class KDEVPLATFORMLANGUAGE_EXPORT LicensePage : public QWizardPage
 {
     Q_OBJECT
 
 public:
     LicensePage(QWizard* parent);
-    virtual ~LicensePage();
+    virtual ~LicensePage(void);
+    
+    bool validatePage(void);
 
 public Q_SLOTS:
     virtual void licenseComboChanged(int license);
 
 private:
+    // data
     class LicensePagePrivate* const d;
+    
+    // methods
+    void        initializeLicenses(void);
+    QString &   readLicense(int licenseIndex);
+    bool        saveLicense(void);
 };
 
 /**
