@@ -163,14 +163,14 @@ public:
       }
     }
     {
-      DUChainReadLocker lock( DUChain::lock() );
+      /*DUChainReadLocker lock( DUChain::lock() );
       //foreach(DUContext* context, topLevelContext->childContexts());
       kDebug() << "built top-level context with" << top->localDeclarations().count() << "declarations," << top->localDeclarations().count() << " Definitions and" << top->childContexts().size() << "Child-Contexts";
 
       foreach( DUContext* contexts, top->childContexts() )
       {
         kDebug() << "CHILD:" << contexts->scopeIdentifier( true ) << "Parent:" << ( dynamic_cast<TopDUContext*>( contexts->parentContext() ) ? "top-context" : "" );
-      }
+      }*/
     }
     m_compilingContexts = false;
     return top;
@@ -300,9 +300,9 @@ protected:
   inline DUContext* lastContext() const { return m_lastContext; }
   /// Clears the last closed context.
   inline void clearLastContext() { m_lastContext = 0; }
-  
+
   inline void setLastContext(DUContext* context) { m_lastContext = context; }
-  
+
   /**
    * Determine if we are recompiling an existing definition-use chain, or if
    * a new chain is being created from scratch.
@@ -697,7 +697,7 @@ protected:
     Q_ASSERT( m_compilingContexts );
     DUContext* ret = 0L;
     if(range.start > range.end)
-      kDebug(9007) << "Bad context-range" << range.textRange();
+      kDebug() << "Bad context-range" << range.textRange();
 
     {
       if ( recompiling() )
@@ -712,10 +712,10 @@ protected:
 
 //         if(iface)
 //           kDebug() << "translated by" << (translated.start.textCursor() - range.start.textCursor()) << (translated.end.textCursor() - range.end.textCursor()) << "to revision" << iface->currentRevision();
-        
+
         int currentIndex = nextContextIndex();
         int lookingAhead = 0;
-        
+
         for ( ; currentIndex < childContexts.count(); ++currentIndex )
         {
           DUContext* child = childContexts.at( currentIndex );
@@ -786,7 +786,7 @@ protected:
         if (!identifier.isEmpty())
           ret->setLocalScopeIdentifier(identifier);
 
-        ret->setInSymbolTable(type == DUContext::Class || type == DUContext::Namespace || type == DUContext::Global || type == DUContext::Helper || type == DUContext::Enum);
+        setInSymbolTable(ret);
 
 //         if( recompiling() )
 //           kDebug() << "created new context while recompiling for " << identifier.toString() << "(" << ret->range().textRange() << ")";
@@ -798,6 +798,16 @@ protected:
     return ret;
   }
 
+  ///This function should call context->setInSymbolTable(..) with an appropriate decision. The duchain is write-locked when this is called.
+  virtual void setInSymbolTable(DUContext* context) {
+    if(!context->parentContext()->inSymbolTable()) {
+      context->setInSymbolTable(false);
+      return;
+    }
+    DUContext::ContextType type = context->type();
+    context->setInSymbolTable(type == DUContext::Class || type == DUContext::Namespace || type == DUContext::Global || type == DUContext::Helper || type == DUContext::Enum);
+  }
+
 private:
 
   Identifier m_identifier;
