@@ -16,8 +16,8 @@
    Boston, MA 02110-1301, USA.
 */
 
-#ifndef KDEV_USEBUILDER_H
-#define KDEV_USEBUILDER_H
+#ifndef ABSTRACTUSEBUILDER_H
+#define ABSTRACTUSEBUILDER_H
 
 #include "../declaration.h"
 #include "../use.h"
@@ -88,7 +88,7 @@ protected:
 
     SimpleRange newRange = editorFindRange(name, name);
 
-    DUChainWriteLocker lock(DUChain::lock());
+    DUChainWriteLocker lock(DUChain::lock()); ///@todo Don't call findDeclarations during write-lock, it can lead to UI lockups
     QList<Declaration*> declarations = LanguageSpecificUseBuilderBase::currentContext()->findDeclarations(id, newRange.start);
     foreach (Declaration* declaration, declarations)
       if (!declaration->isForwardDeclaration()) {
@@ -102,6 +102,8 @@ protected:
     newUse( newRange, !declarations.isEmpty() ? declarations.first() : 0 );
   }
 
+  ///@todo Work this over! We must not pass around "Declaration*" values if the duchain is not locked.
+
   /**
    * Register a new use for a \a declaration with a \a node.
    *
@@ -122,12 +124,8 @@ protected:
   void newUse(SimpleRange newRange, Declaration* declaration)
   {
     DUChainWriteLocker lock(DUChain::lock());
-    if(!declaration) {
-      kDebug() << "Tried to create use of zero declaration";
-      return;
-    }
 
-    bool encountered = false;
+    bool encountered = false; ///@todo This can cause I/O, so it would be better if it was possible with only a read-lock
     int declarationIndex = LanguageSpecificUseBuilderBase::currentContext()->topContext()->indexForUsedDeclaration(declaration);
     int contextUpSteps = 0; //We've got to use the stack here, and not parentContext(), because the order may be different
 
@@ -230,7 +228,7 @@ protected:
 
       LockedSmartInterface iface = LanguageSpecificUseBuilderBase::editor()->smart();
       //Delete all ranges that were not re-used
-      if(this->currentContext()->smartRange()) {
+      if(this->currentContext()->smartRange() && iface) {
         this->currentContext()->takeUseRanges();
         foreach(KTextEditor::SmartRange* range, currentUseTracker().reuseRanges) {
 #ifdef DEBUG_UPDATE_MATCHING
@@ -249,7 +247,7 @@ protected:
       for(int a = 0; a < tracker.createUses.size(); ++a) {
         KTextEditor::SmartRange* range = 0;
         
-        if(this->currentContext()->smartRange()) {
+        if(this->currentContext()->smartRange() && iface) {
           range = tracker.createUses[a].second;
           Q_ASSERT(range);
         }
@@ -277,5 +275,5 @@ private:
 
 }
 
-#endif // USEBUILDER_H
+#endif // ABSTRACTUSEBUILDER_H
 
