@@ -2,6 +2,7 @@
     Copyright 2002-2005 Roberto Raggi <roberto@kdevelop.org>
     Copyright 2006 Adam Treat <treat@kde.org>
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -132,6 +133,16 @@ void ClassFunctionDeclaration::setIsAbstract(bool abstract)
   d_func_dynamic()->m_functionFlags = (ClassFunctionFlags)setFlag(abstract, AbstractFunctionFlag, d_func()->m_functionFlags);
 }
 
+bool ClassFunctionDeclaration::isFinal() const
+{
+  return d_func()->m_functionFlags & FinalFunctionFlag;
+}
+
+void ClassFunctionDeclaration::setIsFinal(bool final)
+{
+  d_func_dynamic()->m_functionFlags = (ClassFunctionFlags)setFlag(final, FinalFunctionFlag, d_func()->m_functionFlags);
+}
+
 bool ClassFunctionDeclaration::isSignal() const
 {
   return d_func()->m_functionFlags & FunctionSignalFlag;
