@@ -45,35 +45,39 @@
 #include "functiondefinition.h"
 #include "codemodel.h"
 #include "specializationstore.h"
+#include "types/typeutils.h"
+#include "types/typealiastype.h"
 
 using namespace KTextEditor;
 
 namespace KDevelop
 {
 
+///@todo Use reference counting
 Repositories::StringRepository commentRepository("Comment Repository");
 
 REGISTER_DUCHAIN_ITEM(Declaration);
 
-DeclarationData::DeclarationData() 
-  : m_comment(0), m_isDefinition(false), m_inSymbolTable(false),  
-    m_isTypeAlias(false), m_anonymousInContext(false)
+DeclarationData::DeclarationData()
+  : m_comment(0), m_isDefinition(false), m_inSymbolTable(false),
+    m_isTypeAlias(false), m_anonymousInContext(false), m_isFinal(false)
 {
   m_kind = Declaration::Instance;
 }
-  
-DeclarationData::DeclarationData( const DeclarationData& rhs ) : DUChainBaseData(rhs)
+
+DeclarationData::DeclarationData( const DeclarationData& rhs ) : DUChainBaseData(rhs),
+m_internalContext(rhs.m_internalContext),
+m_type(rhs.m_type),
+m_identifier(rhs.m_identifier),
+m_declaration(rhs.m_declaration),
+m_comment(rhs.m_comment),
+m_kind(rhs.m_kind),
+m_isDefinition(rhs.m_isDefinition),
+m_inSymbolTable(rhs.m_inSymbolTable),
+m_isTypeAlias(rhs.m_isTypeAlias),
+m_anonymousInContext(rhs.m_anonymousInContext),
+m_isFinal(rhs.m_isFinal)
 {
-  m_identifier = rhs.m_identifier;
-  m_declaration = rhs.m_declaration;
-  m_type = rhs.m_type;
-  m_kind = rhs.m_kind;
-  m_isDefinition = rhs.m_isDefinition;
-  m_isTypeAlias = rhs.m_isTypeAlias;
-  m_inSymbolTable = rhs.m_inSymbolTable;
-  m_comment = rhs.m_comment;
-  m_anonymousInContext = rhs.m_anonymousInContext;
-  m_internalContext = rhs.m_internalContext;
 }
 
 Declaration::Kind Declaration::kind() const {
@@ -84,6 +88,7 @@ Declaration::Kind Declaration::kind() const {
 void Declaration::setKind(Kind kind) {
   DUCHAIN_D_DYNAMIC(Declaration);
   d->m_kind = kind;
+  updateCodeModel();
 }
 
 bool Declaration::inDUChain() const {
@@ -93,7 +98,7 @@ bool Declaration::inDUChain() const {
   if( !context() )
     return false;
   TopDUContext* top = topContext();
-  return top && top->inDuChain();
+  return top && top->inDUChain();
 }
 
 Declaration::Declaration( const SimpleRange& range, DUContext* context )
@@ -140,22 +145,22 @@ Declaration::Declaration( DeclarationData & dd, const SimpleRange& range )
 Declaration::~Declaration()
 {
   uint oldOwnIndex = m_indexInTopContext;
-  
+
   TopDUContext* topContext = this->topContext();
-  
+
   //Only perform the actions when the top-context isn't being deleted, or when it hasn't been stored to disk
   if(!topContext->deleting() || !topContext->isOnDisk()) {
     DUCHAIN_D_DYNAMIC(Declaration);
     // Inserted by the builder after construction has finished.
     if( d->m_internalContext.context() )
       d->m_internalContext.context()->setOwner(0);
-    
+
     if (d->m_inSymbolTable && !d->m_identifier.isEmpty()) {
       QualifiedIdentifier id = qualifiedIdentifier();
       PersistentSymbolTable::self().removeDeclaration(id, this);
       CodeModel::self().removeItem(url(), id);
     }
-    
+
     d->m_inSymbolTable = false;
   }
 
@@ -165,9 +170,9 @@ Declaration::~Declaration()
     if(!topContext->deleting() || !topContext->isOnDisk() || context()->d_func()->isDynamic())
       context()->m_dynamicData->removeDeclaration(this);
   }
-    
+
   clearOwnIndex();
-  
+
   if(!topContext->deleting() || !topContext->isOnDisk()) {
     setContext(0);
 
@@ -262,11 +267,11 @@ Declaration* IndexedDeclaration::declaration() const {
 void Declaration::rebuildDynamicData(DUContext* parent, uint ownIndex)
 {
   DUChainBase::rebuildDynamicData(parent, ownIndex);
-  
+
   m_context = parent;
   m_topContext = parent->topContext();
   m_indexInTopContext = ownIndex;
-  
+
   parent->m_dynamicData->addDeclarationToHash(d_func()->m_identifier.identifier(), this);
 }
 
@@ -275,14 +280,14 @@ void Declaration::setIdentifier(const Identifier& identifier)
   ENSURE_CAN_WRITE
   DUCHAIN_D_DYNAMIC(Declaration);
   bool wasInSymbolTable = d->m_inSymbolTable;
-  
+
   setInSymbolTable(false);
-    
+
   if( m_context && !d->m_anonymousInContext )
     m_context->changingIdentifier( this, d->m_identifier, identifier );
 
   d->m_identifier = identifier;
-  
+
   setInSymbolTable(wasInSymbolTable);
   //DUChain::declarationChanged(this, DUChainObserver::Change, DUChainObserver::Identifier);
 }
@@ -295,7 +300,7 @@ IndexedType Declaration::indexedType() const
 AbstractType::Ptr Declaration::abstractType( ) const
 {
   //ENSURE_CAN_READ Commented out for performance reasons
-  return d_func()->m_type.type();
+  return d_func()->m_type.abstractType();
 }
 
 void Declaration::setAbstractType(AbstractType::Ptr type)
@@ -313,7 +318,7 @@ void Declaration::setAbstractType(AbstractType::Ptr type)
     //DUChain::declarationChanged(this, DUChainObserver::Addition, DUChainObserver::DataType);
 }
 
-Declaration* Declaration::specialize(uint /*specialization*/, const TopDUContext* /*topContext*/, int /*upDistance*/)
+Declaration* Declaration::specialize(IndexedInstantiationInformation /*specialization*/, const TopDUContext* /*topContext*/, int /*upDistance*/)
 {
   return this;
 }
@@ -356,7 +361,7 @@ void Declaration::setContext(DUContext* context, bool anonymous)
 {
   Q_ASSERT(!context || context->topContext());
   ///@todo re-enable. In C++ support we need a short window to put visible declarations into template contexts
-  if(!specialization()) {
+  if(!specialization().index()) {
     //problem: specialization() doesn't work during destructor
 //     ENSURE_CAN_WRITE
   }
@@ -365,8 +370,8 @@ void Declaration::setContext(DUContext* context, bool anonymous)
 
   //We don't need to clear, because it's not allowed to move from one top-context into another
 //   clearOwnIndex();
-  
-  
+
+
   DUCHAIN_D_DYNAMIC(Declaration);
   if (m_context && context) {
     Q_ASSERT(m_context->topContext() == context->topContext());
@@ -383,14 +388,14 @@ void Declaration::setContext(DUContext* context, bool anonymous)
     m_topContext = context->topContext();
   else
     m_topContext = 0;
-  
+
   d->m_anonymousInContext = anonymous;
   m_context = context;
 
   if (context) {
     if(!m_indexInTopContext)
       allocateOwnIndex();
-    
+
     if(!d->m_anonymousInContext) {
       context->m_dynamicData->addDeclaration(this);
       //DUChain::declarationChanged(this, DUChainObserver::Addition, DUChainObserver::Context, m_context);
@@ -402,14 +407,14 @@ void Declaration::setContext(DUContext* context, bool anonymous)
 }
 
 void Declaration::clearOwnIndex() {
-  
+
   if(!m_indexInTopContext)
     return;
-  
+
   if(!context() || (!d_func()->m_anonymousInContext && !context()->isAnonymous())) {
     ENSURE_CAN_WRITE
   }
-  
+
   if(m_indexInTopContext) {
     Q_ASSERT(m_topContext);
     m_topContext->m_dynamicData->clearDeclarationIndex(this);
@@ -418,7 +423,7 @@ void Declaration::clearOwnIndex() {
 }
 
 void Declaration::allocateOwnIndex() {
-  
+
   ///@todo Fix multithreading stuff with template instantiation, preferably using some internal mutexes
 //   if(context() && (!context()->isAnonymous() && !d_func()->m_anonymousInContext)) {
 //     ENSURE_CAN_WRITE
@@ -428,10 +433,10 @@ void Declaration::allocateOwnIndex() {
 
   m_indexInTopContext = m_topContext->m_dynamicData->allocateDeclarationIndex(this, d_func()->m_anonymousInContext || !context() || context()->isAnonymous());
   Q_ASSERT(m_indexInTopContext);
-  
+
   if(!m_topContext->m_dynamicData->getDeclarationForIndex(m_indexInTopContext))
     kFatal() << "Could not re-retrieve declaration" << "index:" << m_indexInTopContext;
-  
+
 }
 
 const Declaration* Declaration::logicalDeclaration(const TopDUContext* topContext) const {
@@ -467,10 +472,18 @@ DUContext * Declaration::logicalInternalContext(const TopDUContext* topContext)
 
   if( d_func()->m_isTypeAlias ) {
     ///If this is a type-alias, return the internal context of the actual type.
-    AbstractType::Ptr t = abstractType();
-    IdentifiedType* idType = dynamic_cast<IdentifiedType*>(t.unsafeData());
-    if( idType && idType->declaration(topContext) && idType->declaration(topContext) != this )
-      return idType->declaration(topContext)->logicalInternalContext( topContext );
+    TypeAliasType::Ptr t = type<TypeAliasType>();
+    if(t) {
+      AbstractType::Ptr target = t->type();
+      
+      IdentifiedType* idType = dynamic_cast<IdentifiedType*>(target.unsafeData());
+      if( idType ) {
+        Declaration* decl = idType->declaration(topContext);
+        if(decl && decl != this) {
+          return decl->logicalInternalContext( topContext );
+        }
+      }
+    }
   }
 
   return internalContext();
@@ -497,15 +510,15 @@ void Declaration::setInternalContext(DUContext* context)
     m_topContext = context->topContext();
     allocateOwnIndex();
   }
-  
+
   DUContext* oldInternalContext = d->m_internalContext.context();
-  
+
   d->m_internalContext = context;
 
   //Q_ASSERT( !oldInternalContext || oldInternalContext->owner() == this );
   if( oldInternalContext && oldInternalContext->owner() == this )
     oldInternalContext->setOwner(0);
-  
+
 
   if( context )
     context->setOwner(this);
@@ -544,6 +557,16 @@ void Declaration::setDeclarationIsDefinition(bool dd)
 //   }
 }
 
+bool Declaration::isFinal() const
+{
+  return d_func()->m_isFinal;
+}
+
+void Declaration::setFinal(bool final)
+{
+  d_func_dynamic()->m_isFinal = final;
+}
+
 ///@todo see whether it would be useful to create an own TypeAliasDeclaration sub-class for this
 bool Declaration::isTypeAlias() const {
   DUCHAIN_D(Declaration);
@@ -555,15 +578,15 @@ void Declaration::setIsTypeAlias(bool isTypeAlias) {
   d->m_isTypeAlias = isTypeAlias;
 }
 
-uint Declaration::specialization() const {
-  return 0;
+IndexedInstantiationInformation Declaration::specialization() const {
+  return IndexedInstantiationInformation();
 }
 
 void Declaration::activateSpecialization()
 {
-  if(specialization()) {
+  if(specialization().index()) {
     DeclarationId baseId(id());
-    baseId.setSpecialization(0);
+    baseId.setSpecialization(IndexedInstantiationInformation());
     SpecializationStore::self().set(baseId, specialization());
   }
 }
@@ -585,20 +608,23 @@ bool Declaration::inSymbolTable() const
 
 CodeModelItem::Kind kindForDeclaration(Declaration* decl) {
   CodeModelItem::Kind kind = CodeModelItem::Unknown;
-  
+
+  if(decl->kind() == Declaration::Namespace)
+    return CodeModelItem::Namespace;
+
   if(decl->isFunctionDeclaration()) {
     kind = CodeModelItem::Function;
   }
-  
+
   if(decl->kind() == Declaration::Type && decl->type<StructureType>())
     kind = CodeModelItem::Class;
-  
+
   if(kind == CodeModelItem::Unknown && decl->kind() == Declaration::Instance)
     kind = CodeModelItem::Variable;
-  
+
   if(decl->isForwardDeclaration())
     kind = (CodeModelItem::Kind)(kind | CodeModelItem::ForwardDeclaration);
-  
+
   return kind;
 }
 
@@ -618,14 +644,14 @@ void Declaration::setInSymbolTable(bool inSymbolTable)
     if(!d->m_inSymbolTable && inSymbolTable) {
       QualifiedIdentifier id(qualifiedIdentifier());
       PersistentSymbolTable::self().addDeclaration(id, this);
-      
+
       CodeModel::self().addItem(url(), id, kindForDeclaration(this));
     }
-  
+
     else if(d->m_inSymbolTable && !inSymbolTable) {
       QualifiedIdentifier id(qualifiedIdentifier());
       PersistentSymbolTable::self().removeDeclaration(id, this);
-      
+
       CodeModel::self().removeItem(url(), id);
     }
   }
@@ -677,7 +703,7 @@ bool Declaration::equalQualifiedIdentifier(const Declaration* rhs) const {
   DUCHAIN_D(Declaration);
   if(d->m_identifier != rhs->d_func()->m_identifier)
     return false;
-  
+
   return m_context->equalScopeIdentifier(m_context);
 }
 
