@@ -22,7 +22,7 @@
 
 #include <QtCore/QList>
 #include <QtCore/QMap>
-#include <QtCore/qcontainerfwd.h>
+//#include <QtCore/qcontainerfwd.h>
 
 #include "../editor/documentrangeobject.h"
 #include "identifier.h"
@@ -65,14 +65,16 @@ public:
   enum AccessPolicy {
     Public    /**< a public declaration */,
     Protected /**< a protected declaration */,
-    Private   /**< a private declaration */
+    Private   /**< a private declaration */,
+    DefaultAccess /**<a declaration with default access; in java, only package-level access. */
   };
   /// Enumeration of the types of declarations
   enum Kind {
     Type     /**< A type is declared, like a class-declaration or function-declaration, or a typedef("class MyClass {};") */,
     Instance /**< An instance of a type is declared("MyClass m;") */,
     NamespaceAlias/**< This is a namespace-alias. You can safely cast this object to NamespaceAliasDeclaration. */,
-    Alias /**<This is an alias-declaration. You can safely cast this object to AliasDeclaration. */
+    Alias, /**<This is an alias-declaration. You can safely cast this object to AliasDeclaration. */
+    Namespace /**< Declaration of a namespace. */
   };
 
   /**
@@ -125,6 +127,11 @@ public:
   /// Set whether this declaration is a type alias. \param typeAlias true if the declaration is a type alias, otherwise false.
   void setIsTypeAlias(bool typeAlias);
 
+  /// Access whether the declaration is final, ie, it can only be written to once [Java]
+  bool isFinal() const;
+  /// Set whether the declaration is final. \param final true if the declaration is final, otherwise false.
+  void setFinal(bool final);
+
   /**
    * Retrieve the declaration which is specialized with the given \a specialization index as seen from \a topContext.
    *
@@ -133,7 +140,7 @@ public:
    * \param upDistance upwards distance in the context-structure of the
    *                   given specialization-info. This allows specializing children.
    * */
-  virtual Declaration* specialize(uint specialization, const TopDUContext* topContext, int upDistance = 0);
+  virtual Declaration* specialize(IndexedInstantiationInformation specialization, const TopDUContext* topContext, int upDistance = 0);
 
   /**
    * Retrieve the context that is opened by this declaration, if one exists.
@@ -355,7 +362,7 @@ public:
   /**
    *
    * */
-  virtual uint specialization() const;
+  virtual IndexedInstantiationInformation specialization() const;
 
   /**
    * @see DeclarationId
@@ -376,7 +383,7 @@ public:
 
   ///Whether this declaration has been inserted anonymously into its parent-context
   bool isAnonymous() const;
-  
+
   /**
    * Clear the index for this declaration in the top context that was allocated with allocateOwnIndex().
    */
