@@ -74,11 +74,23 @@ public:
    * @param notifyReady An optional pointer to a QObject that should contain a slot
    *                    "void updateReady(KDevelop::IndexedString url, KDevelop::ReferencedTopDUContext topContext)".
    *                    The notification is guaranteed to be called once for each call to updateContextForUrl. The given top-context
-   *                    may be invalid if the update failed.
+   *                    may be invalid if the update failed. A queued connection is used if a re-parse has to be done.
    * @return Whether an update was scheduled. If false is returned, the top-context is already up-to-date, and has the needed features.
    */
    Q_SCRIPTABLE void updateContextForUrl(const IndexedString& document, TopDUContext::Features minFeatures, QObject* notifyReady = 0) const;
   
+   /**
+    * Convenience-function similar to updateContextForUrl that blocks this thread until the update of the given document is ready,
+    * and returns the top-context.
+    * @param document The document to update
+    * @param features The requested features. If you want to force a full update of the context, give TopDUContext::ForceUpdate.
+    *                 If you want to force an update including all imports, use TopDUContext::ForceUpdateRecursive.
+    * @return The up-to-date top-context, or zero if the update failed
+    * @warning The duchain must _not_ be locked when this is called!
+    *
+    */
+   KDevelop::ReferencedTopDUContext waitForUpdate(const IndexedString& document, TopDUContext::Features minFeatures, bool wantProxyContext = false);
+   
   /**
    * Return any chain for the given document
    * If available, the version accepting IndexedString should be used instead of this, for performance reasons.
@@ -150,6 +162,9 @@ public:
    */
   Q_SCRIPTABLE void addDocumentChain(TopDUContext* chain);
 
+  /// Returns true if the global duchain instance has already been deleted
+  Q_SCRIPTABLE static bool deleted();
+  
   /// Returns the global static instance.
   Q_SCRIPTABLE static DUChain* self();
 
@@ -202,6 +217,24 @@ public:
   ///Call this from within tests.
   void disablePersistentStorage();
   
+  ///Stores the whole duchain and all its repositories in the current state to disk
+  ///The duchain must not be locked in any way
+  void storeToDisk();
+  
+  ///Compares the whole duchain and all its repositories in the current state to disk
+  ///When the comparison fails, debug-output will show why
+  ///The duchain must not be locked when calling this
+  ///@return true If the current memory state equals the disk state, else false
+  bool compareToDisk();
+  
+  ///Does a big cleanup of the repositories, removing all items that are not needed any more for disk-persistency
+  ///The duchain must not be locked when calling this
+  ///@warning Only do this right after startup, or after shutdown, in a moment where indexed
+  ///         repository items are not needed any more, since the reference-counting mechanisms only respect disk persistency.
+  ///         All items that are not disk-referenced are invalidated during this step, which will lead to inconsistency, crashes,
+  ///         etc. when done in the wrong moment.
+  void finalCleanup();
+  
 Q_SIGNALS:
   ///Is emitted when the declaration has been selected somewhere in the user-interface, for example in the completion-list
   void declarationSelected(DeclarationPointer decl);
@@ -214,7 +247,9 @@ public Q_SLOTS:
 private Q_SLOTS:
   void documentActivated(KDevelop::IDocument* doc);
   void documentAboutToBeDeleted(KTextEditor::Document* doc);
+  void documentAboutToBeDeletedFinal(KTextEditor::Document* doc);
   void documentLoadedPrepare(KDevelop::IDocument* document);
+  void documentRenamed(KDevelop::IDocument* document);
   void aboutToQuit();
 private:
   /// Increases the reference-count for the given top-context. The result: It will not be unloaded.
