@@ -40,6 +40,7 @@
 #include "functiondefinition.h"
 #include "specializationstore.h"
 #include "persistentsymboltable.h"
+#include "classdeclaration.h"
 
 using namespace KDevelop;
 using namespace KTextEditor;
@@ -48,31 +49,33 @@ CodeCompletionModel::CompletionProperties DUChainUtils::completionProperties(Dec
 {
   CodeCompletionModel::CompletionProperties p;
 
-  if (ClassMemberDeclaration* member = dynamic_cast<ClassMemberDeclaration*>(dec)) {
-    switch (member->accessPolicy()) {
-      case Declaration::Public:
-        p |= CodeCompletionModel::Public;
-        break;
-      case Declaration::Protected:
-        p |= CodeCompletionModel::Protected;
-        break;
-      case Declaration::Private:
-        p |= CodeCompletionModel::Private;
-        break;
-    }
+  if(dec->context()->type() == DUContext::Class) {
+    if (ClassMemberDeclaration* member = dynamic_cast<ClassMemberDeclaration*>(dec)) {
+      switch (member->accessPolicy()) {
+        case Declaration::Public:
+          p |= CodeCompletionModel::Public;
+          break;
+        case Declaration::Protected:
+          p |= CodeCompletionModel::Protected;
+          break;
+        case Declaration::Private:
+          p |= CodeCompletionModel::Private;
+          break;
+      }
 
-    if (member->isStatic())
-      p |= CodeCompletionModel::Static;
-    if (member->isAuto())
-      {}//TODO
-    if (member->isFriend())
-      p |= CodeCompletionModel::Friend;
-    if (member->isRegister())
-      {}//TODO
-    if (member->isExtern())
-      {}//TODO
-    if (member->isMutable())
-      {}//TODO
+      if (member->isStatic())
+        p |= CodeCompletionModel::Static;
+      if (member->isAuto())
+        {}//TODO
+      if (member->isFriend())
+        p |= CodeCompletionModel::Friend;
+      if (member->isRegister())
+        {}//TODO
+      if (member->isExtern())
+        {}//TODO
+      if (member->isMutable())
+        {}//TODO
+    }
   }
 
   if (AbstractFunctionDeclaration* function = dynamic_cast<AbstractFunctionDeclaration*>(dec)) {
@@ -164,9 +167,9 @@ QIcon DUChainUtils::iconForProperties(KTextEditor::CodeCompletionModel::Completi
       RETURN_CACHED_ICON("CVprotected_slot")
     else if( p & CodeCompletionModel::Private )
       RETURN_CACHED_ICON("CVprivate_slot")
-    else
+    else if(p & CodeCompletionModel::Public )
       RETURN_CACHED_ICON("CVpublic_slot")
-
+    else RETURN_CACHED_ICON("slot")
   else if( p & CodeCompletionModel::Signal )
     if( p & CodeCompletionModel::Protected )
       RETURN_CACHED_ICON("CVprotected_signal")
@@ -305,6 +308,9 @@ Declaration* DUChainUtils::declarationForDefinition(Declaration* definition, Top
 }
 
 Declaration* DUChainUtils::declarationInLine(const KDevelop::SimpleCursor& cursor, DUContext* ctx) {
+  if(!ctx)
+    return 0;
+  
   foreach(Declaration* decl, ctx->localDeclarations())
     if(decl->range().start.line == cursor.line)
       return decl;
@@ -402,11 +408,15 @@ QList<IndexedDeclaration> DUChainUtils::collectAllVersions(Declaration* decl) {
 QList<Declaration*> DUChainUtils::getInheriters(const Declaration* decl, uint& maxAllowedSteps, bool collectVersions)
 {
   QList<Declaration*> ret;
+  
+  if(!dynamic_cast<const ClassDeclaration*>(decl))
+    return ret;
+
   if(maxAllowedSteps == 0)
     return ret;
 
   if(decl->internalContext() && decl->internalContext()->type() == DUContext::Class)
-    FOREACH_FUNCTION(IndexedDUContext importer, decl->internalContext()->indexedImporters) {
+    FOREACH_ARRAY(IndexedDUContext importer, decl->internalContext()->indexedImporters()) {
       
       DUContext* imp = importer.data();
       
@@ -427,12 +437,14 @@ QList<Declaration*> DUChainUtils::getInheriters(const Declaration* decl, uint& m
     const IndexedDeclaration* allDeclarations;
     PersistentSymbolTable::self().declarations(decl->qualifiedIdentifier(), count, allDeclarations);
     for(uint a = 0; a < count; ++a) {
+      ++maxAllowedSteps;
+      
       if(allDeclarations[a].data() && allDeclarations[a].data() != decl) {
         ret += getInheriters(allDeclarations[a].data(), maxAllowedSteps, false);
-        
-        if(maxAllowedSteps == 0)
-          return ret;
       }
+      
+      if(maxAllowedSteps == 0)
+        return ret;
     }
   }
   
@@ -511,3 +523,18 @@ Declaration* DUChainUtils::getOverridden(const Declaration* decl) {
     
   return 0;
 }
+
+DUContext* DUChainUtils::getFunctionContext(Declaration* decl) {
+  DUContext* functionContext = decl->internalContext();
+  if(functionContext && functionContext->type() != DUContext::Function) {
+    foreach(DUContext::Import import, functionContext->importedParentContexts()) {
+      DUContext* ctx = import.context(decl->topContext());
+      if(ctx && ctx->type() == DUContext::Function)
+        functionContext = ctx;
+    }
+  }
+  
+  if(functionContext && functionContext->type() == DUContext::Function)
+    return functionContext;
+  return 0;
+}
