@@ -2,6 +2,7 @@
  * DUChain Utilities
  *
  * Copyright 2007 Hamish Rodda <rodda@kde.org>
+ * Copyright 2007-2009 David Nolden <david.nolden.kdevelop@art-master.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Library General Public License as
@@ -81,6 +82,7 @@ namespace DUChainUtils {
   ///@param collectVersions If this is true, the persistent symbol table is used to first find all registered
   ///                       versions of this class, and then get the inheriters from them all together. This is needed for C++.
   ///@param maxAllowedSteps The maximum of steps allowed. If this is zero in the end, this means the search has been stopped with the max. reached
+  ///                                           If you really want _all_ inheriters, you should initialize it with a very large value.
   KDEVPLATFORMLANGUAGE_EXPORT QList<Declaration*> getInheriters(const Declaration* decl, uint& maxAllowedSteps, bool collectVersions = true);
 
   ///Gets all functions that override the function @param overriddenDeclaration, starting the search at @param currentClass
@@ -95,6 +97,10 @@ namespace DUChainUtils {
 
   ///Returns the declaration that is overridden by the given one, or zero.
   KDEVPLATFORMLANGUAGE_EXPORT Declaration* getOverridden(const Declaration* decl);
+  
+  ///If the given declaration is a function-declaration, this follows the context-structure up to the function-context that contains the arguments,
+  ///and returns it.
+  KDEVPLATFORMLANGUAGE_EXPORT DUContext* getFunctionContext(Declaration* decl);
 }
 }
 
