@@ -1,5 +1,6 @@
 /* This  is part of KDevelop
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2009 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -43,7 +44,7 @@
 #include "duchainregister.h"
 #include "topducontextdynamicdata.h"
 #include "arrayhelpers.h"
-#include "codemodel.h"
+#include "importers.h"
 
 ///It is fine to use one global static mutex here
 
@@ -71,7 +72,8 @@ DEFINE_LIST_MEMBER_HASH(DUContextData, m_uses, Use)
 
 REGISTER_DUCHAIN_ITEM(DUContext);
 
-const Identifier globalImportIdentifier("{...import...}");
+//This is ugly, but it prevents a possible crash during destruction. We don't need ~Identifier to be called.
+const Identifier& globalImportIdentifier(*new Identifier("{...import...}"));
 
 void DUContext::rebuildDynamicData(DUContext* parent, uint ownIndex) {
 
@@ -115,12 +117,36 @@ void DUContextDynamicData::scopeIdentifier(bool includeClasses, QualifiedIdentif
     target += m_context->d_func()->m_scopeIdentifier;
 }
 
-bool DUContextDynamicData::isThisImportedBy(const DUContext* context) const {
+bool DUContextDynamicData::importsSafeButSlow(const DUContext* context, const TopDUContext* source, ImportsHash& checked) const {
   if( this == context->m_dynamicData )
     return true;
 
-  FOREACH_FUNCTION( IndexedDUContext ctx, m_context->d_func()->m_importers ) {
-    if( IndexedTopDUContext(ctx.topContextIndex()).isLoaded() && ctx.context() && ctx.context()->m_dynamicData->isThisImportedBy(context) )
+  if(checked.find(this) != checked.end())
+    return false;
+  checked.insert(std::make_pair(this, true));
+
+  FOREACH_FUNCTION( const DUContext::Import& ctx, m_context->d_func()->m_importedContexts ) {
+    DUContext* import = ctx.context(source);
+    if(import == context || (import && import->m_dynamicData->importsSafeButSlow(context, source, checked)))
+      return true;
+  }
+
+  return false;
+}
+
+bool DUContextDynamicData::imports(const DUContext* context, const TopDUContext* source, int maxDepth) const {
+  if( this == context->m_dynamicData )
+    return true;
+
+  if(maxDepth == 0) {
+    ImportsHash checked(500);
+    checked.set_empty_key(0);
+    return importsSafeButSlow(context, source, checked);
+  }
+
+  FOREACH_FUNCTION( const DUContext::Import& ctx, m_context->d_func()->m_importedContexts ) {
+    DUContext* import = ctx.context(source);
+    if(import == context || (import && import->m_dynamicData->imports(context, source, maxDepth-1)))
       return true;
   }
 
@@ -289,7 +315,7 @@ bool DUContextDynamicData::needsLocalDeclarationsHash()
   //including those in propagating sub-contexts.
   //Then, also make sure that we create the declaration hash after loading if needed
   return false;
-  
+
   if(m_context->d_func()->m_localDeclarationsSize() > 15)
     return true;
 
@@ -347,7 +373,7 @@ void DUContextDynamicData::addDeclaration( Declaration * newDeclaration )
   }
 
   SimpleCursor start = newDeclaration->range().start;
-
+///@todo Do binary search to find the position
   bool inserted = false;
   for (int i = m_context->d_func_dynamic()->m_localDeclarationsSize()-1; i >= 0; --i) {
     Declaration* child = m_context->d_func_dynamic()->m_localDeclarations()[i].data(m_topContext);
@@ -361,13 +387,13 @@ void DUContextDynamicData::addDeclaration( Declaration * newDeclaration )
       insertToArray(m_context->d_func_dynamic()->m_localDeclarationsList(), newDeclaration, i+1);
       if(!m_context->d_func()->m_localDeclarations()[i+1].data(m_topContext))
         kFatal() << "Inserted a not addressable declaration";
-      
+
       inserted = true;
       break;
     }
   }
-  if( !inserted )
-    m_context->d_func_dynamic()->m_localDeclarationsList().append(newDeclaration);
+  if( !inserted ) //We haven't found any child that is before this one, so prepend it
+    insertToArray(m_context->d_func_dynamic()->m_localDeclarationsList(), newDeclaration, 0);
 
     addDeclarationToHash(newDeclaration->identifier(), newDeclaration);
   }
@@ -411,32 +437,25 @@ void DUContextDynamicData::addChildContext( DUContext * context )
     //If this context is temporary, added declarations should be as well
     Q_ASSERT(indexed.localIndex() > (0xffffffff/2));
   }
-  
+
   bool inserted = false;
 
   int childCount = m_context->d_func()->m_childContextsSize();
 
-  //Optimization: In most cases while parsing, the new child-context will be added to the end, so check if it is the case.
-  if(m_context->d_func()->m_childContextsSize() != 0) {
-    if(m_context->d_func()->m_childContexts()[childCount-1].data(m_topContext)->range().start <= context->range().start)
-      goto insertAtEnd;
-  }
-
-  for (int i = 0; i < childCount; ++i) {
+  for (int i = childCount-1; i >= 0; --i) {///@todo Do binary search to find the position
     DUContext* child = m_context->d_func()->m_childContexts()[i].data(m_topContext);
     if (context == child)
       return;
-    if (context->range().start < child->range().start) {
-      insertToArray(m_context->d_func_dynamic()->m_childContextsList(), indexed, i);
+    if (context->range().start >= child->range().start) {
+      insertToArray(m_context->d_func_dynamic()->m_childContextsList(), indexed, i+1);
       context->m_dynamicData->m_parentContext = m_context;
       inserted = true;
       break;
     }
   }
 
-  insertAtEnd:
   if( !inserted ) {
-    m_context->d_func_dynamic()->m_childContextsList().append(indexed);
+    m_context->d_func_dynamic()->m_childContextsList().insert(indexed, 0);
     context->m_dynamicData->m_parentContext = m_context;
   }
 
@@ -462,12 +481,20 @@ bool DUContextDynamicData::removeChildContext( DUContext* context ) {
 void DUContextDynamicData::addImportedChildContext( DUContext * context )
 {
 //   ENSURE_CAN_WRITE
-  if(arrayContains(m_context->d_func_dynamic()->m_importersList(), IndexedDUContext(context))) {
-    kDebug(9505) << m_context->scopeIdentifier(true).toString() << "importer added multiple times:" << context->scopeIdentifier(true).toString();
-    return;
-  }
+  DUContext::Import import(m_context, context);
+  
+  if(import.isDirect()) {
+    //Direct importers are registered directly within the data
+    if(arrayContains(m_context->d_func_dynamic()->m_importersList(), IndexedDUContext(context))) {
+      kDebug(9505) << m_context->scopeIdentifier(true).toString() << "importer added multiple times:" << context->scopeIdentifier(true).toString();
+      return;
+    }
 
-  m_context->d_func_dynamic()->m_importersList().append(context);
+    m_context->d_func_dynamic()->m_importersList().append(context);
+  }else{
+    //Indirect importers are registered separately
+    Importers::self().addImporter(import.indirectDeclarationId(), IndexedDUContext(context));
+  }
 
   //DUChain::contextChanged(m_context, DUChainObserver::Addition, DUChainObserver::ImportedChildContexts, context);
 }
@@ -476,9 +503,14 @@ void DUContextDynamicData::addImportedChildContext( DUContext * context )
 void DUContextDynamicData::removeImportedChildContext( DUContext * context )
 {
 //   ENSURE_CAN_WRITE
-  removeOne(m_context->d_func_dynamic()->m_importersList(), IndexedDUContext(context));
-  //if( != 0 )
-    //DUChain::contextChanged(m_context, DUChainObserver::Removal, DUChainObserver::ImportedChildContexts, context);
+  DUContext::Import import(m_context, context);
+  
+  if(import.isDirect()) {
+    removeOne(m_context->d_func_dynamic()->m_importersList(), IndexedDUContext(context));
+  }else{
+    //Indirect importers are registered separately
+    Importers::self().removeImporter(import.indirectDeclarationId(), IndexedDUContext(context));
+  }
 }
 
 int DUContext::depth() const
@@ -562,17 +594,19 @@ DUContext::~DUContext( )
     QualifiedIdentifier id(scopeIdentifier(true));
     if(d->m_inSymbolTable && parentContext()) {
       PersistentSymbolTable::self().removeContext(id, this);
-
-      //We put namespaces into the code-model
-      if(d->m_contextType == DUContext::Namespace)
-        CodeModel::self().removeItem(url(), id);
     }
-    
+
     if(d->m_owner.declaration())
       d->m_owner.declaration()->setInternalContext(0);
 
-    while( d->m_importersSize() != 0 )
-      d->m_importers()[0].data()->removeImportedParentContext(this);
+    while( d->m_importersSize() != 0 ) {
+      if(d->m_importers()[0].data())
+        d->m_importers()[0].data()->removeImportedParentContext(this);
+      else {
+        kDebug() << "importer disappeared";
+        d->m_importersList().removeOne(d->m_importers()[0]);
+      }
+    }
 
     clearImportedParentContexts();
   }
@@ -593,9 +627,9 @@ DUContext::~DUContext( )
       m_dynamicData->m_parentContext->m_dynamicData->removeChildContext(this);
     //DUChain::contextChanged(this, DUChainObserver::Deletion, DUChainObserver::NotApplicable);
   }
-  
+
   top->m_dynamicData->clearContextIndex(this);
-  
+
   Q_ASSERT(d_func()->isDynamic() == (!top->deleting() || !top->isOnDisk() || top->m_dynamicData->isTemporaryContextIndex(m_dynamicData->m_indexInTopContext)));
 }
 
@@ -645,7 +679,11 @@ void DUContext::setPropagateDeclarations(bool propagate)
 {
   ENSURE_CAN_WRITE
   DUCHAIN_D_DYNAMIC(DUContext);
+  
   QMutexLocker lock(&DUContextDynamicData::m_localDeclarationsMutex);
+  
+  if(propagate == d->m_propagateDeclarations)
+    return;
 
   m_dynamicData->m_parentContext->m_dynamicData->disableLocalDeclarationsHash();
 
@@ -672,7 +710,7 @@ QList<Declaration*> DUContext::findLocalDeclarations( const Identifier& identifi
 bool contextIsChildOrEqual(const DUContext* childContext, const DUContext* context) {
   if(childContext == context)
     return true;
-  
+
   if(childContext->parentContext())
     return contextIsChildOrEqual(childContext->parentContext(), context);
   else
@@ -702,7 +740,7 @@ void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, con
             }
           }
 
-          if( declaration->kind() == Declaration::NamespaceAlias )
+          if( declaration->kind() == Declaration::NamespaceAlias && !(m_flags & NoFiltering) )
             return 0;
 
           if((m_flags & OnlyFunctions) && !declaration->isFunctionDeclaration())
@@ -743,9 +781,9 @@ void DUContext::findLocalDeclarationsInternal( const Identifier& identifier, con
     }else if(d_func()->m_inSymbolTable && !this->localScopeIdentifier().isEmpty() && !identifier.isEmpty()) {
        //This context is in the symbol table, use the symbol-table to speed up the search
        QualifiedIdentifier id(scopeIdentifier(true) + identifier);
-       
+
        TopDUContext* top = topContext();
-       
+
        uint count;
        const IndexedDeclaration* declarations;
        PersistentSymbolTable::self().declarations(id, count, declarations);
@@ -798,10 +836,7 @@ bool DUContext::findDeclarationsInternal( const SearchItem::PtrList & baseIdenti
   ///Step 1: Apply namespace-aliases and -imports
   SearchItem::PtrList aliasedIdentifiers;
   //Because of namespace-imports and aliases, this identifier may need to be searched under multiple names
-  if( d_func()->m_contextType == Namespace )
-    applyAliases(baseIdentifiers, aliasedIdentifiers, position, false);
-  else
-    aliasedIdentifiers = baseIdentifiers;
+  applyAliases(baseIdentifiers, aliasedIdentifiers, position, false,  type() != DUContext::Namespace && type() != DUContext::Global);
 
 
   if( d->m_importedContextsSize() != 0 ) {
@@ -817,8 +852,9 @@ bool DUContext::findDeclarationsInternal( const SearchItem::PtrList & baseIdenti
 
         while( !context && import > 0 ) {
           --import;
+          context = d->m_importedContexts()[import].context(source);
         }
-        
+
         if(context == this) {
           kDebug() << "resolved self as import:" << scopeIdentifier(true);
           continue;
@@ -864,39 +900,44 @@ bool DUContext::imports(const DUContext* origin, const SimpleCursor& /*position*
 {
   ENSURE_CAN_READ
 
-  return origin->m_dynamicData->isThisImportedBy(this);
+  return m_dynamicData->imports(origin, topContext(), 4);
 }
 
-
-void DUContext::addImportedParentContext( DUContext * context, const SimpleCursor& position, bool anonymous, bool /*temporary*/ )
-{
+bool DUContext::addIndirectImport(const DUContext::Import& import) {
   ENSURE_CAN_WRITE
   DUCHAIN_D_DYNAMIC(DUContext);
 
-  if(context == this) {
-    kDebug(9007) << "Tried to import self";
-    return;
-  }
-  
-  Import import(context, position);
-
   for(unsigned int a = 0; a < d->m_importedContextsSize(); ++a) {
     if(d->m_importedContexts()[a] == import) {
-      d->m_importedContextsList()[a].position = position;
-      return;
+      d->m_importedContextsList()[a].position = import.position;
+      return true;
     }
   }
 
-  if( !anonymous && import.isBackwardMapped() ) {
-    ENSURE_CAN_WRITE_(context)
-    context->m_dynamicData->addImportedChildContext(this);
-  }
-
   ///Do not sort the imported contexts by their own line-number, it makes no sense.
   ///Contexts added first, aka template-contexts, should stay in first place, so they are searched first.
 
   d->m_importedContextsList().append(import);
+  return false;
+}
 
+void DUContext::addImportedParentContext( DUContext * context, const SimpleCursor& position, bool anonymous, bool /*temporary*/ )
+{
+  ENSURE_CAN_WRITE
+
+  if(context == this) {
+    kDebug() << "Tried to import self";
+    return;
+  }
+
+  Import import(context, this, position);
+  if(addIndirectImport(import))
+    return;
+
+  if( !anonymous ) {
+    ENSURE_CAN_WRITE_(context)
+    context->m_dynamicData->addImportedChildContext(this);
+  }
   //DUChain::contextChanged(this, DUChainObserver::Addition, DUChainObserver::ImportedParentContexts, context);
 }
 
@@ -904,8 +945,8 @@ void DUContext::removeImportedParentContext( DUContext * context )
 {
   ENSURE_CAN_WRITE
   DUCHAIN_D_DYNAMIC(DUContext);
-  
-  Import import(context, SimpleCursor::invalid());
+
+  Import import(context, this, SimpleCursor::invalid());
 
   for(unsigned int a = 0; a < d->m_importedContextsSize(); ++a) {
     if(d->m_importedContexts()[a] == import) {
@@ -922,14 +963,16 @@ void DUContext::removeImportedParentContext( DUContext * context )
   //DUChain::contextChanged(this, DUChainObserver::Removal, DUChainObserver::ImportedParentContexts, context);
 }
 
-const IndexedDUContext* DUContext::indexedImporters() const
+KDevVarLengthArray<IndexedDUContext> DUContext::indexedImporters() const
 {
-  return d_func()->m_importers();
-}
-
-uint DUContext::indexedImportersSize() const
-{
-  return d_func()->m_importersSize();
+  KDevVarLengthArray<IndexedDUContext> ret;
+  if(owner())
+    ret = Importers::self().importers(owner()->id()); //Add indirect importers to the list
+  
+  FOREACH_FUNCTION(IndexedDUContext ctx, d_func()->m_importers)
+    ret.append(ctx);
+   
+  return ret;
 }
 
 QVector<DUContext*> DUContext::importers() const
@@ -939,6 +982,14 @@ QVector<DUContext*> DUContext::importers() const
   QVector<DUContext*> ret;
   FOREACH_FUNCTION(IndexedDUContext ctx, d_func()->m_importers)
     ret << ctx.context();
+  
+  if(owner()) {
+    //Add indirect importers to the list
+    KDevVarLengthArray<IndexedDUContext> indirect = Importers::self().importers(owner()->id());
+    FOREACH_ARRAY(IndexedDUContext ctx, indirect) {
+      ret << ctx.context();
+    }
+  }
 
   return ret;
 }
@@ -978,7 +1029,7 @@ bool DUContext::parentContextOf(DUContext* context) const
 QList<Declaration*> DUContext::allLocalDeclarations(const Identifier& identifier) const
 {
   IndexedIdentifier indexedIdentifier(identifier);
-  
+
   ENSURE_CAN_READ
   QMutexLocker lock(&DUContextDynamicData::m_localDeclarationsMutex);
 
@@ -1015,8 +1066,9 @@ QList< QPair<Declaration*, int> > DUContext::allDeclarations(const SimpleCursor&
   return ret;
 }
 
-QVector<Declaration*> DUContext::localDeclarations() const
+QVector<Declaration*> DUContext::localDeclarations(const TopDUContext* source) const
 {
+  Q_UNUSED(source);
   ENSURE_CAN_READ
 
   QMutexLocker lock(&DUContextDynamicData::m_localDeclarationsMutex);
@@ -1032,51 +1084,60 @@ QVector<Declaration*> DUContext::localDeclarations() const
 
 void DUContext::mergeDeclarationsInternal(QList< QPair<Declaration*, int> >& definitions, const SimpleCursor& position, QHash<const DUContext*, bool>& hadContexts, const TopDUContext* source, bool searchInParents, int currentDepth) const
 {
-  DUCHAIN_D(DUContext);
-  if(hadContexts.contains(this))
+  if((currentDepth > 300 && currentDepth < 1000) || currentDepth > 1300) {
+    kDebug() << "too much depth";
     return;
-  hadContexts[this] = true;
-
-  if( (type() == DUContext::Namespace || type() == DUContext::Global) && currentDepth < 1000 )
-    currentDepth += 1000;
-
-  {
-    QMutexLocker lock(&DUContextDynamicData::m_localDeclarationsMutex);
-    DUContextDynamicData::VisibleDeclarationIterator it(m_dynamicData);
-    while(it) {
-      Declaration* decl = *it;
-        
-      if ( decl && (!position.isValid() || decl->range().start <= position) )
-        definitions << qMakePair(decl, currentDepth);
-      ++it;
-    }
   }
+  DUCHAIN_D(DUContext);
+  
+    if(hadContexts.contains(this) && !searchInParents)
+      return;
+  
+    if(!hadContexts.contains(this)) {
+      hadContexts[this] = true;
 
-  for(int a = d->m_importedContextsSize()-1; a >= 0; --a) {
-    const Import* import(&d->m_importedContexts()[a]);
-    DUContext* context = import->context(source);
-    while( !context && a > 0 ) {
-      --a;
-      import = &d->m_importedContexts()[a];
-      context = import->context(source);
-    }
-    if( !context )
-      break;
-    
-    if(context == this) {
-      kDebug() << "resolved self as import:" << scopeIdentifier(true);
-      continue;
+      if( (type() == DUContext::Namespace || type() == DUContext::Global) && currentDepth < 1000 )
+        currentDepth += 1000;
+
+      {
+        QMutexLocker lock(&DUContextDynamicData::m_localDeclarationsMutex);
+        DUContextDynamicData::VisibleDeclarationIterator it(m_dynamicData);
+        while(it) {
+          Declaration* decl = *it;
+
+          if ( decl && (!position.isValid() || decl->range().start <= position) )
+            definitions << qMakePair(decl, currentDepth);
+          ++it;
+        }
+      }
+
+      for(int a = d->m_importedContextsSize()-1; a >= 0; --a) {
+        const Import* import(&d->m_importedContexts()[a]);
+        DUContext* context = import->context(source);
+        while( !context && a > 0 ) {
+          --a;
+          import = &d->m_importedContexts()[a];
+          context = import->context(source);
+        }
+        if( !context )
+          break;
+
+        if(context == this) {
+          kDebug() << "resolved self as import:" << scopeIdentifier(true);
+          continue;
+        }
+
+
+        if( position.isValid() && import->position.isValid() && position < import->position )
+          continue;
+
+        context->mergeDeclarationsInternal(definitions, SimpleCursor::invalid(), hadContexts, source, searchInParents && context->shouldSearchInParent(InImportedParentContext) &&  context->parentContext()->type() == DUContext::Helper, currentDepth+1);
+      }
     }
     
-
-    if( position.isValid() && import->position.isValid() && position < import->position )
-      continue;
-
-    context->mergeDeclarationsInternal(definitions, SimpleCursor::invalid(), hadContexts, source, false, currentDepth+1);
-  }
-
-  if (searchInParents && parentContext())                            ///Only respect the position if the parent-context is not a class(@todo this is language-dependent)
-    parentContext()->mergeDeclarationsInternal(definitions, parentContext()->type() == DUContext::Class ? parentContext()->range().end : position, hadContexts, source, true, currentDepth+1);
+  ///Only respect the position if the parent-context is not a class(@todo this is language-dependent)
+  if (parentContext() && searchInParents )
+    parentContext()->mergeDeclarationsInternal(definitions, parentContext()->type() == DUContext::Class ? parentContext()->range().end : position, hadContexts, source, searchInParents, currentDepth+1);
 }
 
 void DUContext::deleteLocalDeclarations()
@@ -1102,7 +1163,7 @@ void DUContext::deleteChildContextsRecursively()
   ENSURE_CAN_WRITE
 
   TopDUContext* top = topContext();
-  
+
   QVector<LocalIndexedDUContext> children;
   FOREACH_FUNCTION(LocalIndexedDUContext ctx, d_func()->m_childContexts)
     children << ctx;
@@ -1186,24 +1247,8 @@ void DUContext::setType(ContextType type)
 {
   ENSURE_CAN_WRITE
 
-  bool wasNamespaceInCodeModel = d_func()->m_inSymbolTable && d_func()->m_contextType == DUContext::Namespace && d_func()->m_scopeIdentifier.isValid();
-
   d_func_dynamic()->m_contextType = type;
 
-  bool shouldBeNamespaceInCodeModel = d_func()->m_inSymbolTable && d_func()->m_contextType == DUContext::Namespace && d_func()->m_scopeIdentifier.isValid();
-  
-  if(wasNamespaceInCodeModel && shouldBeNamespaceInCodeModel) {
-    //We're fine
-  }else if(!wasNamespaceInCodeModel && shouldBeNamespaceInCodeModel) {
-    //Add
-    QualifiedIdentifier id(scopeIdentifier(true));
-    CodeModel::self().addItem(url(), id, CodeModelItem::Namespace);
-  }else if(wasNamespaceInCodeModel && !shouldBeNamespaceInCodeModel) {
-    //Remove
-    QualifiedIdentifier id(scopeIdentifier(true));
-    CodeModel::self().removeItem(url(), id);
-  }
-
   //DUChain::contextChanged(this, DUChainObserver::Change, DUChainObserver::ContextType);
 }
 
@@ -1242,11 +1287,11 @@ QVector<KTextEditor::SmartRange*> DUContext::takeUseRanges()
 {
   ENSURE_CAN_WRITE
   QVector<KTextEditor::SmartRange*> ret = m_dynamicData->m_rangesForUses;
-  
+
   foreach(KTextEditor::SmartRange* range, ret)
     if(range)
       range->removeWatcher(this);
-    
+
   m_dynamicData->m_rangesForUses.clear();
   return ret;
 }
@@ -1260,7 +1305,7 @@ QVector<KTextEditor::SmartRange*> DUContext::useRanges()
 void DUContext::deleteUses()
 {
   ENSURE_CAN_WRITE
-  
+
   DUCHAIN_D_DYNAMIC(DUContext);
   d->m_usesList().clear();
 
@@ -1268,14 +1313,14 @@ void DUContext::deleteUses()
 }
 
 bool DUContext::inDUChain() const {
-  if( d_func()->m_anonymousInParent )
+  if( d_func()->m_anonymousInParent || !m_dynamicData->m_parentContext)
     return false;
 
   TopDUContext* top = topContext();
-  return top && top->inDuChain();
+  return top && top->inDUChain();
 }
 
-DUContext* DUContext::specialize(uint /*specialization*/, const TopDUContext* /*topContext*/, int /*upDistance*/) {
+DUContext* DUContext::specialize(IndexedInstantiationInformation /*specialization*/, const TopDUContext* /*topContext*/, int /*upDistance*/) {
   return this;
 }
 
@@ -1283,7 +1328,7 @@ SimpleCursor DUContext::importPosition(const DUContext* target) const
 {
   ENSURE_CAN_READ
   DUCHAIN_D(DUContext);
-  Import import(const_cast<DUContext*>(target), SimpleCursor::invalid());
+  Import import(const_cast<DUContext*>(target), this, SimpleCursor::invalid());
   for(unsigned int a = 0; a < d->m_importedContextsSize(); ++a)
     if(d->m_importedContexts()[a] == import)
       return d->m_importedContexts()[a].position;
@@ -1311,9 +1356,13 @@ QList<DUContext*> DUContext::findContexts(ContextType contextType, const Qualifi
   return ret;
 }
 
-void DUContext::applyAliases(const SearchItem::PtrList& baseIdentifiers, SearchItem::PtrList& identifiers, const SimpleCursor& position, bool canBeNamespace) const {
+void DUContext::applyAliases(const SearchItem::PtrList& baseIdentifiers, SearchItem::PtrList& identifiers, const SimpleCursor& position, bool canBeNamespace, bool onlyImports) const {
 
   QList<Declaration*> imports = allLocalDeclarations(globalImportIdentifier);
+  if(imports.isEmpty() && onlyImports) {
+    identifiers = baseIdentifiers;
+    return;
+  }
 
   FOREACH_ARRAY( const SearchItem::Ptr& identifier, baseIdentifiers ) {
     bool addUnmodified = true;
@@ -1401,7 +1450,7 @@ void DUContext::findContextsInternal(ContextType contextType, const SearchItem::
   ///Step 1: Apply namespace-aliases and -imports
   SearchItem::PtrList aliasedIdentifiers;
   //Because of namespace-imports and aliases, this identifier may need to be searched as under multiple names
-  applyAliases(baseIdentifiers, aliasedIdentifiers, position, contextType == Namespace);
+  applyAliases(baseIdentifiers, aliasedIdentifiers, position, contextType == Namespace, contextType != Namespace);
 
   if( d->m_importedContextsSize() != 0 ) {
     ///Step 2: Give identifiers that are not marked as explicitly-global to imported contexts(explicitly global ones are treatead in TopDUContext)
@@ -1419,12 +1468,12 @@ void DUContext::findContextsInternal(ContextType contextType, const SearchItem::
           --a;
           context = d->m_importedContexts()[a].context(source);
         }
-        
+
         if(context == this) {
           kDebug() << "resolved self as import:" << scopeIdentifier(true);
           continue;
         }
-        
+
         if( !context )
           break;
 
@@ -1553,15 +1602,19 @@ void DUContext::setUseDeclaration(int useNumber, int declarationIndex)
 }
 
 
-DUContext * DUContext::findContextAt(const SimpleCursor & position) const
+DUContext * DUContext::findContextAt(const SimpleCursor & position, bool includeRightBorder) const
 {
   ENSURE_CAN_READ
+  
+//   kDebug() << "searchign" << position.textCursor() << "in:" << scopeIdentifier(true).toString() << range().textRange() << includeRightBorder;
 
-  if (!range().contains(position))
+  if (!range().contains(position) && (!includeRightBorder || range().end != position)) {
+//     kDebug() << "mismatch";
     return 0;
+  }
 
-  FOREACH_FUNCTION(LocalIndexedDUContext child, d_func()->m_childContexts)
-    if (DUContext* specific = child.data(topContext())->findContextAt(position))
+  for(int a = int(d_func()->m_childContextsSize())-1; a >= 0; --a)
+    if (DUContext* specific = d_func()->m_childContexts()[a].data(topContext())->findContextAt(position, includeRightBorder))
       return specific;
 
   return const_cast<DUContext*>(this);
@@ -1623,14 +1676,9 @@ void DUContext::setInSymbolTable(bool inSymbolTable)
         QualifiedIdentifier id(scopeIdentifier(true));
         PersistentSymbolTable::self().addContext(id, this);
 
-        if(d_func()->m_contextType == DUContext::Namespace)
-          CodeModel::self().addItem(url(), id, CodeModelItem::Namespace);
       }else if(d_func()->m_inSymbolTable && !inSymbolTable) {
         QualifiedIdentifier id(scopeIdentifier(true));
         PersistentSymbolTable::self().removeContext(id, this);
-        
-        if(d_func()->m_contextType == DUContext::Namespace)
-          CodeModel::self().removeItem(url(), id);
       }
     }
 
@@ -1645,12 +1693,10 @@ void DUContext::clearImportedParentContexts()
   DUCHAIN_D_DYNAMIC(DUContext);
 
   while( d->m_importedContextsSize() != 0 ) {
-    if( d->m_importedContexts()[0].isBackwardMapped() ) {
-      DUContext* ctx = d->m_importedContexts()[0].context(0);
-      if(ctx)
-        ctx->m_dynamicData->removeImportedChildContext(this);
-    }
-    
+    DUContext* ctx = d->m_importedContexts()[0].context(0);
+    if(ctx)
+      ctx->m_dynamicData->removeImportedChildContext(this);
+
     d->m_importedContextsList().removeOne(d->m_importedContexts()[0]);
   }
 }
@@ -1662,8 +1708,11 @@ void DUContext::cleanIfNotEncountered(const QSet<DUChainBase*>& encountered)
   foreach (Declaration* dec, localDeclarations())
     if (!encountered.contains(dec))
       delete dec;
-
-  FOREACH_FUNCTION(LocalIndexedDUContext childContext, d_func()->m_childContexts)
+    
+  //Copy since the array may change during the iteration
+  KDevVarLengthArray<LocalIndexedDUContext, 10> childrenCopy = d_func_dynamic()->m_childContextsList();
+  
+  FOREACH_ARRAY(LocalIndexedDUContext childContext, childrenCopy)
     if (!encountered.contains(childContext.data(topContext())))
       delete childContext.data(topContext());
 }
@@ -1832,20 +1881,23 @@ void DUContext::SearchItem::addToEachNode(SearchItem::PtrList other) {
     next[a]->addToEachNode(other);
 }
 
-DUContext::Import::Import(DUContext* _context, const SimpleCursor& _position) : position(_position) {
-  if(_context && _context->owner() && _context->owner()->specialization()) {
+DUContext::Import::Import(DUContext* _context, const DUContext* importer, const SimpleCursor& _position) : position(_position) {
+  if(_context && _context->owner() && (_context->owner()->specialization().index() || (importer && importer->topContext() != _context->topContext()))) {
     m_declaration = _context->owner()->id(true);
   }else{
     m_context = _context;
   }
 }
 
+DUContext::Import::Import(const DeclarationId& id, const SimpleCursor& _position) : position(_position) {
+  m_declaration = id;
+}
+
 DUContext* DUContext::Import::context(const TopDUContext* topContext) const {
   if(m_declaration.isValid()) {
-    Q_ASSERT(m_declaration.isDirect());
     Declaration* decl = m_declaration.getDeclaration(topContext);
     if(decl)
-      return decl->internalContext();
+      return decl->logicalInternalContext(topContext);
     else
       return 0;
   }else{
@@ -1853,12 +1905,8 @@ DUContext* DUContext::Import::context(const TopDUContext* topContext) const {
   }
 }
 
-bool DUContext::Import::isBackwardMapped() const {
-  return true; //Always do backward-mapping. We just have to find a solution for temporary imports.
-/*  if(m_declaration.isValid())
-    return !m_declaration.specialization();
-  else
-    return true;*/
+bool DUContext::Import::isDirect() const {
+  return m_context.isValid();
 }
 
 }
