@@ -1,5 +1,6 @@
 /* This file is part of KDevelop
     Copyright 2006 Hamish Rodda <rodda@kde.org>
+    Copyright 2007-2008 David Nolden <david.nolden.kdevelop@art-master.de>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,19 +19,30 @@
 
 #include "identifier.h"
 
-#include <QHash>
+#include <QtCore/QHash>
 #include "stringhelpers.h"
 #include "indexedstring.h"
 #include "appendedlist_static.h"
 #include "repositories/itemrepository.h"
 
+#define ifDebug(x)
+
 namespace KDevelop
 {
 template<bool dynamic = false>
 class IdentifierPrivate
 {
 public:
-  IdentifierPrivate() : m_unique(0), m_hash(0){
+  IdentifierPrivate() : m_unique(0), m_refCount(0), m_hash(0) {
+  }
+  
+  template<bool rhsDynamic>
+  IdentifierPrivate(const IdentifierPrivate<rhsDynamic>& rhs) : m_unique(rhs.m_unique), m_identifier(rhs.m_identifier), m_refCount(0), m_hash(rhs.m_hash) {
+    copyListsFrom(rhs);
+  }
+  
+  ~IdentifierPrivate() {
+    templateIdentifiersList.free(const_cast<IndexedTypeIdentifier*>(templateIdentifiers()));
   }
 
   //Flags the stored hash-value invalid
@@ -47,10 +59,11 @@ public:
 
   int m_unique;
   IndexedString m_identifier;
+  uint m_refCount;
 
   START_APPENDED_LISTS(IdentifierPrivate)
 
-  APPENDED_LIST_FIRST(uint, templateIdentifiers)
+  APPENDED_LIST_FIRST(IndexedTypeIdentifier, templateIdentifiers)
 
   END_APPENDED_LISTS(templateIdentifiers)
 
@@ -63,8 +76,8 @@ public:
       //this must stay thread-safe(may be called by multiple threads at a time)
       //The thread-safety is given because all threads will have the same result, and it will only be written once at the end.
       uint hash = m_identifier.hash();
-      FOREACH_FUNCTION(uint templateIdentifier, templateIdentifiers)
-        hash = hash * 13 + TypeIdentifier(templateIdentifier).hash();
+      FOREACH_FUNCTION(IndexedTypeIdentifier templateIdentifier, templateIdentifiers)
+        hash = hash * 13 + IndexedTypeIdentifier(templateIdentifier).hash();
       hash += m_unique;
       m_hash = hash;
     }
@@ -96,10 +109,15 @@ struct IdentifierItemRequest {
   //Should create an item where the information of the requested item is permanently stored. The pointer
   //@param item equals an allocated range with the size of itemSize().
   void createItem(ConstantIdentifierPrivate* item) const {
-    item->m_hash = m_identifier.m_hash;
-    item->m_unique = m_identifier.m_unique;
-    item->m_identifier = m_identifier.m_identifier;
-    item->copyListsFrom(m_identifier);
+    new (item) ConstantIdentifierPrivate(m_identifier);
+  }
+  
+  static bool persistent(const ConstantIdentifierPrivate* item) {
+    return (bool)item->m_refCount;
+  }
+  
+  static void destroy(ConstantIdentifierPrivate* item, KDevelop::AbstractItemRepository&) {
+    item->~ConstantIdentifierPrivate();
   }
 
   //Should return whether the here requested item equals the given item
@@ -115,6 +133,10 @@ RepositoryManager< ItemRepository<ConstantIdentifierPrivate, IdentifierItemReque
 ///This has to be initialized now, else we will get a crash when multiple threads try accessing it for the first time in the same moment
 uint emptyConstantIdentifierPrivateIndex() {
   static uint index = identifierRepository->index(DynamicIdentifierPrivate());
+  
+  if(index == 0) ///Just so the function is instantiated right now
+    identifierRepository->deleteItem(0);
+  
   return index;
 }
 
@@ -132,19 +154,31 @@ template<bool dynamic>
 class QualifiedIdentifierPrivate
 {
 public:
-  QualifiedIdentifierPrivate() : m_explicitlyGlobal(false), m_isExpression(false), m_isConstant(false), m_isReference(false), m_pointerDepth(0), m_pointerConstantMask(0), m_hash(0) {
+  QualifiedIdentifierPrivate() : m_explicitlyGlobal(false), m_isExpression(false), m_hash(0), m_refCount(0) {
   }
+  
+  template<bool rhsDynamic>
+  QualifiedIdentifierPrivate(const QualifiedIdentifierPrivate<rhsDynamic>& rhs)
+    : m_explicitlyGlobal(rhs.m_explicitlyGlobal)
+    , m_isExpression(rhs.m_isExpression)
+    , m_hash(rhs.m_hash)
+    , m_refCount(0)
+  {
+    copyListsFrom(rhs);
+  }
+  
+  ~QualifiedIdentifierPrivate() {
+    identifiersList.free(const_cast<IndexedIdentifier*>(identifiers()));
+  }
+  
   bool m_explicitlyGlobal:1;
   bool m_isExpression:1;
-  bool m_isConstant:1; //Data for TypeIdentifier, stored here.
-  bool m_isReference:1; //Data for TypeIdentifier, stored here.
-  uchar m_pointerDepth;
-  uint m_pointerConstantMask; //Stores in a mask for each pointer-depth whether it is constant. Supports only max. 32 levels.
   mutable uint m_hash;
+  uint m_refCount;
 
   START_APPENDED_LISTS(QualifiedIdentifierPrivate)
 
-  APPENDED_LIST_FIRST(uint, identifiers)
+  APPENDED_LIST_FIRST(IndexedIdentifier, identifiers)
 
   END_APPENDED_LISTS(identifiers)
 
@@ -159,7 +193,7 @@ public:
     uint currentStart = start;
 
     while( currentStart < (uint)str.length() ) {
-      identifiersList.append(Identifier( str, currentStart, &currentStart ).index());
+      identifiersList.append(IndexedIdentifier(Identifier( str, currentStart, &currentStart )));
       while( currentStart < (uint)str.length() && (str[currentStart] == ' ' ) )
         ++currentStart;
       currentStart += 2; //Skip "::"
@@ -175,12 +209,11 @@ public:
     if( m_hash == 0 )
     {
       uint mhash = 0;
-      FOREACH_FUNCTION( uint identifier, identifiers )
+      FOREACH_FUNCTION( IndexedIdentifier identifier, identifiers )
         mhash = 11*mhash + Identifier(identifier).hash();
 
-      mhash += 17 * m_isConstant + 19 * m_isReference + 23 * m_pointerDepth + 31 * m_pointerConstantMask;
-
-      m_hash = mhash;
+      if(mhash != m_hash)
+        m_hash = mhash;//The local class may be in read-only memory, so only  change m_hash if it's actually a change
     }
     return m_hash;
   }
@@ -210,24 +243,24 @@ struct QualifiedIdentifierItemRequest {
   //Should create an item where the information of the requested item is permanently stored. The pointer
   //@param item equals an allocated range with the size of itemSize().
   void createItem(ConstantQualifiedIdentifierPrivate* item) const {
-    item->m_explicitlyGlobal = m_identifier.m_explicitlyGlobal;
-    item->m_isExpression = m_identifier.m_isExpression;
-    item->m_isConstant = m_identifier.m_isConstant;
-    item->m_isReference = m_identifier.m_isReference;
-    item->m_pointerDepth = m_identifier.m_pointerDepth;
-    item->m_pointerConstantMask = m_identifier.m_pointerConstantMask;
-    item->m_hash = m_identifier.m_hash;
-    item->copyListsFrom(m_identifier);
+    Q_ASSERT(shouldDoDUChainReferenceCounting(item));
+    Q_ASSERT(shouldDoDUChainReferenceCounting(((char*)item) + (itemSize()-1)));
+    new (item) ConstantQualifiedIdentifierPrivate(m_identifier);
+  }
+  
+  static bool persistent(const ConstantQualifiedIdentifierPrivate* item) {
+    return (bool)item->m_refCount;
+  }
+  
+  static void destroy(ConstantQualifiedIdentifierPrivate* item, KDevelop::AbstractItemRepository&) {
+    Q_ASSERT(shouldDoDUChainReferenceCounting(item));
+    item->~ConstantQualifiedIdentifierPrivate();
   }
 
   //Should return whether the here requested item equals the given item
   bool equals(const ConstantQualifiedIdentifierPrivate* item) const {
     return item->m_explicitlyGlobal == m_identifier.m_explicitlyGlobal &&
     item->m_isExpression == m_identifier.m_isExpression &&
-    item->m_isConstant == m_identifier.m_isConstant &&
-    item->m_isReference == m_identifier.m_isReference &&
-    item->m_pointerDepth == m_identifier.m_pointerDepth &&
-    item->m_pointerConstantMask == m_identifier.m_pointerConstantMask &&
     item->m_hash == m_identifier.m_hash &&
     m_identifier.listsEqual(*item);
   }
@@ -239,6 +272,8 @@ RepositoryManager< ItemRepository<ConstantQualifiedIdentifierPrivate, QualifiedI
 
 uint emptyConstantQualifiedIdentifierPrivateIndex() {
    static uint index = qualifiedidentifierRepository->index(DynamicQualifiedIdentifierPrivate());
+    if(index == 0) ///Just so the function is instantiated right now
+      identifierRepository->deleteItem(0);
    return index;
 }
 
@@ -246,9 +281,9 @@ const ConstantQualifiedIdentifierPrivate* emptyConstantQualifiedIdentifierPrivat
   return qualifiedidentifierRepository->itemFromIndex(emptyConstantQualifiedIdentifierPrivateIndex());
 }
 
-uint QualifiedIdentifier::combineHash(uint leftHash, uint /*leftSize*/, Identifier appendIdentifier) {
-  return 11*leftHash + appendIdentifier.hash();
-}
+// uint QualifiedIdentifier::combineHash(uint leftHash, uint /*leftSize*/, Identifier appendIdentifier) {
+//   return 11*leftHash + appendIdentifier.hash();
+// }
 
 Identifier::Identifier(const Identifier& rhs) {
   rhs.makeConstant();
@@ -298,7 +333,7 @@ Identifier::Identifier(const QString& id, uint start, uint* takenRange)
   ParamIterator paramIt("<>:", id, start);
   dd->m_identifier = IndexedString(paramIt.prefix().trimmed());
   while( paramIt ) {
-    appendTemplateIdentifier( TypeIdentifier(*paramIt) );
+    appendTemplateIdentifier( IndexedTypeIdentifier(IndexedQualifiedIdentifier(QualifiedIdentifier(*paramIt))) );
     ++paramIt;
   }
 
@@ -360,12 +395,12 @@ void Identifier::setIdentifier(const IndexedString& identifier)
   dd->m_identifier = identifier;
 }
 
-TypeIdentifier Identifier::templateIdentifier(int num) const
+IndexedTypeIdentifier Identifier::templateIdentifier(int num) const
 {
   if(!m_index)
-    return TypeIdentifier(dd->templateIdentifiers()[num]);
+    return dd->templateIdentifiers()[num];
   else
-    return TypeIdentifier(cd->templateIdentifiers()[num]);
+    return cd->templateIdentifiers()[num];
 }
 
 uint Identifier::templateIdentifiersCount() const
@@ -376,10 +411,10 @@ uint Identifier::templateIdentifiersCount() const
     return cd->templateIdentifiersSize();
 }
 
-void Identifier::appendTemplateIdentifier(const TypeIdentifier& identifier)
+void Identifier::appendTemplateIdentifier(const IndexedTypeIdentifier& identifier)
 {
   prepareWrite();
-  dd->templateIdentifiersList.append(identifier.index());
+  dd->templateIdentifiersList.append(identifier);
 }
 
 void Identifier::clearTemplateIdentifiers()
@@ -395,12 +430,12 @@ uint Identifier::index() const
   return m_index;
 }
 
-void Identifier::setTemplateIdentifiers(const QList<TypeIdentifier>& templateIdentifiers)
+void Identifier::setTemplateIdentifiers(const QList<IndexedTypeIdentifier>& templateIdentifiers)
 {
   prepareWrite();
   dd->templateIdentifiersList.clear();
-  foreach(const TypeIdentifier& id, templateIdentifiers)
-    dd->templateIdentifiersList.append(id.index());
+  foreach(const IndexedTypeIdentifier& id, templateIdentifiers)
+    dd->templateIdentifiersList.append(id);
 }
 
 QString Identifier::toString() const
@@ -482,10 +517,17 @@ void Identifier::prepareWrite() {
   dd->clearHash();
 }
 
+bool QualifiedIdentifier::inRepository() const {
+  if(m_index)
+    return true;
+  else
+    return (bool)qualifiedidentifierRepository->findIndex( QualifiedIdentifierItemRequest(*dd) );
+}
+
 QualifiedIdentifier::QualifiedIdentifier(uint index) : m_index(index), cd( qualifiedidentifierRepository->itemFromIndex(index) ) {
 }
 
-QualifiedIdentifier::QualifiedIdentifier(const QString& id)
+QualifiedIdentifier::QualifiedIdentifier(const QString& id, bool isExpression)
   : m_index(0), dd(new DynamicQualifiedIdentifierPrivate)
 {
   if (id.startsWith("::")) {
@@ -495,6 +537,9 @@ QualifiedIdentifier::QualifiedIdentifier(const QString& id)
     dd->m_explicitlyGlobal = false;
     dd->splitIdentifiers(id, 0);
   }
+  
+  if(isExpression)
+    setIsExpression(true);
 }
 
 QualifiedIdentifier::QualifiedIdentifier(const Identifier& id)
@@ -504,7 +549,7 @@ QualifiedIdentifier::QualifiedIdentifier(const Identifier& id)
     dd->m_explicitlyGlobal = true;
   } else {
     dd->m_explicitlyGlobal = false;
-    dd->identifiersList.append(id.index());
+    dd->identifiersList.append(IndexedIdentifier(id));
   }
 }
 
@@ -527,16 +572,8 @@ QualifiedIdentifier::QualifiedIdentifier(const QualifiedIdentifier& id)
     m_index = 0;
     dd = new QualifiedIdentifierPrivate<true>(*id.dd);
   }
-
-  if( cd->m_pointerConstantMask || cd->m_isReference || cd->m_pointerDepth || cd->m_isConstant ) {
-    //When copying from a type-identifier, do not share the d-pointer, because it contains invalid information.
-    prepareWrite();
-    dd->m_pointerConstantMask = 0;
-    dd->m_isReference = 0;
-    dd->m_pointerDepth = 0;
-    dd->m_isConstant = 0;
-  }
 }
+
 QStringList QualifiedIdentifier::toStringList() const
 {
   QStringList ret;
@@ -544,11 +581,11 @@ QStringList QualifiedIdentifier::toStringList() const
     ret.append(QString());
 
   if(m_index) {
-    FOREACH_FUNCTION(uint index, cd->identifiers)
-      ret << Identifier(index).toString();
+    FOREACH_FUNCTION(IndexedIdentifier index, cd->identifiers)
+      ret << index.identifier().toString();
   }else{
-    FOREACH_FUNCTION(uint index, dd->identifiers)
-      ret << Identifier(index).toString();
+    FOREACH_FUNCTION(IndexedIdentifier index, dd->identifiers)
+      ret << index.identifier().toString();
   }
 
   return ret;
@@ -562,24 +599,24 @@ QString QualifiedIdentifier::toString(bool ignoreExplicitlyGlobal) const
 
   bool first = true;
   if(m_index) {
-    FOREACH_FUNCTION(uint index, cd->identifiers)
+    FOREACH_FUNCTION(IndexedIdentifier index, cd->identifiers)
     {
       if( !first )
         ret += "::";
       else
         first = false;
 
-      ret += Identifier(index).toString();
+      ret += index.identifier().toString();
     }
   }else{
-    FOREACH_FUNCTION(uint index, dd->identifiers)
+    FOREACH_FUNCTION(IndexedIdentifier index, dd->identifiers)
     {
       if( !first )
         ret += "::";
       else
         first = false;
 
-      ret += Identifier(index).toString();
+      ret += index.identifier().toString();
     }
   }
 
@@ -686,22 +723,22 @@ bool QualifiedIdentifier::sameIdentifiers(const QualifiedIdentifier& rhs) const
     return dd->listsEqual(*rhs.cd);
 }
 
-bool QualifiedIdentifier::isSame(const QualifiedIdentifier& rhs, bool ignoreExplicitlyGlobal) const
-{
-  if( cd == rhs.cd )
-    return true;
-
-  if (!ignoreExplicitlyGlobal && (explicitlyGlobal() != rhs.explicitlyGlobal()))
-    return false;
-
-  if( isExpression() != rhs.isExpression() )
-    return false;
-
-  if( hash() != rhs.hash() )
-    return false;
-
-  return sameIdentifiers(rhs);
-}
+// bool QualifiedIdentifier::isSame(const QualifiedIdentifier& rhs, bool ignoreExplicitlyGlobal) const
+// {
+//   if( cd == rhs.cd )
+//     return true;
+// 
+//   if (!ignoreExplicitlyGlobal && (explicitlyGlobal() != rhs.explicitlyGlobal()))
+//     return false;
+// 
+//   if( isExpression() != rhs.isExpression() )
+//     return false;
+// 
+//   if( hash() != rhs.hash() )
+//     return false;
+// 
+//   return sameIdentifiers(rhs);
+// }
 
 bool QualifiedIdentifier::operator==(const QualifiedIdentifier& rhs) const
 {
@@ -725,57 +762,6 @@ QualifiedIdentifier& QualifiedIdentifier::operator=(const QualifiedIdentifier& r
   return *this;
 }
 
-#if 0
-
-QualifiedIdentifier::MatchTypes QualifiedIdentifier::match(const Identifier& rhs) const
-{
-  if( d->m_identifiers.isEmpty() )
-    return NoMatch;
-
-  if( d->m_identifiers[d->m_identifiers.size()-1] == rhs ) {
-    if( d->m_identifiers.size() == 1 )
-      return ExactMatch;
-    else
-      return EndsWith;
-  }
-
-  return NoMatch;
-}
-
-QualifiedIdentifier::MatchTypes QualifiedIdentifier::match(const QualifiedIdentifier& rhs) const
-{
-  const int difference = count() - rhs.count();
-  if (difference == 0) {
-    if (hash() == rhs.hash())
-      return ExactMatch;
-    else
-      return NoMatch;
-
-  } else if (difference > 0) {
-    // eg 6 vs 4... we need at(2)
-    if (rhs.explicitlyGlobal())
-      return NoMatch;
-
-    for( int a = difference; a < count(); a++ )
-      if( rhs.d->m_identifiers[a-difference] != d->m_identifiers[a] )
-        return NoMatch;
-
-    return EndsWith;
-  } else {
-    if (explicitlyGlobal())
-      return NoMatch;
-
-    for( int a = -difference; a < rhs.count(); a++ )
-        if( rhs.d->m_identifiers[a] != d->m_identifiers[a+difference] )
-          return NoMatch;
-
-    return TargetEndsWith;
-  }
-
-  return NoMatch;
-}
-#endif
-
 bool QualifiedIdentifier::beginsWith(const QualifiedIdentifier& other) const
 {
   uint c = count();
@@ -817,7 +803,7 @@ uint QualifiedIdentifier::hash() const {
     return dd->hash();
 }
 
-uint qHash(const TypeIdentifier& id)
+uint qHash(const IndexedTypeIdentifier& id)
 {
   return id.hash();
 }
@@ -837,48 +823,21 @@ bool QualifiedIdentifier::isQualified() const
   return count() > 1 || explicitlyGlobal();
 }
 
-// QString Identifier::mangled() const
-// { ///@todo work this over
-//   static QRegExp simpleIdentifier("[a-zA-Z0-9_]*");
-//   if (simpleIdentifier.exactMatch(d->m_identifier.str()))
-//     return QString("%1%2").arg(d->m_identifier.str().length()).arg(d->m_identifier.str());
-//
-//   // Get the encoded utf form
-//   QString utf = QString::fromLatin1(d->m_identifier.str().toUtf8());
-//
-//   return QString("U%1%2").arg(utf.length()).arg(utf);
-// }
-
-// QString QualifiedIdentifier::mangled() const
-// {
-//   if (isEmpty())
-//     return QString();
-//
-//   if (count() == 1)
-//     return first().mangled();
-//
-//   QString ret('Q');
-//
-//   if (count() > 9)
-//     ret += QString(",%1,").arg(count());
-//   else
-//     ret += count();
-//
-//   for (int i = 0; i < count(); ++i)
-//     ret += at(i).mangled();
-//
-//   return ret;
-// }
-
 void QualifiedIdentifier::push(const Identifier& id)
 {
+  if(id.isEmpty())
+    return;
+  
   prepareWrite();
 
-  dd->identifiersList.append(id.index());
+  dd->identifiersList.append(IndexedIdentifier(id));
 }
 
 void QualifiedIdentifier::push(const QualifiedIdentifier& id)
 {
+  if(id.isEmpty())
+    return;
+  
   prepareWrite();
   id.makeConstant();
 
@@ -888,6 +847,8 @@ void QualifiedIdentifier::push(const QualifiedIdentifier& id)
 void QualifiedIdentifier::pop()
 {
   prepareWrite();
+  if(!dd->identifiersSize())
+    return;
   dd->identifiersList.resize(dd->identifiersList.size()-1);
 }
 
@@ -918,7 +879,7 @@ int QualifiedIdentifier::count() const
 
 Identifier QualifiedIdentifier::first() const
 {
-  if( (m_index && cd->identifiersSize() == 0) || dd->identifiersSize() == 0 )
+  if( (m_index && cd->identifiersSize() == 0) || (!m_index && dd->identifiersSize() == 0) )
     return Identifier();
   else
     return at(0);
@@ -983,10 +944,6 @@ void QualifiedIdentifier::prepareWrite() {
     dd = new QualifiedIdentifierPrivate<true>;
     dd->m_explicitlyGlobal = oldCc->m_explicitlyGlobal;
     dd->m_isExpression = oldCc->m_isExpression;
-    dd->m_isConstant = oldCc->m_isConstant;
-    dd->m_isReference = oldCc->m_isReference;
-    dd->m_pointerDepth = oldCc->m_pointerDepth;
-    dd->m_pointerConstantMask = oldCc->m_pointerConstantMask;
     dd->m_hash = oldCc->m_hash;
 
     dd->copyListsFrom(*oldCc);
@@ -996,87 +953,65 @@ void QualifiedIdentifier::prepareWrite() {
   dd->clearHash();
 }
 
-bool TypeIdentifier::isSame(const TypeIdentifier& rhs, bool ignoreExplicitlyGlobal) const {
-  if(m_index)
-    return QualifiedIdentifier::isSame(rhs, ignoreExplicitlyGlobal) && cd->m_isConstant == rhs.cd->m_isConstant && cd->m_isReference == rhs.cd->m_isReference && cd->m_pointerConstantMask == rhs.cd->m_pointerConstantMask;
-  else
-    return QualifiedIdentifier::isSame(rhs, ignoreExplicitlyGlobal) && dd->m_isConstant == rhs.dd->m_isConstant && dd->m_isReference == rhs.dd->m_isReference && dd->m_pointerConstantMask == rhs.dd->m_pointerConstantMask;
+uint IndexedTypeIdentifier::hash() const {
+    return m_identifier.getIndex() * 13 + (m_isConstant ? 17 : 0) + (m_isReference ? 12371 : 0) + m_pointerConstMask * 89321 + m_pointerDepth * 1023;
 }
 
-bool TypeIdentifier::operator==(const TypeIdentifier& rhs) const {
-  if(m_index)
-    return QualifiedIdentifier::operator==(rhs) && cd->m_isConstant == rhs.cd->m_isConstant && cd->m_isReference == rhs.cd->m_isReference && cd->m_pointerConstantMask == rhs.cd->m_pointerConstantMask && cd->m_pointerDepth == rhs.cd->m_pointerDepth;
-  else
-    return QualifiedIdentifier::operator==(rhs) && dd->m_isConstant == rhs.dd->m_isConstant && dd->m_isReference == rhs.dd->m_isReference && dd->m_pointerConstantMask == rhs.dd->m_pointerConstantMask && dd->m_pointerDepth == rhs.dd->m_pointerDepth;
+bool IndexedTypeIdentifier::operator==(const IndexedTypeIdentifier& rhs) const {
+    return m_identifier == rhs.m_identifier && m_isConstant == rhs.m_isConstant && m_isReference == rhs.m_isReference && m_pointerConstMask == rhs.m_pointerConstMask && m_pointerDepth == rhs.m_pointerDepth;
 }
 
-bool TypeIdentifier::operator!=(const TypeIdentifier& rhs) const {
+bool IndexedTypeIdentifier::operator!=(const IndexedTypeIdentifier& rhs) const {
   return !operator==(rhs);
 }
 
-bool TypeIdentifier::isReference() const {
-  if(m_index)
-    return cd->m_isReference;
-  else
-    return dd->m_isReference;
+bool IndexedTypeIdentifier::isReference() const {
+  return m_isReference;
 }
 
-void TypeIdentifier::setIsReference(bool isRef) {
-  prepareWrite();
-  dd->m_isReference = isRef;
+void IndexedTypeIdentifier::setIsReference(bool isRef) {
+  m_isReference = isRef;
 }
 
-bool TypeIdentifier::isConstant() const {
-  if(m_index)
-    return cd->m_isConstant;
-  else
-    return dd->m_isConstant;
+bool IndexedTypeIdentifier::isConstant() const {
+  return m_isConstant;
 }
 
-void TypeIdentifier::setIsConstant(bool isConst) {
-  prepareWrite();
-  dd->m_isConstant = isConst;
+void IndexedTypeIdentifier::setIsConstant(bool isConst) {
+  m_isConstant = isConst;
 }
 
 ///Returns the pointer depth. Example for C++: "char*" has pointer-depth 1, "char***" has pointer-depth 3
-int TypeIdentifier::pointerDepth() const {
-  if(m_index)
-    return cd->m_pointerDepth;
-  else
-    return dd->m_pointerDepth;
+int IndexedTypeIdentifier::pointerDepth() const {
+  return m_pointerDepth;
 }
 
 /**Sets the pointer-depth to the specified count
   * For efficiency-reasons the maximum currently is 32. */
-void TypeIdentifier::setPointerDepth(int depth) {
-  prepareWrite();
+void IndexedTypeIdentifier::setPointerDepth(int depth) {
   ///Clear the mask in removed fields
-  for(int s = depth; s < (int)dd->m_pointerDepth; ++s)
+  for(int s = depth; s < (int)m_pointerDepth; ++s)
     setIsConstPointer(s, false);
 
-  dd->m_pointerDepth = depth;
+  m_pointerDepth = depth;
 }
 
-bool TypeIdentifier::isConstPointer(int depthNumber) const {
-  if(m_index)
-    return cd->m_pointerConstantMask & (1 << depthNumber);
-  else
-    return dd->m_pointerConstantMask & (1 << depthNumber);
+bool IndexedTypeIdentifier::isConstPointer(int depthNumber) const {
+  return m_pointerConstMask & (1 << depthNumber);
 }
 
-void TypeIdentifier::setIsConstPointer(int depthNumber, bool constant) {
-  prepareWrite();
+void IndexedTypeIdentifier::setIsConstPointer(int depthNumber, bool constant) {
   if(constant)
-    dd->m_pointerConstantMask |= (1 << depthNumber);
+    m_pointerConstMask |= (1 << depthNumber);
   else
-    dd->m_pointerConstantMask &= (~(1 << depthNumber));
+    m_pointerConstMask &= (~(1 << depthNumber));
 }
 
-QString TypeIdentifier::toString(bool ignoreExplicitlyGlobal) const {
+QString IndexedTypeIdentifier::toString(bool ignoreExplicitlyGlobal) const {
   QString ret;
   if(isConstant())
     ret += "const ";
-  ret += QualifiedIdentifier::toString(ignoreExplicitlyGlobal);
+  ret += m_identifier.identifier().toString(ignoreExplicitlyGlobal);
   for(int a = 0; a < pointerDepth(); ++a) {
     ret += '*';
     if( isConstPointer(a) )
@@ -1088,34 +1023,70 @@ QString TypeIdentifier::toString(bool ignoreExplicitlyGlobal) const {
   return ret;
 }
 
-TypeIdentifier::TypeIdentifier(uint index) : QualifiedIdentifier(index) {
+IndexedTypeIdentifier::IndexedTypeIdentifier(KDevelop::IndexedQualifiedIdentifier identifier) : m_identifier(identifier), m_isConstant(false), m_isReference(false), m_pointerDepth(0), m_pointerConstMask(0) {
 }
 
-TypeIdentifier::TypeIdentifier() : QualifiedIdentifier() {
-}
-
-TypeIdentifier::TypeIdentifier(const QString& str) : QualifiedIdentifier(str) {
-}
-
-TypeIdentifier::TypeIdentifier(const TypeIdentifier& id) : QualifiedIdentifier() {
-  id.makeConstant();
-  m_index = id.m_index;
-  cd = id.cd;
-}
-
-TypeIdentifier::TypeIdentifier(const QualifiedIdentifier& id) : QualifiedIdentifier(id) {
+IndexedTypeIdentifier::IndexedTypeIdentifier(const QString& identifier, bool isExpression) : m_identifier(QualifiedIdentifier(identifier, isExpression)), m_isConstant(false), m_isReference(false), m_pointerDepth(0), m_pointerConstMask(0) {
 }
 
 IndexedIdentifier::IndexedIdentifier() : index(emptyConstantIdentifierPrivateIndex()) {
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    increase(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
 }
 
 IndexedIdentifier::IndexedIdentifier(const Identifier& id) : index(id.index()) {
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    increase(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+}
+
+IndexedIdentifier::IndexedIdentifier(const IndexedIdentifier& rhs) : index(rhs.index) {
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    increase(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+}
+
+IndexedIdentifier::~IndexedIdentifier() {
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    decrease(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
 }
 
 IndexedIdentifier& IndexedIdentifier::operator=(const Identifier& id) {
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    decrease(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+
   index = id.index();
+
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    increase(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
   return *this;
 }
+
+IndexedIdentifier& IndexedIdentifier::operator=(const IndexedIdentifier& id) {
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    decrease(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+
+  index = id.index;
+
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(identifierRepository->mutex());
+    increase(identifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+  return *this;
+}
+
 bool IndexedIdentifier::operator==(const IndexedIdentifier& rhs) const {
   return index == rhs.index;
 }
@@ -1136,23 +1107,113 @@ IndexedIdentifier::operator Identifier() const {
   return Identifier(index);
 }
 
-IndexedQualifiedIdentifier::IndexedQualifiedIdentifier() : index(emptyConstantQualifiedIdentifierPrivateIndex()) {
-}
-
 bool IndexedQualifiedIdentifier::isValid() const {
   return index != emptyConstantQualifiedIdentifierPrivateIndex();
 }
 
+int cnt = 0;
+
+IndexedQualifiedIdentifier IndexedTypeIdentifier::identifier() const
+{
+  return m_identifier;
+}
+
+
+void IndexedTypeIdentifier::setIdentifier(KDevelop::IndexedQualifiedIdentifier id)
+{
+  m_identifier = id;
+}
+
+IndexedQualifiedIdentifier::IndexedQualifiedIdentifier() : index(emptyConstantQualifiedIdentifierPrivateIndex()) {
+  ifDebug( kDebug() << "(" << ++cnt << ")" << m_id << identifier().toString() << index; )
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << "increasing"; )
+    
+    //kDebug() << "(" << ++cnt << ")" << this << identifier().toString() << "inc" << index;
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    increase(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+}
+
 IndexedQualifiedIdentifier::IndexedQualifiedIdentifier(const QualifiedIdentifier& id) : index(id.index()) {
+  ifDebug( kDebug() << "(" << ++cnt << ")" << m_id << identifier().toString() << index; )
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << "increasing"; )
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    increase(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+}
+
+IndexedQualifiedIdentifier::IndexedQualifiedIdentifier(const IndexedQualifiedIdentifier& id) : index(id.index) {
+  ifDebug( kDebug() << "(" << ++cnt << ")" << m_id << identifier().toString() << index; )
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << "increasing"; )
+    
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    increase(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
 }
 
 IndexedQualifiedIdentifier& IndexedQualifiedIdentifier::operator=(const QualifiedIdentifier& id) {
+  ifDebug( kDebug() << "(" << ++cnt << ")" << m_id << identifier().toString() << index; )
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << "decreasing"; )
+    
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    decrease(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+  
   index = id.index();
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << index << "increasing"; )
+    
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    increase(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+  
   return *this;
 }
+
+IndexedQualifiedIdentifier& IndexedQualifiedIdentifier::operator=(const IndexedQualifiedIdentifier& rhs) {
+  
+  ifDebug( kDebug() << "(" << ++cnt << ")" << m_id << identifier().toString() << index; )
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    ifDebug( kDebug() << "decreasing"; )
+    
+    decrease(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+  
+  index = rhs.index;
+  
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << index << "increasing"; )
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    increase(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+  
+  return *this;
+}
+
+IndexedQualifiedIdentifier::~IndexedQualifiedIdentifier() {
+  ifDebug( kDebug() << "(" << ++cnt << ")" << m_id << identifier().toString() << index; )
+  if(shouldDoDUChainReferenceCounting(this)) {
+    ifDebug( kDebug() << index << "decreasing"; )
+    QMutexLocker lock(qualifiedidentifierRepository->mutex());
+    decrease(qualifiedidentifierRepository->dynamicItemFromIndexSimple(index)->m_refCount, index);
+  }
+}
+
 bool IndexedQualifiedIdentifier::operator==(const IndexedQualifiedIdentifier& rhs) const {
   return index == rhs.index;
 }
+
 bool IndexedQualifiedIdentifier::operator==(const QualifiedIdentifier& id) const {
   return index == id.index();
 }
@@ -1165,35 +1226,6 @@ IndexedQualifiedIdentifier::operator QualifiedIdentifier() const {
   return QualifiedIdentifier(index);
 }
 
-IndexedTypeIdentifier::IndexedTypeIdentifier() : index(emptyConstantQualifiedIdentifierPrivateIndex()) {
-}
-
-bool IndexedTypeIdentifier::isValid() const {
-  return index != emptyConstantQualifiedIdentifierPrivateIndex();
-}
-
-IndexedTypeIdentifier::IndexedTypeIdentifier(const TypeIdentifier& id) : index(id.index()) {
-}
-
-IndexedTypeIdentifier& IndexedTypeIdentifier::operator=(const TypeIdentifier& id) {
-  index = id.index();
-  return *this;
-}
-bool IndexedTypeIdentifier::operator==(const IndexedTypeIdentifier& rhs) const {
-  return index == rhs.index;
-}
-bool IndexedTypeIdentifier::operator==(const TypeIdentifier& id) const {
-  return index == id.index();
-}
-
-TypeIdentifier IndexedTypeIdentifier::identifier() const {
-  return TypeIdentifier(index);
-}
-
-IndexedTypeIdentifier::operator TypeIdentifier() const {
-  return TypeIdentifier(index);
-}
-
 }
 
 // kate: space-indent on; indent-width 2; tab-width 4; replace-tabs on; auto-insert-doxygen on
