@@ -19,6 +19,9 @@
 #include "./instantiationinformation.h"
 #include "identifier.h"
 #include "repositories/itemrepository.h"
+#include "types/typeutils.h"
+#include <typeinfo>
+#include "types/typealiastype.h"
 
 
 namespace KDevelop {
@@ -30,24 +33,25 @@ QualifiedIdentifier InstantiationInformation::applyToIdentifier(const QualifiedI
   if(id.count() > 1) {
     ret = id;
     ret.pop();
-    if(previousInstantiationInformation)
-      ret = IndexedInstantiationInformation(previousInstantiationInformation).information().applyToIdentifier(ret);
+    if(previousInstantiationInformation.index())
+      ret = previousInstantiationInformation.information().applyToIdentifier(ret);
   }
 
   Identifier lastId(id.last());
 
-  KDevVarLengthArray<TypeIdentifier> oldTemplateIdentifiers;
+  KDevVarLengthArray<IndexedTypeIdentifier> oldTemplateIdentifiers;
   for(uint a = 0; a < lastId.templateIdentifiersCount(); ++a)
     oldTemplateIdentifiers.append(lastId.templateIdentifier(a));
   lastId.clearTemplateIdentifiers();
 
   for(uint a = 0; a < templateParametersSize(); ++a) {
-    if(templateParameters()[a].type()) {
-      TypeIdentifier id(templateParameters()[a].type()->toString());
-      id.setIsExpression(true);
+    if(templateParameters()[a].abstractType()) {
+      QualifiedIdentifier qualified(templateParameters()[a].abstractType()->toString());
+      qualified.setIsExpression(true);
+      IndexedTypeIdentifier id(qualified);
       lastId.appendTemplateIdentifier(id);
     }else{
-      lastId.appendTemplateIdentifier(oldTemplateIdentifiers.size() > a ? oldTemplateIdentifiers[a] : TypeIdentifier());
+      lastId.appendTemplateIdentifier((uint) oldTemplateIdentifiers.size() > a ? oldTemplateIdentifiers[a] : IndexedTypeIdentifier());
     }
   }
 
@@ -58,27 +62,31 @@ QualifiedIdentifier InstantiationInformation::applyToIdentifier(const QualifiedI
   return ret;
 }
 
+void InstantiationInformation::addTemplateParameter(KDevelop::AbstractType::Ptr type) {
+  templateParametersList().append(type->indexed());
+}
+
 QString InstantiationInformation::toString(bool local) const {
     QString ret;
-    if(previousInstantiationInformation && !local)
-        ret = IndexedInstantiationInformation(previousInstantiationInformation).information().toString() + "::";
+    if(previousInstantiationInformation.index() && !local)
+        ret = previousInstantiationInformation.information().toString() + "::";
     ret += '<';
-    for(int a = 0; a < templateParametersSize(); ++a) {
+    for(uint a = 0; a < templateParametersSize(); ++a) {
         if(a)
             ret += ", ";
-        if(templateParameters()[a].type())
-          ret += templateParameters()[a].type()->toString();
+        if(templateParameters()[a].abstractType())
+          ret += templateParameters()[a].abstractType()->toString();
     }
     ret += '>';
     return ret;
 }
 
-InstantiationInformation::InstantiationInformation() : previousInstantiationInformation(0) {
+InstantiationInformation::InstantiationInformation() : m_refCount(0) {
   initializeAppendedLists();
 }
 
-InstantiationInformation::InstantiationInformation(const InstantiationInformation& rhs) : previousInstantiationInformation(rhs.previousInstantiationInformation) {
-  initializeAppendedLists();
+InstantiationInformation::InstantiationInformation(const InstantiationInformation& rhs, bool dynamic) : previousInstantiationInformation(rhs.previousInstantiationInformation), m_refCount(0) {
+  initializeAppendedLists(dynamic);
   copyListsFrom(rhs);
 }
 
@@ -93,7 +101,7 @@ InstantiationInformation& InstantiationInformation::operator=(const Instantiatio
 }
 
 bool InstantiationInformation::operator==(const InstantiationInformation& rhs) const {
-  if(previousInstantiationInformation != rhs.previousInstantiationInformation)
+  if(!(previousInstantiationInformation == rhs.previousInstantiationInformation))
     return false;
   return listsEqual(rhs);
 }
@@ -104,10 +112,10 @@ uint InstantiationInformation::hash() const {
     ret = (ret + param.hash()) * 117;
   }
 
-  return (ret + previousInstantiationInformation) * 31;
+  return (ret + previousInstantiationInformation.index()) * 31;
 }
 
-KDevelop::ItemRepository<InstantiationInformation, AppendedListItemRequest<InstantiationInformation> > instantiationInformationRepository("C++ Instantiation Information Repository");
+KDevelop::ItemRepository<InstantiationInformation, AppendedListItemRequest<InstantiationInformation> > instantiationInformationRepository("Instantiation Information Repository");
 
 
 const uint standardInstantiationInformationIndex = instantiationInformationRepository.index( InstantiationInformation() );
@@ -118,6 +126,46 @@ IndexedInstantiationInformation::IndexedInstantiationInformation() : m_index(0)
 IndexedInstantiationInformation::IndexedInstantiationInformation(uint index) : m_index(index) {
   if(m_index == standardInstantiationInformationIndex)
     m_index = 0;
+  
+  if(m_index && shouldDoDUChainReferenceCounting(this))
+  {
+    QMutexLocker lock(instantiationInformationRepository.mutex());
+    increase(instantiationInformationRepository.dynamicItemFromIndexSimple(m_index)->m_refCount, m_index);
+  }
+}
+
+IndexedInstantiationInformation::IndexedInstantiationInformation(const IndexedInstantiationInformation& rhs) : m_index(rhs.m_index) {
+  if(m_index && shouldDoDUChainReferenceCounting(this))
+  {
+    QMutexLocker lock(instantiationInformationRepository.mutex());
+    increase(instantiationInformationRepository.dynamicItemFromIndexSimple(m_index)->m_refCount, m_index);
+  }
+}
+
+IndexedInstantiationInformation& IndexedInstantiationInformation::operator=(const IndexedInstantiationInformation& rhs) {
+
+  if(m_index && shouldDoDUChainReferenceCounting(this))
+  {
+    QMutexLocker lock(instantiationInformationRepository.mutex());
+    decrease(instantiationInformationRepository.dynamicItemFromIndexSimple(m_index)->m_refCount, m_index);
+  }
+  
+  m_index = rhs.m_index;
+  
+  if(m_index && shouldDoDUChainReferenceCounting(this))
+  {
+    QMutexLocker lock(instantiationInformationRepository.mutex());
+    increase(instantiationInformationRepository.dynamicItemFromIndexSimple(m_index)->m_refCount, m_index);
+  }
+  return *this;
+}
+
+IndexedInstantiationInformation::~IndexedInstantiationInformation() {
+  if(m_index && shouldDoDUChainReferenceCounting(this))
+  {
+    QMutexLocker lock(instantiationInformationRepository.mutex());
+    decrease(instantiationInformationRepository.dynamicItemFromIndexSimple(m_index)->m_refCount, m_index);
+  }
 }
 
 bool IndexedInstantiationInformation::isValid() const {
