@@ -35,6 +35,11 @@
 #include "../types/referencetype.h"
 #include "../types/typeutils.h"
 #include "../persistentsymboltable.h"
+#include <interfaces/icore.h>
+#include <interfaces/idocumentationcontroller.h>
+#include <duchain/types/typealiastype.h>
+#include <duchain/classdeclaration.h>
+#include <typeinfo>
 
 namespace KDevelop {
 AbstractDeclarationNavigationContext::AbstractDeclarationNavigationContext( DeclarationPointer decl, KDevelop::TopDUContextPointer topContext, AbstractNavigationContext* previousContext)
@@ -57,6 +62,7 @@ QString AbstractDeclarationNavigationContext::name() const
 QString AbstractDeclarationNavigationContext::html(bool shorten)
 {
   clear();
+  m_shorten = shorten;
   modifyHtml()  += "<html><body><p><small><small>";
 
   addExternalHtml(m_prefix);
@@ -71,8 +77,12 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
     makeLink( m_previousContext->name(), m_previousContext->name(), NavigationAction(m_previousContext) );
     modifyHtml() += "<br>";
   }
-
+  
+  KSharedPtr<IDocumentation> doc;
+  
   if( !shorten ) {
+    doc = ICore::self()->documentationController()->documentationForDeclaration(m_declaration.data());
+    
     const AbstractFunctionDeclaration* function = dynamic_cast<const AbstractFunctionDeclaration*>(m_declaration.data());
     if( function ) {
       htmlFunction();
@@ -81,9 +91,16 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
         modifyHtml() += importantHighlight("typedef ");
 
       if(m_declaration->type<EnumeratorType>())
-        modifyHtml() += i18n("enumerator") + ' ';
+        modifyHtml() += i18n("enumerator ");
 
-      eventuallyMakeTypeLinks( m_declaration->abstractType() );
+      AbstractType::Ptr useType = m_declaration->abstractType();
+      if(m_declaration->isTypeAlias()) {
+        //Do not show the own name as type of typedefs
+        if(useType.cast<TypeAliasType>())
+          useType = useType.cast<TypeAliasType>()->type();
+      }
+      
+      eventuallyMakeTypeLinks( useType );
 
       modifyHtml() += ' ' + nameHighlight(Qt::escape(declarationName(m_declaration))) + "<br>";
     }else{
@@ -93,18 +110,18 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
 
       if(m_declaration->type<EnumerationType>()) {
         EnumerationType::Ptr enumeration = m_declaration->type<EnumerationType>();
-        modifyHtml() += i18n("enumeration") + ' ' + Qt::escape(m_declaration->identifier().toString()) + "<br>";
+        modifyHtml() += i18n("enumeration %1 <br/>", Qt::escape(m_declaration->identifier().toString()) );
       }
 
       if(m_declaration->isForwardDeclaration()) {
         ForwardDeclaration* forwardDec = static_cast<ForwardDeclaration*>(m_declaration.data());
         Declaration* resolved = forwardDec->resolve(m_topContext.data());
         if(resolved) {
-          modifyHtml() += '(' + i18n("resolved forward-declaration") + ": ";
+          modifyHtml() += i18n("( resolved forward-declaration: ");
           makeLink(resolved->identifier().toString(), KDevelop::DeclarationPointer(resolved), NavigationAction::NavigateDeclaration );
-          modifyHtml() += ") ";
+          modifyHtml() += i18n(") ");
         }else{
-          modifyHtml() += i18n("(unresolved forward-declaration)") + ' ';
+          modifyHtml() += i18n("(unresolved forward-declaration) ");
           QualifiedIdentifier id = forwardDec->qualifiedIdentifier();
           uint count;
           const IndexedDeclaration* decls;
@@ -123,41 +140,44 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
         }
       }
     }
-
-    QualifiedIdentifier identifier = m_declaration->qualifiedIdentifier();
-    if( identifier.count() > 1 ) {
-      if( m_declaration->context() && m_declaration->context()->owner() )
-      {
-        Declaration* decl = m_declaration->context()->owner();
-
-        FunctionDefinition* definition = dynamic_cast<FunctionDefinition*>(decl);
-        if(definition && definition->declaration())
-          decl = definition->declaration();
-
-        if(decl->abstractType().cast<EnumerationType>())
-          modifyHtml() += labelHighlight(i18n("Enum: "));
-        else
-          modifyHtml() += labelHighlight(i18n("Container: "));
-
-        makeLink( declarationName(DeclarationPointer(decl)), DeclarationPointer(decl), NavigationAction::NavigateDeclaration );
-        modifyHtml() += " ";
-      } else {
-        QualifiedIdentifier parent = identifier;
-        parent.pop();
-        modifyHtml() += labelHighlight(i18n("Scope: ")) + Qt::escape(parent.toString()) + ' ';
-      }
-    }
   }else{
     AbstractType::Ptr showType = m_declaration->abstractType();
     if(showType && showType.cast<FunctionType>()) {
-      modifyHtml() += labelHighlight(i18n("Returns: "));
       showType = showType.cast<FunctionType>()->returnType();
-    }else{
+      if(showType)
+        modifyHtml() += labelHighlight(i18n("Returns: "));
+    }else  if(showType) {
       modifyHtml() += labelHighlight(i18n("Type: "));
     }
     
-    if(showType)
-      modifyHtml() += Qt::escape(showType->toString()) + ' ';
+    if(showType) {
+      eventuallyMakeTypeLinks(showType);
+      modifyHtml() += " ";
+    }
+  }
+  
+  QualifiedIdentifier identifier = m_declaration->qualifiedIdentifier();
+  if( identifier.count() > 1 ) {
+    if( m_declaration->context() && m_declaration->context()->owner() )
+    {
+      Declaration* decl = m_declaration->context()->owner();
+
+      FunctionDefinition* definition = dynamic_cast<FunctionDefinition*>(decl);
+      if(definition && definition->declaration())
+        decl = definition->declaration();
+
+      if(decl->abstractType().cast<EnumerationType>())
+        modifyHtml() += labelHighlight(i18n("Enum: "));
+      else
+        modifyHtml() += labelHighlight(i18n("Container: "));
+
+      makeLink( declarationName(DeclarationPointer(decl)), DeclarationPointer(decl), NavigationAction::NavigateDeclaration );
+      modifyHtml() += " ";
+    } else {
+      QualifiedIdentifier parent = identifier;
+      parent.pop();
+      modifyHtml() += labelHighlight(i18n("Scope: %1 ", typeHighlight(Qt::escape(parent.toString()))));
+    }
   }
   
   if( shorten && !m_declaration->comment().isEmpty() ) {
@@ -175,7 +195,7 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
 
   QString access = stringFromAccess(m_declaration);
   if( !access.isEmpty() )
-    modifyHtml() += labelHighlight(i18n("Access: ")) + propertyHighlight(Qt::escape(access)) + ' ';
+    modifyHtml() += labelHighlight(i18n("Access: %1 ", propertyHighlight(Qt::escape(access))));
 
 
   ///@todo Enumerations
@@ -195,11 +215,11 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
   QString kind = declarationKind(m_declaration);
   if( !kind.isEmpty() ) {
     if( !detailsHtml.isEmpty() )
-      modifyHtml() += labelHighlight(i18n("Kind: ")) + importantHighlight(Qt::escape(kind)) + ' ' + detailsHtml + ' ';
+      modifyHtml() += labelHighlight(i18n("Kind: %1 %2 ", importantHighlight(Qt::escape(kind)), detailsHtml));
     else
-      modifyHtml() += labelHighlight(i18n("Kind: ")) + importantHighlight(Qt::escape(kind)) + ' ';
+      modifyHtml() += labelHighlight(i18n("Kind: %1 ", importantHighlight(Qt::escape(kind))));
   } else if( !detailsHtml.isEmpty() ) {
-    modifyHtml() += labelHighlight(i18n("Modifiers: ")) +  importantHighlight(Qt::escape(kind)) + ' ';
+    modifyHtml() += labelHighlight(i18n("Modifiers: %1 ",  importantHighlight(Qt::escape(kind))));
   }
 
   modifyHtml() += "<br />";
@@ -207,16 +227,6 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
   if(!shorten)
     htmlAdditionalNavigation();
   
-  if( !shorten && !m_declaration->comment().isEmpty() ) {
-    QString comment = m_declaration->comment();
-    comment.replace("<br />", "\n"); //do not escape html newlines within the comment
-    comment.replace("<br/>", "\n");
-    comment = Qt::escape(comment);
-    comment.replace('\n', "<br />"); //Replicate newlines in html
-    modifyHtml() += commentHighlight(comment);
-    modifyHtml() += "<br />";
-  }
-
   if( !shorten ) {
     if(dynamic_cast<FunctionDefinition*>(m_declaration.data()))
       modifyHtml() += labelHighlight(i18n( "Def.: " ));
@@ -240,9 +250,34 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
       }
     }
     
-    modifyHtml() += " ";
+    modifyHtml() += " "; //The action name _must_ stay "show_uses", since that is also used from outside
     makeLink(i18n("Show uses"), "show_uses", NavigationAction(m_declaration, NavigationAction::NavigateUses));
   }
+  
+  if( !shorten && (!m_declaration->comment().isEmpty() || doc) ) {
+    modifyHtml() += "<br />";
+    QString comment = m_declaration->comment();
+    if(comment.isEmpty() && doc) {
+      comment = doc->description();
+      if(!comment.isEmpty()) {
+        modifyHtml() += commentHighlight(comment) /*+ "<br />"*/;
+      }
+    } else if(!comment.isEmpty()) {
+      comment.replace("<br />", "\n"); //do not escape html newlines within the comment
+      comment.replace("<br/>", "\n");
+      comment = Qt::escape(comment);
+      comment.replace('\n', "<br />"); //Replicate newlines in html
+      modifyHtml() += commentHighlight(comment);
+      modifyHtml() += "<br />";
+    }
+  }
+  
+    if(!shorten && doc && doc->providesWidget()) {
+      modifyHtml() += "<br />" + i18n("Show documentation for ");
+      makeLink( m_declaration->qualifiedIdentifier().toString(), m_declaration, NavigationAction::ShowDocumentation );
+    }
+  
+  
     //modifyHtml() += "<br />";
 
   addExternalHtml(m_suffix);
@@ -252,6 +287,10 @@ QString AbstractDeclarationNavigationContext::html(bool shorten)
   return currentHtml();
 }
 
+KDevelop::AbstractType::Ptr AbstractDeclarationNavigationContext::typeToShow(KDevelop::AbstractType::Ptr type) {
+  return type;
+}
+
 void AbstractDeclarationNavigationContext::htmlFunction()
 {
   const AbstractFunctionDeclaration* function = dynamic_cast<const AbstractFunctionDeclaration*>(m_declaration.data());
@@ -278,10 +317,10 @@ void AbstractDeclarationNavigationContext::htmlFunction()
     KDevelop::DUContext* argumentContext = DUChainUtils::getArgumentContext(m_declaration.data());
 
     if(argumentContext) {
-      int firstDefaultParam = argumentContext->localDeclarations().count() - function->defaultParametersSize();
+      int firstDefaultParam = argumentContext->localDeclarations(m_topContext.data()).count() - function->defaultParametersSize();
       int currentArgNum = 0;
 
-      foreach(Declaration* argument, argumentContext->localDeclarations()) {
+      foreach(Declaration* argument, argumentContext->localDeclarations(m_topContext.data())) {
         if( !first )
           modifyHtml() += ", ";
         first = false;
@@ -312,9 +351,9 @@ void AbstractDeclarationNavigationContext::htmlAdditionalNavigation()
     Declaration* overridden = DUChainUtils::getOverridden(m_declaration.data());
 
     if(overridden) {
-        modifyHtml() += i18n("Overrides a") + ' ';
+        modifyHtml() += i18n("Overrides a ");
         makeLink(i18n("function"), QString("jump_to_overridden"), NavigationAction(DeclarationPointer(overridden), KDevelop::NavigationAction::NavigateDeclaration));
-        modifyHtml() += ' ' + i18n("from") + ' ';
+        modifyHtml() += i18n(" from ");
         makeLink(overridden->context()->scopeIdentifier(true).toString(), QString("jump_to_overridden_container"), NavigationAction(DeclarationPointer(overridden->context()->owner()), KDevelop::NavigationAction::NavigateDeclaration));
         
         modifyHtml() += "<br />";
@@ -327,9 +366,9 @@ void AbstractDeclarationNavigationContext::htmlAdditionalNavigation()
                                                 SimpleCursor::invalid(), AbstractType::Ptr(), m_topContext.data(), DUContext::DontSearchInParent);
       uint num = 0;
       foreach(Declaration* decl, decls) {
-        modifyHtml() += i18n("Hides a") + ' ';
+        modifyHtml() += i18n("Hides a ");
         makeLink(i18n("function"), QString("jump_to_hide_%1").arg(num), NavigationAction(DeclarationPointer(decl), KDevelop::NavigationAction::NavigateDeclaration));
-        modifyHtml() += ' ' + i18n("from") + ' ';
+        modifyHtml() += i18n(" from ");
         makeLink(decl->context()->scopeIdentifier(true).toString(), QString("jump_to_hide_container_%1").arg(num), NavigationAction(DeclarationPointer(decl->context()->owner()), KDevelop::NavigationAction::NavigateDeclaration));
         
         modifyHtml() += "<br />";
@@ -345,7 +384,7 @@ void AbstractDeclarationNavigationContext::htmlAdditionalNavigation()
         QList<Declaration*> overriders = DUChainUtils::getOverriders(classDecl, classFunDecl, maxAllowedSteps);
         
         if(!overriders.isEmpty()) {
-          modifyHtml() += i18n("Overridden in") + ' ';
+          modifyHtml() += i18n("Overridden in ");
           bool first = true;
           foreach(Declaration* overrider, overriders) {
             if(!first)
@@ -368,7 +407,7 @@ void AbstractDeclarationNavigationContext::htmlAdditionalNavigation()
   QList<Declaration*> inheriters = DUChainUtils::getInheriters(m_declaration.data(), maxAllowedSteps);
   
   if(!inheriters.isEmpty()) {
-      modifyHtml() += i18n("Inherited by") + ' ';
+      modifyHtml() += i18n("Inherited by ");
       bool first = true;
       foreach(Declaration* importer, inheriters) {
         if(!first)
@@ -403,10 +442,38 @@ void AbstractDeclarationNavigationContext::htmlClass()
 {
   StructureType::Ptr klass = m_declaration->abstractType().cast<StructureType>();
   Q_ASSERT(klass);
-
-  ///@todo Use information from ClassDeclaration to print whether it's a struct/union/etc.
-  modifyHtml() += "class ";
-  eventuallyMakeTypeLinks( klass.cast<AbstractType>() );
+  
+  ClassDeclaration* classDecl = dynamic_cast<ClassDeclaration*>(klass->declaration(m_topContext.data()));
+  if(classDecl) {
+    switch ( classDecl->classType() ) {
+      case ClassDeclarationData::Class:
+        modifyHtml() += "class ";
+        break;
+      case ClassDeclarationData::Struct:
+        modifyHtml() += "struct ";
+        break;
+      case ClassDeclarationData::Union:
+        modifyHtml() += "union ";
+        break;
+      case ClassDeclarationData::Interface:
+        modifyHtml() += "interface ";
+        break;
+      default:
+        modifyHtml() += "<unknown type> ";
+        break;
+    }
+    eventuallyMakeTypeLinks( klass.cast<AbstractType>() );
+    
+    FOREACH_FUNCTION( const KDevelop::BaseClassInstance& base, classDecl->baseClasses ) {
+      modifyHtml() += ", " + stringFromAccess(base.access) + " " + (base.virtualInheritance ? QString("virtual") : QString()) + " ";
+      eventuallyMakeTypeLinks(base.baseClass.abstractType());
+    }
+    modifyHtml() += " ";
+  } else {
+    /// @todo How can we get here? and should this really be a class?
+    modifyHtml() += "class ";
+    eventuallyMakeTypeLinks( klass.cast<AbstractType>() );
+  }
 }
 
 void AbstractDeclarationNavigationContext::htmlIdentifiedType(AbstractType::Ptr type, const IdentifiedType* idType)
@@ -426,50 +493,50 @@ void AbstractDeclarationNavigationContext::htmlIdentifiedType(AbstractType::Ptr
     //We leave out the * and & reference and pointer signs, those are added to the end
     makeLink(id.toString() , DeclarationPointer(idType->declaration(m_topContext.data())), NavigationAction::NavigateDeclaration );
   } else {
-    modifyHtml() += Qt::escape(type->toString());
+    kDebug() << "could not resolve declaration:" << idType->declarationId().isDirect() << idType->qualifiedIdentifier().toString() << "in top-context" << m_topContext->url().str();
+    modifyHtml() += typeHighlight(Qt::escape(type->toString()));
   }
 }
 
 void AbstractDeclarationNavigationContext::eventuallyMakeTypeLinks( AbstractType::Ptr type )
 {
+  type = typeToShow(type);
+  
   if( !type ) {
-    modifyHtml() += Qt::escape("<no type>");
+    modifyHtml() += typeHighlight(Qt::escape("<no type>"));
     return;
   }
-  AbstractType::Ptr target = TypeUtils::targetType( type, m_topContext.data() );
+
+  AbstractType::Ptr target = TypeUtils::targetTypeKeepAliases( type, m_topContext.data() );
   const IdentifiedType* idType = dynamic_cast<const IdentifiedType*>( target.unsafeData() );
 
-  if( idType ) {
+  kDebug() << "making type-links for" << type->toString() << typeid(*type).name();
+  
+  if( idType && idType->declaration(m_topContext.data()) ) {
     ///@todo This is C++ specific, move into subclass
     
     if(target->modifiers() & AbstractType::ConstModifier)
-      modifyHtml() += "const ";
+      modifyHtml() += typeHighlight("const ");
     
-    htmlIdentifiedType(type, idType);
+    htmlIdentifiedType(target, idType);
 
-    PointerType::Ptr pointer = type.cast<PointerType>();
-    ReferenceType::Ptr ref = type.cast<ReferenceType>();
+    //We need to exchange the target type, else template-parameters may confuse this
+    SimpleTypeExchanger exchangeTarget(target, AbstractType::Ptr());
     
-    //Add reference and pointer
-    while(pointer || ref) {
-      if(pointer) {
-        modifyHtml() += Qt::escape("*");
-        if(pointer->modifiers() & AbstractType::ConstModifier)
-          modifyHtml() += " const";
-        ref = pointer->baseType().cast<ReferenceType>();
-        pointer = pointer->baseType().cast<PointerType>();
-      }
-      if(ref) {
-        modifyHtml() += Qt::escape("&");
-        if(ref->modifiers() & AbstractType::ConstModifier)
-          modifyHtml() += " const";
-        pointer = ref->baseType().cast<PointerType>();
-        ref = ref->baseType().cast<ReferenceType>();
-      }
+    AbstractType::Ptr exchanged = exchangeTarget.exchange(type);
+    
+    if(exchanged) {
+      QString typeSuffixString = exchanged->toString();
+      QRegExp suffixExp("\\&|\\*");
+      int suffixPos = typeSuffixString.indexOf(suffixExp);
+      if(suffixPos != -1)
+        modifyHtml() += typeHighlight(typeSuffixString.mid(suffixPos));
     }
 
   } else {
-    modifyHtml() += Qt::escape(type->toString());
+    if(idType)
+      kDebug() << "identified type could not be resolved:" << idType->qualifiedIdentifier() << idType->declarationId().isValid() << idType->declarationId().isDirect();
+    modifyHtml() += typeHighlight(Qt::escape(type->toString()));
   }
 }
 
@@ -510,7 +577,7 @@ QString AbstractDeclarationNavigationContext::declarationName( DeclarationPointe
   }
 
   if( !decl )
-    return i18nc("An unknown declaration that is unknown", "Unknown");
+    return i18nc("A declaration that is unknown", "Unknown");
   else
     return decl->identifier().toString();
 }
@@ -572,8 +639,11 @@ QStringList AbstractDeclarationNavigationContext::declarationDetails(Declaration
         details << "destructor";
       if( classFunDecl->isConversionFunction() )
         details << "conversion-function";
+      if( classFunDecl->isAbstract() )
+        details << "abstract";
     }
   }
+  
   return details;
 }
 
