@@ -35,15 +35,25 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractDeclarationNavigationContext : public
     virtual QString html(bool shorten = false);
 
     DeclarationPointer declaration() const;
+    
+    ///Execute an action. For example "show_uses" shows the uses of the declaration.
+    ///Returns the context pointer for the new state.
+    virtual NavigationContextPointer executeKeyAction(QString key);
 
   protected:
     DeclarationPointer m_declaration;
 
+    
     static QString stringFromAccess(Declaration::AccessPolicy access);
     static QString stringFromAccess(DeclarationPointer decl);
     static QString declarationName( DeclarationPointer decl );
     static QStringList declarationDetails(DeclarationPointer decl);
 
+    ///This can be used for example to resolve typedefs within the type.
+    ///All types that are visualized in the navigation-context are/should be mangled through this.
+    ///The default-implementation returns the original type.
+    virtual AbstractType::Ptr typeToShow(AbstractType::Ptr type);
+    
     ///Print the function-signature in a way that return-type and argument can be jumped to
     virtual void htmlFunction();
     ///Navigation for additional less important links, like what function was overloaded etc.
@@ -55,8 +65,6 @@ class KDEVPLATFORMLANGUAGE_EXPORT AbstractDeclarationNavigationContext : public
     ///Creates and registers a link for the given type that jumps to its declaration and to the template-argument declarations
     virtual void eventuallyMakeTypeLinks( KDevelop::AbstractType::Ptr type );
 
-    virtual NavigationContextPointer executeKeyAction(QString key);
-
     ///Creates a link that triggers a recomputation of this context with m_fullBackwardSearch set to true
     void createFullBackwardSearchLink(QString string);
     
