@@ -37,12 +37,22 @@
 #include "../types/enumerationtype.h"
 #include "../types/referencetype.h"
 #include "../types/pointertype.h"
+#include <interfaces/idocumentationcontroller.h>
 
 
 namespace KDevelop {
+  
+void AbstractNavigationContext::setTopContext(KDevelop::TopDUContextPointer context) {
+  m_topContext = context;
+}
+
+KDevelop::TopDUContextPointer AbstractNavigationContext::topContext() const {
+  return m_topContext;
+}
+
 
 AbstractNavigationContext::AbstractNavigationContext( KDevelop::TopDUContextPointer topContext, AbstractNavigationContext* previousContext)
-  : m_selectedLink(0), m_linkCount(-1), m_currentPositionLine(0),
+  : m_shorten(false), m_selectedLink(0), m_linkCount(-1), m_currentPositionLine(0),
     m_previousContext(previousContext), m_topContext(topContext)
 {
 }
@@ -75,12 +85,18 @@ void AbstractNavigationContext::addExternalHtml( const QString& text )
 void AbstractNavigationContext::makeLink( const QString& name, DeclarationPointer declaration, NavigationAction::Type actionType )
 {
   NavigationAction action( declaration, actionType );
-  QString targetId = QString("%1").arg((quint64)declaration.data());
+  QString targetId = QString::number((quint64)declaration.data() * actionType);
   makeLink(name, targetId, action);
 }
 
 void AbstractNavigationContext::makeLink( const QString& name, QString targetId, const NavigationAction& action)
 {
+  if(m_shorten) {
+    //Do not create links in shortened mode, it's only for viewing
+    modifyHtml() += typeHighlight(Qt::escape(name));
+    return;
+  }
+  
   m_links[ targetId ] = action;
   m_intLinks[ m_linkCount ] = action;
   m_linkLines[ m_linkCount ] = m_currentLine;
@@ -88,10 +104,10 @@ void AbstractNavigationContext::makeLink( const QString& name, QString targetId,
     m_currentPositionLine = -1;
     m_selectedLink = m_linkCount;
   }
-
+  
   QString str = Qt::escape(name);
-  if( m_linkCount == m_selectedLink ) ///@todo Change background-color here instead of foreground-color
-    str = "<font color=\"#880088\">" + str + "</font>";
+  if( m_linkCount == m_selectedLink )
+    str = "<font style=\"background-color:#f1f1f1;\" color=\"#880088\">" + str + "</font>";
 
   modifyHtml() +=  "<a href=\"" + targetId + "\"" + ((m_linkCount == m_selectedLink && m_currentPositionLine == -1) ? QString(" name = \"currentPosition\"") : QString()) + ">" + str + "</a>";
 
@@ -110,6 +126,15 @@ void AbstractNavigationContext::clear() {
     m_linkLines.clear();
 }
 
+
+NavigationContextPointer AbstractNavigationContext::executeLink (QString link)
+{
+  if(!m_links.contains(link))
+    return NavigationContextPointer(this);
+
+  return execute(m_links[link]);
+}
+
 NavigationContextPointer AbstractNavigationContext::executeKeyAction(QString key) {
   Q_UNUSED(key);
   return NavigationContextPointer(this);
@@ -125,7 +150,7 @@ NavigationContextPointer AbstractNavigationContext::execute(NavigationAction& ac
 
 
   if( !action.decl && (action.type != NavigationAction::JumpToSource || action.document.isEmpty()) ) {
-      kDebug(9007) << "Navigation-action has invalid declaration" << endl;
+      kDebug() << "Navigation-action has invalid declaration" << endl;
       return NavigationContextPointer(this);
   }
   qRegisterMetaType<KUrl>("KUrl");
@@ -133,7 +158,7 @@ NavigationContextPointer AbstractNavigationContext::execute(NavigationAction& ac
 
   switch( action.type ) {
     case NavigationAction::None:
-      kDebug(9007) << "Tried to execute an invalid action in navigation-widget" << endl;
+      kDebug() << "Tried to execute an invalid action in navigation-widget" << endl;
       break;
     case NavigationAction::NavigateDeclaration:
     {
@@ -144,7 +169,7 @@ NavigationContextPointer AbstractNavigationContext::execute(NavigationAction& ac
     } break;
     case NavigationAction::NavigateUses:
       return registerChild(new UsesNavigationContext(action.decl.data(), this));
-  case NavigationAction::JumpToSource:
+    case NavigationAction::JumpToSource:
       {
         KUrl doc = action.document;
         KTextEditor::Cursor cursor = action.cursor;
@@ -167,11 +192,20 @@ NavigationContextPointer AbstractNavigationContext::execute(NavigationAction& ac
         QMetaObject::invokeMethod( ICore::self()->documentController(), "openDocument", Qt::QueuedConnection, Q_ARG(KUrl, doc), Q_ARG(KTextEditor::Cursor, cursor) );
         break;
       }
+    case NavigationAction::ShowDocumentation: {
+        KSharedPtr<IDocumentation> doc=ICore::self()->documentationController()->documentationForDeclaration(action.decl.data());
+        ICore::self()->documentationController()->showDocumentation(doc);
+      }
+      break;
   }
 
   return NavigationContextPointer( this );
 }
 
+void AbstractNavigationContext::setPreviousContext(KDevelop::AbstractNavigationContext* previous) {
+  m_previousContext = previous;
+}
+
 NavigationContextPointer AbstractNavigationContext::registerChild( AbstractNavigationContext* context ) {
   m_children << NavigationContextPointer(context);
   return m_children.last();
@@ -185,7 +219,7 @@ NavigationContextPointer AbstractNavigationContext::registerChild(DeclarationPoi
   if(abstractNavigationWidget)
     ret = abstractNavigationWidget->context();
   delete navigationWidget;
-  ret->m_previousContext = this;
+  ret->setPreviousContext(this);
   m_children << ret;
   return ret;
 }
@@ -311,10 +345,10 @@ NavigationContextPointer AbstractNavigationContext::accept(IndexedDeclaration de
 
 NavigationContextPointer AbstractNavigationContext::acceptLink(const QString& link) {
   if( !m_links.contains(link) ) {
-    kDebug(9007) << "Executed unregistered link " << link << endl;
+    kDebug() << "Executed unregistered link " << link << endl;
     return NavigationContextPointer(this);
   }
-
+  
   return execute(m_links[link]);
 }
 
@@ -356,7 +390,8 @@ QString AbstractNavigationContext::declarationKind(DeclarationPointer decl)
   return kind;
 }
 
-QString AbstractNavigationContext::html(bool /*shorten*/) {
+QString AbstractNavigationContext::html(bool shorten) {
+  m_shorten = shorten;
   return QString();
 }
 
@@ -399,6 +434,7 @@ QString AbstractNavigationContext::currentHtml() const {
 }
 
 
+const Colorizer AbstractNavigationContext::typeHighlight("006000");
 const Colorizer AbstractNavigationContext::errorHighlight("990000");
 const Colorizer AbstractNavigationContext::labelHighlight("000035");
 const Colorizer AbstractNavigationContext::codeHighlight("005000");
