@@ -48,7 +48,7 @@
 namespace KDevelop {
 
 AbstractNavigationWidget::AbstractNavigationWidget()
-  : m_currentWidget(0), m_browser(0)
+  : m_browser(0), m_currentWidget(0)
 {
   setPalette( QApplication::palette() );
   resize(100, 100);
@@ -61,6 +61,7 @@ QSize AbstractNavigationWidget::sizeHint() const
   if(m_browser) {
     updateIdealSize();
     QSize ret = QSize(qMin(m_idealTextSize.width(), maxNavigationWidgetWidth), qMin(m_idealTextSize.height(), 300));
+    
     if(m_currentWidget) {
       ret.setHeight( ret.height() + m_currentWidget->sizeHint().height() );
       if(m_currentWidget->sizeHint().width() > ret.width())
@@ -76,6 +77,9 @@ QSize AbstractNavigationWidget::sizeHint() const
 
 void AbstractNavigationWidget::initBrowser(int height) {
   m_browser = new KTextBrowser;
+  
+  // since we can embed arbitrary HTML we have to make sure it stays readable by forcing a black-white palette
+  m_browser->setPalette( QPalette( Qt::black, Qt::white ) );
 
   m_browser->setOpenLinks(false);
   m_browser->setOpenExternalLinks(false);
@@ -135,6 +139,13 @@ void AbstractNavigationWidget::update() {
     m_currentText = html;
     m_idealTextSize = QSize();
 
+    QSize hint = sizeHint();
+    if(hint.height() >= m_idealTextSize.height()) {
+      m_browser->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    }else{
+      m_browser->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    }
+    
     m_browser->verticalScrollBar()->setValue(scrollPos);
     m_browser->scrollToAnchor("currentPosition");
     m_browser->show();
@@ -182,6 +193,13 @@ void AbstractNavigationWidget::keyPressEvent(QKeyEvent* event) {
   QWidget::keyPressEvent(event);
 }
 
+void AbstractNavigationWidget::executeContextAction(QString action) {
+  DUChainReadLocker lock( DUChain::lock() );
+  setContext(m_context->executeLink(action));
+  Q_ASSERT(m_context);
+  update();
+}
+
 void AbstractNavigationWidget::next() {
   DUChainReadLocker lock( DUChain::lock() );
   Q_ASSERT( m_context );
