@@ -157,11 +157,11 @@ void OneUseWidget::resizeEvent ( QResizeEvent * event ) {
   int maxCutOff = m_sourceLine.length() - (m_range.end.column - m_range.start.column);
 
   //Reset so we also get more context while up-sizing
-  setText(i18n("<a href='open'>Line <b>%1</b></a>", m_range.start.line+1) + ' ' + highlightAndEscapeUseText(m_sourceLine, cutOff, m_range));
+  setText(i18n("<a href='open'>Line <b>%1</b></a> %2", m_range.start.line+1, highlightAndEscapeUseText(m_sourceLine, cutOff, m_range)));
 
   while(sizeHint().width() > size.width() && cutOff < maxCutOff) {
     //We've got to save space
-    setText(i18n("<a href='open'>Line <b>%1</b></a>", m_range.start.line+1) + ' ' + highlightAndEscapeUseText(m_sourceLine, cutOff, m_range));
+    setText(i18n("<a href='open'>Line <b>%1</b></a> %2", m_range.start.line+1, highlightAndEscapeUseText(m_sourceLine, cutOff, m_range)));
     cutOff += 5;
   }
 
@@ -417,14 +417,14 @@ m_allDeclarations(allDeclarations) {
       KDevelop::IProject* project = ICore::self()->projectController()->findProjectForUrl(topContext.data()->url().toUrl());
       if(project) {
         projectName = project->name();
-        fileName = project->relativeUrl(topContext.data()->url().toUrl()).path();
+        fileName = project->relativeUrl(topContext.data()->url().toUrl()).toLocalFile();
       }
     }
 
     if(topContext.isLoaded())
       usesCount = DUChainUtils::contextCountUses(topContext.data(), declaration.data());
 
-    label->setText(i18n("<b>Project:</b> %1 &nbsp; <b>File:</b> %2 &nbsp; <i>(%3 uses)</i>", projectName, fileName, usesCount));
+    label->setText(i18np("<b>Project:</b> %2 &nbsp; <b>File:</b> %3 &nbsp; <i>(1 use)</i>", "<b>Project:</b> %2 &nbsp; <b>File:</b> %3 &nbsp; <i>(%1 uses)</i>", usesCount, projectName, fileName));
 
     m_button->setIcon(KIcon("go-next"));
     connect(m_button, SIGNAL(clicked(bool)), this, SLOT(labelClicked()));
