@@ -27,10 +27,17 @@
 #include "identifier.h"
 #include "ducontext.h"
 #include "topducontext.h"
+#include "duchain.h"
+#include "duchainlock.h"
 #include <util/embeddedfreetree.h>
 
+//For now, just _always_ use the cache
+const uint MinimumCountForCache = 1;
+
 namespace KDevelop {
 
+Utils::BasicSetRepository recursiveImportCacheRepository("Recursive Imports Cache", 0, false);
+
 DEFINE_LIST_MEMBER_HASH(PersistentSymbolTableItem, declarations, IndexedDeclaration)
 
 class PersistentSymbolTableItem {
@@ -38,8 +45,8 @@ class PersistentSymbolTableItem {
   PersistentSymbolTableItem() : centralFreeItem(-1) {
     initializeAppendedLists();
   }
-  PersistentSymbolTableItem(const PersistentSymbolTableItem& rhs) : id(rhs.id), centralFreeItem(rhs.centralFreeItem) {
-    initializeAppendedLists();
+  PersistentSymbolTableItem(const PersistentSymbolTableItem& rhs, bool dynamic = true) : id(rhs.id), centralFreeItem(rhs.centralFreeItem) {
+    initializeAppendedLists(dynamic);
     copyListsFrom(rhs);
   }
   
@@ -47,10 +54,10 @@ class PersistentSymbolTableItem {
     freeAppendedLists();
   }
   
-  unsigned int hash() const {
+  inline unsigned int hash() const {
     //We only compare the declaration. This allows us implementing a map, although the item-repository
     //originally represents a set.
-    return id.index;
+    return id.getIndex();
   }
   
   unsigned int itemSize() const {
@@ -87,10 +94,15 @@ class PersistentSymbolTableRequestItem {
   }
 
   void createItem(PersistentSymbolTableItem* item) const {
-    item->initializeAppendedLists(false);
-    item->id = m_item.id;
-    item->centralFreeItem = m_item.centralFreeItem;
-    item->copyListsFrom(m_item);
+    new (item) PersistentSymbolTableItem(m_item, false);
+  }
+  
+  static void destroy(PersistentSymbolTableItem* item, KDevelop::AbstractItemRepository&) {
+    item->~PersistentSymbolTableItem();
+  }
+
+  static bool persistent(const PersistentSymbolTableItem*) {
+    return true; //Nothing to do
   }
   
   bool equals(const PersistentSymbolTableItem* item) const {
@@ -107,8 +119,8 @@ class PersistentContextTableItem {
   PersistentContextTableItem() : centralFreeItem(-1) {
     initializeAppendedLists();
   }
-  PersistentContextTableItem(const PersistentContextTableItem& rhs) : id(rhs.id), centralFreeItem(rhs.centralFreeItem) {
-    initializeAppendedLists();
+  PersistentContextTableItem(const PersistentContextTableItem& rhs, bool dynamic = true) : id(rhs.id), centralFreeItem(rhs.centralFreeItem) {
+    initializeAppendedLists(dynamic);
     copyListsFrom(rhs);
   }
   
@@ -116,10 +128,10 @@ class PersistentContextTableItem {
     freeAppendedLists();
   }
   
-  unsigned int hash() const {
+  inline unsigned int hash() const {
     //We only compare the context. This allows us implementing a map, although the item-repository
     //originally represents a set.
-    return id.index;
+    return id.getIndex();
   }
   
   size_t itemSize() const {
@@ -156,10 +168,15 @@ class PersistentContextTableRequestItem {
   }
 
   void createItem(PersistentContextTableItem* item) const {
-    item->initializeAppendedLists(false);
-    item->id = m_item.id;
-    item->centralFreeItem = m_item.centralFreeItem;
-    item->copyListsFrom(m_item);
+    new (item) PersistentContextTableItem(m_item, false);
+  }
+  
+  static void destroy(PersistentContextTableItem* item, KDevelop::AbstractItemRepository&) {
+    item->~PersistentContextTableItem();
+  }
+  
+  static bool persistent(const PersistentContextTableItem*) {
+    return true; //Nothing to do
   }
   
   bool equals(const PersistentContextTableItem* item) const {
@@ -169,15 +186,43 @@ class PersistentContextTableRequestItem {
   const PersistentContextTableItem& m_item;
 };
 
+template<class ValueType>
+struct CacheEntry {
+  
+  typedef KDevVarLengthArray<ValueType> Data;
+  typedef QHash<TopDUContext::IndexedRecursiveImports, Data > DataHash;
+  
+  DataHash m_hash;
+};
 
 struct PersistentSymbolTablePrivate {
   PersistentSymbolTablePrivate() : m_declarations("Persistent Declaration Table"), m_contexts("Persistent Context Table") {
+    m_contexts.setMutex(m_declarations.mutex());
   }
   //Maps declaration-ids to declarations
-  ItemRepository<PersistentSymbolTableItem, PersistentSymbolTableRequestItem> m_declarations;
-  ItemRepository<PersistentContextTableItem, PersistentContextTableRequestItem> m_contexts;
+  ItemRepository<PersistentSymbolTableItem, PersistentSymbolTableRequestItem, true, false> m_declarations;
+  ItemRepository<PersistentContextTableItem, PersistentContextTableRequestItem, true, false> m_contexts;
+  
+  
+  QHash<IndexedQualifiedIdentifier, CacheEntry<IndexedDeclaration> > m_declarationsCache;
+  QHash<IndexedQualifiedIdentifier, CacheEntry<IndexedDUContext> > m_contextsCache;
+  
+  //We cache the imports so the currently used nodes are very close in memory, which leads to much better CPU cache utilization
+  QHash<TopDUContext::IndexedRecursiveImports, PersistentSymbolTable::CachedIndexedRecursiveImports> m_importsCache;
 };
 
+void PersistentSymbolTable::clearCache()
+{
+  ENSURE_CHAIN_WRITE_LOCKED
+  {
+    QMutexLocker lock(d->m_declarations.mutex());
+    QMutexLocker lock2(d->m_contexts.mutex());
+    d->m_importsCache.clear();
+    d->m_contextsCache.clear();
+    d->m_declarationsCache.clear();
+  }
+}
+
 PersistentSymbolTable::PersistentSymbolTable() : d(new PersistentSymbolTablePrivate())
 {
 }
@@ -189,6 +234,11 @@ PersistentSymbolTable::~PersistentSymbolTable()
 
 void PersistentSymbolTable::addDeclaration(const IndexedQualifiedIdentifier& id, const IndexedDeclaration& declaration)
 {
+  QMutexLocker lock(d->m_declarations.mutex());
+  ENSURE_CHAIN_WRITE_LOCKED
+  
+  d->m_declarationsCache.remove(id);
+  
   PersistentSymbolTableItem item;
   item.id = id;
   PersistentSymbolTableRequestItem request(item);
@@ -204,7 +254,7 @@ void PersistentSymbolTable::addDeclaration(const IndexedQualifiedIdentifier& id,
     if(alg.indexOf(declaration) != -1)
       return;
     
-    PersistentSymbolTableItem* editableItem = d->m_declarations.dynamicItemFromIndex(index);
+    DynamicItem<PersistentSymbolTableItem, true> editableItem = d->m_declarations.dynamicItemFromIndex(index);
     
     EmbeddedTreeAddItem<IndexedDeclaration, IndexedDeclarationHandler> add(const_cast<IndexedDeclaration*>(editableItem->declarations()), editableItem->declarationsSize(), editableItem->centralFreeItem, declaration);
     
@@ -230,6 +280,12 @@ void PersistentSymbolTable::addDeclaration(const IndexedQualifiedIdentifier& id,
 
 void PersistentSymbolTable::removeDeclaration(const IndexedQualifiedIdentifier& id, const IndexedDeclaration& declaration)
 {
+  QMutexLocker lock(d->m_declarations.mutex());
+  ENSURE_CHAIN_WRITE_LOCKED
+  
+  d->m_declarationsCache.remove(id);
+  Q_ASSERT(!d->m_declarationsCache.contains(id));
+  
   PersistentSymbolTableItem item;
   item.id = id;
   PersistentSymbolTableRequestItem request(item);
@@ -245,7 +301,7 @@ void PersistentSymbolTable::removeDeclaration(const IndexedQualifiedIdentifier&
     if(alg.indexOf(declaration) == -1)
       return;
     
-    PersistentSymbolTableItem* editableItem = d->m_declarations.dynamicItemFromIndex(index);
+    DynamicItem<PersistentSymbolTableItem, true> editableItem = d->m_declarations.dynamicItemFromIndex(index);
     
     EmbeddedTreeRemoveItem<IndexedDeclaration, IndexedDeclarationHandler> remove(const_cast<IndexedDeclaration*>(editableItem->declarations()), editableItem->declarationsSize(), editableItem->centralFreeItem, declaration);
     
@@ -268,7 +324,119 @@ void PersistentSymbolTable::removeDeclaration(const IndexedQualifiedIdentifier&
     d->m_declarations.index(request);
 }
 
+struct DeclarationCacheVisitor {
+  DeclarationCacheVisitor(KDevVarLengthArray<IndexedDeclaration>& _cache) : cache(_cache) {
+  }
+  
+  bool operator()(const IndexedDeclaration& decl) const {
+    cache.append(decl);
+    return true;
+  }
+  
+  KDevVarLengthArray<IndexedDeclaration>& cache;
+};
+
+PersistentSymbolTable::FilteredDeclarationIterator PersistentSymbolTable::getFilteredDeclarations(const IndexedQualifiedIdentifier& id, const TopDUContext::IndexedRecursiveImports& visibility) const {
+  
+  QMutexLocker lock(d->m_declarations.mutex());
+  ENSURE_CHAIN_READ_LOCKED
+  
+  Declarations decls = getDeclarations(id).iterator();
+  
+  CachedIndexedRecursiveImports cachedImports;
+  
+  QHash<TopDUContext::IndexedRecursiveImports, CachedIndexedRecursiveImports>::const_iterator it = d->m_importsCache.find(visibility);
+  if(it != d->m_importsCache.end()) {
+    cachedImports = *it;
+  }else{
+    cachedImports = CachedIndexedRecursiveImports(visibility.set().stdSet());
+    d->m_importsCache.insert(visibility, cachedImports);
+  }
+  
+  if(decls.dataSize() > MinimumCountForCache)
+  {
+    //Do visibility caching
+    CacheEntry<IndexedDeclaration>& cached(d->m_declarationsCache[id]);
+    CacheEntry<IndexedDeclaration>::DataHash::const_iterator cacheIt = cached.m_hash.find(visibility);
+    if(cacheIt != cached.m_hash.end())
+      return FilteredDeclarationIterator(Declarations::Iterator(cacheIt->constData(), cacheIt->size(), -1), cachedImports);
+
+    CacheEntry<IndexedDeclaration>::DataHash::iterator insertIt = cached.m_hash.insert(visibility, KDevVarLengthArray<IndexedDeclaration>());
+    
+    KDevVarLengthArray<IndexedDeclaration>& cache(*insertIt);
+    
+    {
+      typedef ConvenientEmbeddedSetTreeFilterVisitor<IndexedDeclaration, IndexedDeclarationHandler, IndexedTopDUContext, CachedIndexedRecursiveImports, DeclarationTopContextExtractor, DeclarationCacheVisitor> FilteredDeclarationCacheVisitor;
+    
+      //The visitor visits all the declarations from within its constructor
+      DeclarationCacheVisitor v(cache);
+      FilteredDeclarationCacheVisitor visitor(v, decls.iterator(), cachedImports);
+    }
+    
+    return FilteredDeclarationIterator(Declarations::Iterator(cache.constData(), cache.size(), -1), cachedImports, true);
+  }else{
+    return FilteredDeclarationIterator(decls.iterator(), cachedImports);
+  }
+}
+
+struct DUContextCacheVisitor {
+  DUContextCacheVisitor(KDevVarLengthArray<IndexedDUContext>& _cache) : cache(_cache) {
+  }
+  
+  bool operator()(const IndexedDUContext& decl) const {
+    cache.append(decl);
+    return true;
+  }
+  
+  KDevVarLengthArray<IndexedDUContext>& cache;
+};
+
+PersistentSymbolTable::FilteredDUContextIterator PersistentSymbolTable::getFilteredContexts(const IndexedQualifiedIdentifier& id, const TopDUContext::IndexedRecursiveImports& visibility) const {
+  QMutexLocker lock(d->m_contexts.mutex());
+  ENSURE_CHAIN_READ_LOCKED
+  
+  Contexts contexts = getContexts(id);
+  
+  CachedIndexedRecursiveImports cachedImports;
+  
+  QHash<TopDUContext::IndexedRecursiveImports, CachedIndexedRecursiveImports>::const_iterator it = d->m_importsCache.find(visibility);
+  if(it != d->m_importsCache.end()) {
+    cachedImports = *it;
+  }else{
+    cachedImports = CachedIndexedRecursiveImports(visibility.set().stdSet());
+    d->m_importsCache.insert(visibility, cachedImports);
+  }
+  
+  if(contexts.dataSize() > MinimumCountForCache)
+  {
+    //Do visibility caching
+    CacheEntry<IndexedDUContext>& cached(d->m_contextsCache[id]);
+    CacheEntry<IndexedDUContext>::DataHash::const_iterator cacheIt = cached.m_hash.find(visibility);
+    if(cacheIt != cached.m_hash.end())
+      return FilteredDUContextIterator(Contexts::Iterator(cacheIt->constData(), cacheIt->size(), -1), cachedImports);
+
+    CacheEntry<IndexedDUContext>::DataHash::iterator insertIt = cached.m_hash.insert(visibility, KDevVarLengthArray<IndexedDUContext>());
+    
+    KDevVarLengthArray<IndexedDUContext>& cache(*insertIt);
+    
+    {
+      typedef ConvenientEmbeddedSetTreeFilterVisitor<IndexedDUContext, IndexedDUContextHandler, IndexedTopDUContext, CachedIndexedRecursiveImports, DUContextTopContextExtractor, DUContextCacheVisitor> FilteredDUContextCacheVisitor;
+    
+      //The visitor visits all the declarations from within its constructor
+      DUContextCacheVisitor v(cache);
+      FilteredDUContextCacheVisitor visitor(v, contexts.iterator(), cachedImports);
+    }
+    
+    return FilteredDUContextIterator(Contexts::Iterator(cache.constData(), cache.size(), -1), cachedImports, true);
+  }else{
+    return FilteredDUContextIterator(contexts.iterator(), cachedImports);
+  }
+}
+
 PersistentSymbolTable::Declarations PersistentSymbolTable::getDeclarations(const IndexedQualifiedIdentifier& id) const {
+  QMutexLocker lock(d->m_declarations.mutex());
+  ENSURE_CHAIN_READ_LOCKED
+  
   PersistentSymbolTableItem item;
   item.id = id;
   PersistentSymbolTableRequestItem request(item);
@@ -285,6 +453,9 @@ PersistentSymbolTable::Declarations PersistentSymbolTable::getDeclarations(const
 
 void PersistentSymbolTable::declarations(const IndexedQualifiedIdentifier& id, uint& countTarget, const IndexedDeclaration*& declarationsTarget) const
 {
+  QMutexLocker lock(d->m_declarations.mutex());
+  ENSURE_CHAIN_READ_LOCKED
+  
   PersistentSymbolTableItem item;
   item.id = id;
   PersistentSymbolTableRequestItem request(item);
@@ -303,6 +474,11 @@ void PersistentSymbolTable::declarations(const IndexedQualifiedIdentifier& id, u
 
 void PersistentSymbolTable::addContext(const IndexedQualifiedIdentifier& id, const IndexedDUContext& context)
 {
+  QMutexLocker lock(d->m_contexts.mutex());
+  ENSURE_CHAIN_WRITE_LOCKED
+  
+  d->m_contextsCache.remove(id);
+  
   PersistentContextTableItem item;
   item.id = id;
   PersistentContextTableRequestItem request(item);
@@ -318,7 +494,9 @@ void PersistentSymbolTable::addContext(const IndexedQualifiedIdentifier& id, con
     if(alg.indexOf(context) != -1)
       return;
     
-    PersistentContextTableItem* editableItem = d->m_contexts.dynamicItemFromIndex(index);
+    QMutexLocker lock(d->m_contexts.mutex());
+    
+    DynamicItem<PersistentContextTableItem, true> editableItem = d->m_contexts.dynamicItemFromIndex(index);
     
     EmbeddedTreeAddItem<IndexedDUContext, IndexedDUContextHandler> add(const_cast<IndexedDUContext*>(editableItem->contexts()), editableItem->contextsSize(), editableItem->centralFreeItem, context);
     
@@ -344,6 +522,11 @@ void PersistentSymbolTable::addContext(const IndexedQualifiedIdentifier& id, con
 
 void PersistentSymbolTable::removeContext(const IndexedQualifiedIdentifier& id, const IndexedDUContext& context)
 {
+  QMutexLocker lock(d->m_contexts.mutex());
+  ENSURE_CHAIN_WRITE_LOCKED
+  
+  d->m_contextsCache.remove(id);
+  
   PersistentContextTableItem item;
   item.id = id;
   PersistentContextTableRequestItem request(item);
@@ -359,7 +542,9 @@ void PersistentSymbolTable::removeContext(const IndexedQualifiedIdentifier& id,
     if(alg.indexOf(context) == -1)
       return;
     
-    PersistentContextTableItem* editableItem = d->m_contexts.dynamicItemFromIndex(index);
+    QMutexLocker lock(d->m_contexts.mutex());
+    
+    DynamicItem<PersistentContextTableItem, true> editableItem = d->m_contexts.dynamicItemFromIndex(index);
     
     EmbeddedTreeRemoveItem<IndexedDUContext, IndexedDUContextHandler> remove(const_cast<IndexedDUContext*>(editableItem->contexts()), editableItem->contextsSize(), editableItem->centralFreeItem, context);
     
@@ -383,6 +568,9 @@ void PersistentSymbolTable::removeContext(const IndexedQualifiedIdentifier& id,
 }
 
 PersistentSymbolTable::Contexts PersistentSymbolTable::getContexts(const IndexedQualifiedIdentifier& id) const {
+  QMutexLocker lock(d->m_contexts.mutex());
+  ENSURE_CHAIN_READ_LOCKED
+
   PersistentContextTableItem item;
   item.id = id;
   PersistentContextTableRequestItem request(item);
@@ -398,6 +586,9 @@ PersistentSymbolTable::Contexts PersistentSymbolTable::getContexts(const Indexed
 }
 
 void PersistentSymbolTable::contexts(const IndexedQualifiedIdentifier& id, uint& countTarget, const IndexedDUContext*& contextsTarget) const {
+  QMutexLocker lock(d->m_contexts.mutex());
+  ENSURE_CHAIN_READ_LOCKED
+
   PersistentContextTableItem item;
   item.id = id;
   PersistentContextTableRequestItem request(item);
@@ -466,6 +657,8 @@ struct ContextVisitor {
 
 void PersistentSymbolTable::selfAnalysis() {
   {
+    QMutexLocker lock(d->m_declarations.mutex());
+    
     Visitor v;
     kDebug() << d->m_declarations.statistics();
     d->m_declarations.visitAllItems(v);
@@ -473,6 +666,8 @@ void PersistentSymbolTable::selfAnalysis() {
   }
 
   {
+    QMutexLocker lock(d->m_contexts.mutex());
+    
     ContextVisitor v;
     kDebug() << d->m_contexts.statistics();
     d->m_contexts.visitAllItems(v);
