@@ -35,7 +35,7 @@ namespace KDevelop {
 
 uint staticItemRepositoryVersion() {
   //Increase this to reset incompatible item-repositories
-  return 49;
+  return 58;
 }
 
 AbstractItemRepository::~AbstractItemRepository() {
@@ -139,7 +139,8 @@ void ItemRepositoryRegistry::registerRepository(AbstractItemRepository* reposito
 }
 
 QString ItemRepositoryRegistry::path() const {
-  QMutexLocker lock(&m_mutex);
+  //We cannot lock the mutex here, since this may be called with one of the repositories locked,
+  //and that may lead to a deadlock when at the same time a storing is requested
   return m_path;
 }
 
@@ -200,10 +201,18 @@ void ItemRepositoryRegistry::deleteDataDirectory() {
   QMutexLocker lock(&m_mutex);
   QFileInfo pathInfo(m_path);
   QDir d(m_path);
+
+  //lockForWriting creates a file, that prevents any other KDevelop instance from using the directory as it is.
+  //Instead, the other instance will try to delete the directory as well.
+  lockForWriting();
+  
+  // Have to release the lock here, else it will delete the new lock and windows needs all file-handles
+  // to be released before deleting a directory that contains these files
+  
+  m_lock->unlock();
   bool result = removeDirectory(d);
   Q_ASSERT(result);
   Q_ASSERT(m_lock);
-  m_lock->unlock(); //Have to release the lock here, else it will delete the new lock
   //Just remove the old directory, and allocate a new one. Probably it'll be the same one.
   QPair<QString, KLockFile::Ptr> repo = allocateRepository();
   m_path = repo.first;
@@ -303,6 +312,27 @@ void ItemRepositoryRegistry::store() {
   }
 }
 
+void ItemRepositoryRegistry::printAllStatistics() const {
+  QMutexLocker lock(&m_mutex);
+  foreach(AbstractItemRepository* repository, m_repositories.keys()) {
+    kDebug() << "statistics in" << repository->repositoryName() << ":";
+    kDebug() << repository->printStatistics();
+  }
+}
+
+int ItemRepositoryRegistry::finalCleanup() {
+  QMutexLocker lock(&m_mutex);
+  int changed = false;
+  foreach(AbstractItemRepository* repository, m_repositories.keys()) {
+    int added = repository->finalCleanup();
+    changed += added;
+    kDebug() << "cleaned in" << repository->repositoryName() << ":" << added;
+  }
+  
+  return changed;
+}
+
+
 void ItemRepositoryRegistry::close() {
 
   QMutexLocker lock(&m_mutex);
