@@ -1,3 +1,4 @@
+class QFile;
 /* This  is part of KDevelop
     Copyright 2008 David Nolden <david.nolden.kdevelop@art-master.de>
 
@@ -31,6 +32,7 @@ class DUContext;
 class Declaration;
 class IndexedString;
 class IndexedDUContext;
+class DUChainBaseData;
 
 typedef QPair<QByteArray, uint> ArrayWithPosition;
 
@@ -39,6 +41,8 @@ class TopDUContextDynamicData {
   public:
   TopDUContextDynamicData(TopDUContext* topContext);
   ~TopDUContextDynamicData();
+
+  void clearContextsAndDeclartions();
   
   /**
    * Allocates an index for the given declaration in this top-context.
@@ -95,6 +99,9 @@ class TopDUContextDynamicData {
   bool m_deleting; ///Flag used during destruction
   
   private:
+    void unmap();
+    //Converts away from an mmap opened file to a data array
+    
     QString filePath() const;
 
     void loadData() const;
@@ -108,10 +115,12 @@ class TopDUContextDynamicData {
     };
     
     static void verifyDataInfo(const ItemDataInfo& info, const QList<ArrayWithPosition>& data);
-    
-    static const char* pointerInData(const QList<ArrayWithPosition>& data, uint totalOffset);
 
-    ItemDataInfo writeDataInfo(const ItemDataInfo& info, const QList<ArrayWithPosition>& oldData, uint& totalDataOffset);
+    static const char* pointerInData(const QList<ArrayWithPosition>& data, uint totalOffset);
+    
+    const char* pointerInData(uint offset) const;
+
+    ItemDataInfo writeDataInfo(const ItemDataInfo& info, const DUChainBaseData* data, uint& totalDataOffset);
 
     TopDUContext* m_topContext;
     //May contain zero contexts if they were deleted
@@ -137,6 +146,11 @@ class TopDUContextDynamicData {
     mutable QList<ArrayWithPosition> m_topContextData;
     bool m_onDisk;
     mutable bool m_dataLoaded;
+
+    mutable QFile* m_mappedFile;
+    mutable uchar* m_mappedData;
+    mutable size_t m_mappedDataSize;
+    mutable bool m_itemRetrievalForbidden;
 };
 }
 
