@@ -19,6 +19,7 @@ Boston, MA 02110-1301, USA.
 
 #include "iproblem.h"
 #include <duchain/duchainregister.h>
+#include <interfaces/iassistant.h>
 
 namespace KDevelop {
 REGISTER_DUCHAIN_ITEM(Problem);
@@ -35,6 +36,9 @@ Problem::Problem()
 KDevelop::Problem::Problem(KDevelop::ProblemData& data) : DUChainBase(data) {
 }
 
+KDevelop::Problem::~Problem() {
+}
+
 KDevelop::IndexedString KDevelop::Problem::url() const
 {
     return d_func()->url;
@@ -79,7 +83,7 @@ QString Problem::description() const
 
 void Problem::setDescription(const QString & description)
 {
-    d_func_dynamic()->description = ReferenceCountedIndexedString(description);
+    d_func_dynamic()->description = IndexedString(description);
 }
 
 QString Problem::explanation() const
@@ -89,7 +93,7 @@ QString Problem::explanation() const
 
 void Problem::setExplanation(const QString & explanation)
 {
-    d_func_dynamic()->explanation = ReferenceCountedIndexedString(explanation);
+    d_func_dynamic()->explanation = IndexedString(explanation);
 }
 
 ProblemData::Source Problem::source() const
@@ -102,6 +106,18 @@ void Problem::setSource(ProblemData::Source source)
     d_func_dynamic()->source = source;
 }
 
+KSharedPtr< KDevelop::IAssistant > KDevelop::Problem::solutionAssistant() const {
+    return m_solution;
+}
+
+void KDevelop::Problem::setSolutionAssistant(KSharedPtr< KDevelop::IAssistant > assistant) {
+    m_solution = assistant;
+}
+
+KDevelop::Problem::Severity KDevelop::Problem::severity() const {
+    return Error;
+}
+
 /*
   public QSharedData
   QSharedDataPointer<Data> d;
