@@ -29,11 +29,11 @@ Boston, MA 02110-1301, USA.
 #include "../editor/documentrange.h"
 #include "../languageexport.h"
 #include "../duchain/duchainbase.h"
-#include "../duchain/referencecountedindexedstring.h"
 #include <language/duchain/indexedstring.h>
 
 namespace KDevelop
 {
+class IAssistant;
 
 class ProblemData : public DUChainBaseData
 {
@@ -44,7 +44,8 @@ public:
         Preprocessor   /**< problem during pre-processing */,
         Lexer          /**< problem while lexing the file */,
         Parser         /**< problem while parsing the file */,
-        DUChainBuilder /**< problem while building the duchain */
+        DUChainBuilder /**< problem while building the duchain */,
+        SemanticAnalysis /**< problem during semantic analysis */
     };
 
     ProblemData()
@@ -55,8 +56,8 @@ public:
     Source source;
     KDevelop::IndexedString url;
 //     QStack<DocumentCursor> locationStack;
-    ReferenceCountedIndexedString description;
-    ReferenceCountedIndexedString explanation;
+    IndexedString description;
+    IndexedString explanation;
 };
 
 /**
@@ -72,6 +73,7 @@ class KDEVPLATFORMLANGUAGE_EXPORT Problem : public DUChainBase, public KShared
 public:
     Problem();
     Problem(ProblemData& data);
+    ~Problem();
 
     ProblemData::Source source() const;
     void setSource(ProblemData::Source source);
@@ -105,6 +107,28 @@ public:
     QString explanation() const;
     void setExplanation(const QString& explanation);
 
+    enum Severity {
+        Error,
+        Warning,
+        Hint        //For implementation-helpers and such stuff
+    };
+    
+    /**
+     * Should return the severity of this problem. The default-implementation returns Error.
+     * This is used for example to decide for a highlighting color.
+     */
+    virtual Severity severity() const;
+
+     /**
+     * If this problem can be solved, this may return an assistant for the solution.
+     */
+    virtual KSharedPtr<IAssistant> solutionAssistant() const;
+
+    /**
+     * Set an assistant for solving this problem
+     */
+    void setSolutionAssistant(KSharedPtr<IAssistant> assistant);
+    
     enum {
         Identity = 15
     };
@@ -112,6 +136,7 @@ public:
     private:
     Problem(const Problem& other);
     Problem& operator=(const Problem& rhs);
+    KSharedPtr< KDevelop::IAssistant > m_solution;
     
     DUCHAIN_DECLARE_DATA(Problem)
 };
