@@ -40,12 +40,15 @@
 #include <interfaces/ilanguage.h>
 #include <interfaces/iuicontroller.h>
 #include <interfaces/ilanguagecontroller.h>
+#include <interfaces/contextmenuextension.h>
+#include <language/interfaces/codecontext.h>
 #include <language/duchain/duchainlock.h>
 #include <language/duchain/duchain.h>
 #include <language/duchain/ducontext.h>
 #include <language/duchain/declaration.h>
 #include <language/duchain/use.h>
 #include <language/duchain/duchainutils.h>
+#include <language/duchain/dumpchain.h>
 #include <language/duchain/functiondefinition.h>
 #include <language/duchain/parsingenvironment.h>
 #include <language/interfaces/ilanguagesupport.h>
@@ -142,6 +145,9 @@ ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
   nextUse->setText( i18n("&Next Use") );
   nextUse->setShortcut( Qt::META | Qt::SHIFT | Qt::Key_Right );
   connect(nextUse, SIGNAL(triggered(bool)), this, SLOT(nextUseShortcut()));
+  
+  m_findUses = new QAction(i18n("Find Uses"), this);
+  connect(m_findUses, SIGNAL(triggered(bool)), this, SLOT(findUses()));
 }
 
 ContextBrowserPlugin::~ContextBrowserPlugin()
@@ -152,6 +158,10 @@ ContextBrowserPlugin::~ContextBrowserPlugin()
 
 void ContextBrowserPlugin::watchRange(KTextEditor::SmartRange* range)
 {
+  KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(range->document());
+  Q_ASSERT(smart);
+  QMutexLocker lock(smart->smartMutex());
+
   if (range->watchers().contains( this ))
     return;
 
@@ -198,6 +208,49 @@ void ContextBrowserPlugin::mouseExitedRange( KTextEditor::SmartRange* /*range*/,
   m_updateTimer->start(1); // triggers updateViews()
 }
 
+KDevelop::ContextMenuExtension ContextBrowserPlugin::contextMenuExtension(KDevelop::Context* context)
+{
+  KDevelop::ContextMenuExtension menuExt = KDevelop::IPlugin::contextMenuExtension( context );
+
+  KDevelop::DeclarationContext *codeContext = dynamic_cast<KDevelop::DeclarationContext*>(context);
+
+  if (!codeContext)
+      return menuExt;
+
+  DUChainReadLocker lock(DUChain::lock());
+  
+  if(!codeContext->declaration().data())
+    return menuExt;
+  
+  qRegisterMetaType<KDevelop::IndexedDeclaration>("KDevelop::IndexedDeclaration");
+  
+  m_findUses->setData(QVariant::fromValue(codeContext->declaration()));
+  menuExt.addAction(KDevelop::ContextMenuExtension::ExtensionGroup, m_findUses);
+
+  return menuExt;
+}
+
+void ContextBrowserPlugin::findUses()
+{
+  QAction* action = qobject_cast<QAction*>(sender());
+  Q_ASSERT(action);
+  DUChainReadLocker lock(DUChain::lock());
+  
+  KDevelop::IndexedDeclaration decl = action->data().value<KDevelop::IndexedDeclaration>();
+  if(!decl.data())
+    return;
+  QWidget* widget = ICore::self()->uiController()->findToolView(i18n("Code Browser"), m_viewFactory, KDevelop::IUiController::CreateAndRaise);
+  if(!widget)
+    return;
+  ContextBrowserView* view = dynamic_cast<ContextBrowserView*>(widget);
+  Q_ASSERT(view);
+  view->allowLockedUpdate();
+  view->setDeclaration(decl.data(), decl.data()->topContext(), true);
+  KDevelop::AbstractNavigationWidget* navigationWidget = dynamic_cast<KDevelop::AbstractNavigationWidget*>(view->navigationWidget());
+  if(navigationWidget)
+    navigationWidget->executeContextAction("show_uses");
+}
+
 void ContextBrowserPlugin::textHintRequested(const KTextEditor::Cursor& cursor, QString&) {
   m_mouseHoverCursor = SimpleCursor(cursor);
   View* view = dynamic_cast<View*>(sender());
@@ -214,8 +267,10 @@ void ContextBrowserPlugin::textHintRequested(const KTextEditor::Cursor& cursor,
 }
 
 void ContextBrowserPlugin::stopDelayedBrowsing() {
-  if(m_currentToolTip)
-    delete m_currentToolTip;
+  if(m_currentToolTip) {
+    m_currentToolTip->deleteLater();
+    m_currentToolTip = 0;
+  }
 }
 
 void ContextBrowserPlugin::startDelayedBrowsing(KTextEditor::View* view) {
@@ -224,6 +279,14 @@ void ContextBrowserPlugin::startDelayedBrowsing(KTextEditor::View* view) {
   }
 }
 
+void ContextBrowserPlugin::hideTooTip() {
+  if(m_currentToolTip) {
+    m_currentToolTip->deleteLater();
+    m_currentToolTip = 0;
+  }
+}
+
+
 void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cursor position) {
   KUrl viewUrl(view->document()->url());
   QList<ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(viewUrl);
@@ -244,7 +307,7 @@ void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cur
         if(m_currentToolTipDeclaration == IndexedDeclaration(decl) && m_currentToolTip)
           return;
         m_currentToolTipDeclaration = IndexedDeclaration(decl);
-        navigationWidget = decl->context()->createNavigationWidget(decl);
+        navigationWidget = decl->context()->createNavigationWidget(decl, DUChainUtils::standardContextForUrl(viewUrl));
       }
     }
   }
@@ -261,33 +324,57 @@ void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cur
         contextView->setNavigationWidget(navigationWidget);
       }
       
-    if(m_currentToolTip)
-      delete m_currentToolTip;
+    if(m_currentToolTip) {
+      m_currentToolTip->deleteLater();
+      m_currentToolTip = 0;
+    }
     
     KDevelop::NavigationToolTip* tooltip = new KDevelop::NavigationToolTip(view, view->mapToGlobal(view->cursorToCoordinate(position)) + QPoint(20, 40), navigationWidget);
-    tooltip->resize( 580, 230 );
-    kDebug() << tooltip->size();
+    tooltip->resize( navigationWidget->sizeHint() + QSize(10, 10) );
+    kDebug() << "tooltip size" << tooltip->size();
     m_currentToolTip = tooltip;
+    ActiveToolTip::showToolTip(tooltip);
+
+    //First disconnect to prevent multiple connections
+    disconnect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)), this, SLOT(hideTooTip()));
+    disconnect(view, SIGNAL(focusOut(KTextEditor::View*)), this, SLOT(hideTooTip()));
+    connect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)), this, SLOT(hideTooTip()));
+    connect(view, SIGNAL(focusOut(KTextEditor::View*)), this, SLOT(hideTooTip()));
+    
   }else{
     kDebug() << "not showing tooltip, no navigation-widget";
   }
+  
 }
 
 
 void ContextBrowserPlugin::clearMouseHover() {
   m_mouseHoverCursor = SimpleCursor::invalid();
-  m_mouseHoverDocument = KUrl();
+  m_mouseHoverDocument.clear();
 }
 
 
-Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/) {
-  static Attribute::Ptr standardAttribute = Attribute::Ptr();
-  if( !standardAttribute ) {
-    standardAttribute = Attribute::Ptr( new Attribute() );
-    standardAttribute->setBackgroundFillWhitespace(true);
-    standardAttribute->setBackground(Qt::yellow);//QApplication::palette().toolTipBase());
+Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/, bool bold) {
+  if(bold) {
+    static Attribute::Ptr standardBoldAttribute = Attribute::Ptr();
+    if( !standardBoldAttribute ) {
+      standardBoldAttribute= Attribute::Ptr( new Attribute() );
+      standardBoldAttribute->setBackgroundFillWhitespace(true);
+      standardBoldAttribute->setBackground(Qt::yellow);//QApplication::palette().toolTipBase());
+      standardBoldAttribute->setForeground(Qt::black); 
+      standardBoldAttribute->setFontBold(true);
+    }
+    return standardBoldAttribute;
+  }else{
+    static Attribute::Ptr standardAttribute = Attribute::Ptr();
+    if( !standardAttribute ) {
+      standardAttribute = Attribute::Ptr( new Attribute() );
+      standardAttribute->setBackgroundFillWhitespace(true);
+      standardAttribute->setBackground(Qt::yellow);//QApplication::palette().toolTipBase());
+      standardAttribute->setForeground(Qt::black); 
+    }
+    return standardAttribute;
   }
-  return standardAttribute;
 }
 
 Attribute::Ptr highlightedDeclarationAttribute() {
@@ -316,25 +403,27 @@ void ContextBrowserPlugin::changeHighlight( KTextEditor::SmartRange* range, bool
   if( !range )
     return;
 
-  Attribute::Ptr attrib;
   if( highlight ) {
-    if( !m_backups.contains(range) ) {
-      m_backups[range] = range->attribute();
-      watchRange(range);
-    }
 /*    if( declaration )
       attrib = highlightedDeclarationAttribute();
     else*/
-      attrib = highlightedUseAttribute(mouseHighlight);
+
+    Attribute::Ptr attrib = highlightedUseAttribute(mouseHighlight, range->attribute() ? range->attribute()->fontBold() : false);
+    
+    if( !m_backups.contains(range) ) {
+      m_backups[range] = qMakePair(attrib, range->attribute());
+      watchRange(range);
+    }
+    range->setAttribute(attrib);
   }else{
-    if( m_backups.contains(range) ) {
-      attrib = m_backups[range];
+    //Reset old highlighting, if our set attribute is still on the range
+    if( m_backups.contains(range) && m_backups[range].first == range->attribute() ) {
+      range->setAttribute(m_backups[range].second);
       m_backups.remove(range);
     }
     ignoreRange(range);
   }
 
-  range->setAttribute(attrib);
 }
 
 void ContextBrowserPlugin::changeHighlight( View* view, KDevelop::Declaration* decl, bool highlight, bool mouseHighlight ) {
@@ -515,7 +604,6 @@ void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
           addedWidget = foundSpecialObject ?
             showSpecialObjectView(view, position, pickedLanguage, ctx) :
             showDeclarationView(view, position, foundDeclaration, ctx);
-          ///@todo show tooltip
       }
     }
 
@@ -545,20 +633,34 @@ void ContextBrowserPlugin::declarationSelectedInUI(DeclarationPointer decl)
 
 void ContextBrowserPlugin::parseJobFinished(KDevelop::ParseJob* job)
 {
-  KDevelop::DUChainWriteLocker lock( DUChain::lock() );
-  registerAsRangeWatcher(job->duChain());
+  if(job->duChain() && job->duChain()->smartRange()) {
+    KDevelop::DUChainReadLocker lock( DUChain::lock() );
+    registerAsRangeWatcher(job->duChain());
+  }
 }
 
 void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUChainBase* base)
 {
-  if(base->smartRange())
+  if(base->smartRange()) {
     watchRange(base->smartRange());
 }
+}
 
 void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUContext* ctx)
 {
-  if(!ctx)
+  if(!ctx || !ctx->smartRange())
     return;
+
+  QMutex* lockSmartMutex = 0;
+
+  if(!ctx->parentContext()) {
+    KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(ctx->smartRange()->document());
+    Q_ASSERT(smart);
+    lockSmartMutex = smart->smartMutex();
+  }
+  
+  QMutexLocker lock(lockSmartMutex);
+  
   if(dynamic_cast<TopDUContext*>(ctx) && static_cast<TopDUContext*>(ctx)->parsingEnvironmentFile() && static_cast<TopDUContext*>(ctx)->parsingEnvironmentFile()->isProxyContext() && !ctx->importedParentContexts().isEmpty())
     return registerAsRangeWatcher(ctx->importedParentContexts()[0].context(0));
 
@@ -585,7 +687,7 @@ void ContextBrowserPlugin::textDocumentCreated( KDevelop::IDocument* document )
   foreach( View* view, document->textDocument()->views() )
     viewCreated( document->textDocument(), view );
 
-  KDevelop::DUChainWriteLocker lock( DUChain::lock() );
+  KDevelop::DUChainReadLocker lock( DUChain::lock() );
   QList<TopDUContext*> chains = DUChain::self()->chainsForDocument( document->url() );
 
   foreach( TopDUContext* chain, chains )
@@ -643,6 +745,10 @@ void ContextBrowserPlugin::nextUseShortcut()
   switchUse(true);
 }
 
+KTextEditor::Range cursorToRange(SimpleCursor cursor) {
+  return KTextEditor::Range(cursor.textCursor(), cursor.textCursor());
+}
+
 void ContextBrowserPlugin::switchUse(bool forward)
 {
   if(core()->documentController()->activeDocument() && core()->documentController()->activeDocument()->textDocument() && core()->documentController()->activeDocument()->textDocument()->activeView()) {
@@ -666,6 +772,28 @@ void ContextBrowserPlugin::switchUse(bool forward)
       if(!decl) //Try finding a declaration under the cursor
         decl = DUChainUtils::itemUnderCursor(doc->url(), c);
       
+      if(decl) {
+        
+        Declaration* target = 0;
+        if(forward)
+          //Try jumping from definition to declaration
+          target = DUChainUtils::declarationForDefinition(decl, chosen);
+        else if(decl->url().toUrl() == doc->url() && decl->range().contains(c))
+          //Try jumping from declaration to definition
+          target = FunctionDefinition::definition(decl);
+        
+          if(target && target != decl) {
+            SimpleCursor jumpTo = target->range().start;
+            KUrl document = target->url().toUrl();
+            lock.unlock();
+            core()->documentController()->openDocument( document, cursorToRange(jumpTo)  );
+            return;
+          }else{
+            //Always work with the declaration instead of the definition
+            decl = DUChainUtils::declarationForDefinition(decl, chosen);
+          }
+      }
+      
       if(!decl) {
         //Pick the last use we have highlighted
         decl = m_lastHighlightedDeclaration.data();
@@ -685,9 +813,10 @@ void ContextBrowserPlugin::switchUse(bool forward)
               QList<SimpleRange> useRanges = allUses(top, decl, true);
               qSort(useRanges);
               if(!useRanges.isEmpty()) {
+                KUrl url = top->url().toUrl();
                 SimpleRange selectUse = forward ? useRanges.first() : useRanges.back();
                 lock.unlock();
-                core()->documentController()->openDocument(KUrl(top->url().str()), KTextEditor::Range(selectUse.start.textCursor(), selectUse.start.textCursor()));
+                core()->documentController()->openDocument(url, cursorToRange(selectUse.start));
               }
             }
           }
@@ -750,7 +879,12 @@ void ContextBrowserPlugin::switchUse(bool forward)
               kDebug() << "current file" << indexInFiles << "nextFile" << nextFile;
               
               if(nextFile < 0 || nextFile >= usingFiles.size()) {
-                //Open the declaration
+                //Open the declaration, or the definition
+                if(nextFile >= usingFiles.size()) {
+                  Declaration* definition = FunctionDefinition::definition(decl);
+                  if(definition)
+                    decl = definition;
+                }
                 KUrl u(decl->url().str());
                 SimpleRange range = decl->range();
                 range.end = range.start;
