@@ -30,7 +30,6 @@
 #include <ktexteditor/smartrange.h>
 #include <ktexteditor/rangefeedback.h>
 #include <interfaces/iplugin.h>
-#include <interfaces/irunprovider.h>
 #include <language/duchain/duchainpointer.h>
 #include <language/editor/simplecursor.h>
 #include <language/editor/simplerange.h>
@@ -65,6 +64,8 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
 
     void registerToolView(ContextBrowserView* view);
     void unRegisterToolView(ContextBrowserView* view);
+    
+    virtual KDevelop::ContextMenuExtension contextMenuExtension(KDevelop::Context*);
 
   Q_SIGNALS:
     void previousContextShortcut();
@@ -90,6 +91,9 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
 
     void textHintRequested(const KTextEditor::Cursor&, QString&);
 
+    void hideTooTip();
+    void findUses();
+    
   private:
     void switchUse(bool forward);
     void clearMouseHover();
@@ -119,7 +123,8 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
   private:
     void showToolTip(KTextEditor::View* view, KTextEditor::Cursor position);
     QTimer* m_updateTimer;
-    QMap<KTextEditor::SmartRange*, KTextEditor::Attribute::Ptr> m_backups;
+    //Contains the range, the old attribute, and the attribute it was replaced with
+    QMap<KTextEditor::SmartRange*, QPair<KTextEditor::Attribute::Ptr,KTextEditor::Attribute::Ptr> > m_backups;
     QSet<KTextEditor::View*> m_updateViews;
     QMap<KTextEditor::View*, DeclarationPointer> m_highlightedDeclarations;
     QMap<KTextEditor::View*, KTextEditor::SmartRange*> m_highlightedRange; //Special language-object range
@@ -138,6 +143,7 @@ class ContextBrowserPlugin : public KDevelop::IPlugin, public KTextEditor::Smart
     ContextBrowserViewFactory* m_viewFactory;
     QPointer<QWidget> m_currentToolTip;
     IndexedDeclaration m_currentToolTipDeclaration;
+    QAction* m_findUses;
 };
 
 DUContext* contextAt(const SimpleCursor& position, TopDUContext* topContext);
