@@ -43,7 +43,7 @@ ProblemHighlighter::~ProblemHighlighter()
         return;
 
     KDevelop::EditorIntegrator editor;
-    editor.setCurrentUrl(IndexedString(m_document->url().pathOrUrl()), true);
+    editor.setCurrentUrl(IndexedString(m_document->url()), true);
 
     LockedSmartInterface iface = editor.smart();
     if (iface)
@@ -53,7 +53,7 @@ ProblemHighlighter::~ProblemHighlighter()
 void ProblemHighlighter::setProblems(const QList<KDevelop::ProblemPointer>& problems)
 {
     KDevelop::EditorIntegrator editor;
-    IndexedString url(m_document->url().pathOrUrl());
+    IndexedString url(m_document->url());
     editor.setCurrentUrl(url, true);
 
     LockedSmartInterface iface = editor.smart();
@@ -78,7 +78,13 @@ void ProblemHighlighter::setProblems(const QList<KDevelop::ProblemPointer>& prob
             problemRange->smartEnd().advance(1);
 
         KTextEditor::Attribute::Ptr error(new KTextEditor::Attribute());
-        error->setUnderlineColor(Qt::red);
+        if(problem->severity() == Problem::Error)
+            error->setUnderlineColor(Qt::red);
+        else if(problem->severity() == Problem::Warning)
+            error->setUnderlineColor(Qt::magenta);
+        else
+            error->setUnderlineColor(Qt::yellow);
+            
         error->setUnderlineStyle(QTextCharFormat::WaveUnderline);
 
         /*KTextEditor::Attribute::Ptr dyn(new KTextEditor::Attribute());
