@@ -53,6 +53,8 @@
 
 #include "problemreporterplugin.h"
 #include "problemmodel.h"
+#include <kaction.h>
+#include <interfaces/iassistant.h>
 
 //#include "modeltest.h"
 
@@ -69,6 +71,15 @@ ProblemWidget::ProblemWidget(QWidget* parent, ProblemReporterPlugin* plugin)
     setWhatsThis( i18n( "Problems" ) );
     setModel(new ProblemModel(m_plugin));
 
+//     setContextMenuPolicy(Qt::CustomContextMenu);
+    
+    m_fullUpdateAction = new KAction(this);
+    m_fullUpdateAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
+    m_fullUpdateAction->setText(i18n("Force Full Update"));
+    m_fullUpdateAction->setToolTip(i18n("Re-parse the current file and all its imports."));
+    m_fullUpdateAction->setIcon(KIcon("dirsync"));
+    connect(m_fullUpdateAction, SIGNAL(triggered(bool)), this, SLOT(forceFullUpdate()));
+    addAction(m_fullUpdateAction);
     //new ModelTest(model());
 
     connect(this, SIGNAL(activated(const QModelIndex&)), SLOT(itemActivated(const QModelIndex&)));
@@ -84,14 +95,21 @@ ProblemWidget::~ProblemWidget()
 
 void ProblemWidget::collectProblems(QList<ProblemPointer>& allProblems, TopDUContext* context, QSet<TopDUContext*>& hadContexts)
 {
+  if(!context) {
+      kDebug() << "collecting from bad context";
+      return;
+  }
   if(hadContexts.contains(context))
     return;
 
   hadContexts.insert(context);
 
   allProblems += context->problems();
+
   bool isProxy = context->parsingEnvironmentFile() && context->parsingEnvironmentFile()->isProxyContext();
   foreach(const DUContext::Import &ctx, context->importedParentContexts()) {
+      if(!ctx.indexedContext().indexedTopContext().isLoaded())
+          continue;
     TopDUContext* topCtx = dynamic_cast<TopDUContext*>(ctx.context(0));
     if(topCtx) {
       //If we are starting at a proxy-context, only recurse into other proxy-contexts,
@@ -102,6 +120,12 @@ void ProblemWidget::collectProblems(QList<ProblemPointer>& allProblems, TopDUCon
   }
 }
 
+void ProblemWidget::forceFullUpdate() {
+    kDebug() << "forcing full update";
+    DUChainReadLocker lock(DUChain::lock());
+    DUChain::self()->updateContextForUrl(IndexedString(m_activeUrl), KDevelop::TopDUContext::ForceUpdateRecursive);
+}
+
 void ProblemWidget::showProblems(TopDUContext* ctx)
 {
   if(ctx) {
@@ -109,8 +133,9 @@ void ProblemWidget::showProblems(TopDUContext* ctx)
     QSet<TopDUContext*> hadContexts;
     DUChainReadLocker lock(DUChain::lock());
     collectProblems(allProblems, ctx, hadContexts);
-    model()->setProblems(allProblems);
-    resizeColumnToContents(0);
+    model()->setProblems(allProblems, m_activeDirectory);
+    for (int i = 0; i < model()->columnCount(); ++i)
+        resizeColumnToContents(i);
   }else{
     model()->clear();
   }
@@ -118,7 +143,8 @@ void ProblemWidget::showProblems(TopDUContext* ctx)
 
 void ProblemWidget::documentActivated(KDevelop::IDocument* doc)
 {
-  kDebug() << "activated document:" << doc->url();
+  m_activeDirectory = doc->url().upUrl();
+  m_activeUrl = doc->url();
 
   QList<KDevelop::ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(doc->url());
 
@@ -137,13 +163,9 @@ void ProblemWidget::parseJobFinished(KDevelop::ParseJob* job)
   IDocument* active = ICore::self()->documentController()->activeDocument();
 
   if(active) {
-    kDebug() << "active document:" << active->url() << "url:" << url;
     //For now, only show problems from the current document
     if(active->url() == url && job->duChain()) {
       showProblems(job->duChain());
-    }else{
-      //Clear all problems
-      showProblems(0);
     }
   }
 }
@@ -176,6 +198,23 @@ ProblemModel * ProblemWidget::model() const
     return static_cast<ProblemModel*>(QTreeView::model());
 }
 
+void ProblemWidget::contextMenuEvent(QContextMenuEvent* event) {
+    QModelIndex index = indexAt(event->pos());
+    if(index.isValid()) {
+        KDevelop::ProblemPointer problem = model()->problemForIndex(index);
+        if(problem) {
+            KSharedPtr<KDevelop::IAssistant> solution = problem->solutionAssistant();
+            QList<QAction*> actions;
+            if(solution) {
+                foreach(KDevelop::IAssistantAction::Ptr action, solution->actions())
+                    actions << action->toKAction();
+            }
+            if(!actions.isEmpty())
+                QMenu::exec(actions, event->globalPos());
+        }
+    }
+}
+
 void ProblemWidget::showEvent(QShowEvent * event)
 {
     Q_UNUSED(event)
