@@ -42,7 +42,7 @@ QString getRelativeFolder( KDevelop::ProjectBaseItem* item )
           || item->type() == KDevelop::ProjectBaseItem::BuildFolder )
     {
 
-        return item->project()->relativeUrl( item->folder()->url() ).path();
+        return item->project()->relativeUrl( item->folder()->url() ).toLocalFile();
     }else
     {
         return getRelativeFolder( dynamic_cast<KDevelop::ProjectBaseItem*>( item->parent() ) );
@@ -246,6 +246,19 @@ QList<BuildItem> ProjectBuildSetModel::items()
     return m_items ;
 }
 
+void ProjectBuildSetModel::projectClosed( KDevelop::IProject* project )
+{
+    for( int i = m_items.count() - 1; i >= 0; i-- )
+    {
+        if( m_items.at(i).projectName() == project->name() )
+        {
+            beginRemoveRows( QModelIndex(), i, i );
+            m_items.removeAt(i);
+            endRemoveRows();
+        }
+    }  
+}
+
 void ProjectBuildSetModel::saveToProject( KDevelop::IProject* project ) const
 {
     KConfigGroup base = project->projectConfiguration()->group("Buildset");
@@ -280,3 +293,49 @@ void ProjectBuildSetModel::loadFromProject( KDevelop::IProject* project )
         endInsertRows();
     }
 }
+
+void ProjectBuildSetModel::moveRowsDown(int row, int count)
+{
+    QList<BuildItem> items = m_items.mid( row, count );
+    removeRows( row, count );
+    beginInsertRows( QModelIndex(), row+1, row+count );
+    for( int i = 0; i < count; i++ )
+    {
+        m_items.insert( row+1+i, items.at( i ) ); 
+    }
+    endInsertRows();
+}
+
+void ProjectBuildSetModel::moveRowsToBottom(int row, int count)
+{
+    QList<BuildItem> items = m_items.mid( row, count );
+    removeRows( row, count );
+    beginInsertRows( QModelIndex(), rowCount(), rowCount()+count-1 );
+    m_items += items;
+    endInsertRows();
+}
+
+void ProjectBuildSetModel::moveRowsUp(int row, int count)
+{
+    QList<BuildItem> items = m_items.mid( row, count );
+    removeRows( row, count );
+    beginInsertRows( QModelIndex(), row-1, row-2+count );
+    for( int i = 0; i < count; i++ )
+    {
+        m_items.insert( row-1+i, items.at( i ) ); 
+    }
+    endInsertRows();
+}
+
+void ProjectBuildSetModel::moveRowsToTop(int row, int count)
+{
+    QList<BuildItem> items = m_items.mid( row, count );
+    removeRows( row, count );
+    beginInsertRows( QModelIndex(), 0, count-1 );
+    for( int i = 0; i < count; i++ )
+    {
+        m_items.insert( 0+i, items.at( i ) ); 
+    }
+    endInsertRows();
+}
+
