@@ -26,7 +26,6 @@
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QStandardItem>
-#include <QtGui/QStackedWidget>
 #include <QtGui/QToolButton>
 
 #include <kxmlguiwindow.h>
@@ -41,6 +40,8 @@
 #include <kcombobox.h>
 #include <kjob.h>
 
+#include <interfaces/iselectioncontroller.h>
+#include <interfaces/context.h>
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
 #include <interfaces/iuicontroller.h>
@@ -63,10 +64,12 @@ using namespace KDevelop;
 ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidget *parent )
         : QWidget( parent ), m_ui(new Ui::ProjectManagerView), m_plugin(plugin)
 {
-    setWindowTitle(i18n("Projects"));
-
     m_ui->setupUi( this );
 
+    setWindowTitle(i18n("Projects"));
+    setWindowIcon( SmallIcon( "kdevelop" ) ); //FIXME
+    setWhatsThis( i18n( "Project Manager" ) );
+
     m_syncAction = new KAction(this);
     m_syncAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
     m_syncAction->setText(i18n("Locate Current Document"));
@@ -76,15 +79,10 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
     addAction(m_syncAction);
     updateSyncAction();
 
-    m_buildAction = new KAction(this);
-    m_buildAction->setShortcutContext(Qt::WidgetWithChildrenShortcut);
-    m_buildAction->setText(i18n("Build Selected Items"));
-    m_buildAction->setToolTip(i18n("Builds the selected items in the tree."));
-    m_buildAction->setIcon(KIcon("run-build"));
-    connect(m_buildAction, SIGNAL(triggered(bool)), this, SLOT( buildSelectedItems() ));
-    addAction(m_buildAction);
-
-
+    addAction(plugin->actionCollection()->action("project_build"));
+    addAction(plugin->actionCollection()->action("project_install"));
+    addAction(plugin->actionCollection()->action("project_clean"));
+    
     m_ui->projectTreeView->setWhatsThis( i18n( "Project Overview" ) );
     QSizePolicy pol( QSizePolicy::Expanding, QSizePolicy::Expanding );
     pol.setVerticalStretch( 6 );
@@ -96,10 +94,6 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
 //     connect(d->m_filters, SIGNAL(returnPressed()), this, SLOT(filtersChanged()));
 //     vbox->addWidget( m_filters );
 
-    m_ui->hideDetailsButton->setIcon( KIcon( "arrow-down-double.png" ) );
-    connect( m_ui->hideDetailsButton, SIGNAL( clicked() ),
-             this, SLOT( switchDetailView() ) );
-
     pol = QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred );
     pol.setVerticalStretch( 2 );
     m_ui->buildSetView->setProjectView( this );
@@ -109,15 +103,10 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
     QStandardItemModel *overviewModel = ICore::self()->projectController()->projectModel();
     m_modelFilter = new ProjectProxyModel( this );
     m_modelFilter->setSourceModel(overviewModel);
-    m_modelFilter->setDynamicSortFilter(true);
+
     m_ui->projectTreeView->setModel( m_modelFilter );
-    m_ui->projectTreeView->setSortingEnabled(true);
-    m_ui->projectTreeView->sortByColumn( 0, Qt::AscendingOrder );
     m_ui->projectTreeView->setSelectionModel( new ProxySelectionModel( m_ui->projectTreeView, ICore::self()->projectController()->projectSelectionModel(), this ) );
-//     m_ui->projectTreeView->setModel( overviewModel );c
-    setWindowIcon( SmallIcon( "kdevelop" ) ); //FIXME
-    setWindowTitle( i18n( "Project Manager" ) );
-    setWhatsThis( i18n( "Project Manager" ) );
+ 
     connect( m_ui->projectTreeView->selectionModel(), SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection&) ),
              this, SLOT(selectionChanged() ) );
     connect( KDevelop::ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*) ),
@@ -130,17 +119,17 @@ ProjectManagerView::ProjectManagerView( ProjectManagerViewPlugin* plugin, QWidge
 void ProjectManagerView::selectionChanged()
 {
     m_ui->buildSetView->selectionChanged();
-    m_buildAction->setEnabled( !m_ui->projectTreeView->selectionModel()->selectedIndexes().isEmpty() );
-}
-
-void ProjectManagerView::buildSelectedItems()
-{
-    ICore::self()->runController()->registerJob( new BuilderJob(BuilderJob::Build, selectedItems() ) );
+    QList<ProjectBaseItem*> selected;
+    foreach( const QModelIndex& idx, m_ui->projectTreeView->selectionModel()->selectedRows() )
+    {
+        selected << m_modelFilter->itemFromProxyIndex( idx );
+    }
+    KDevelop::ICore::self()->selectionController()->updateSelection( new ProjectItemContext( selected ) );
 }
 
 void ProjectManagerView::updateSyncAction()
 {
-    m_syncAction->setEnabled( !KDevelop::ICore::self()->documentController()->openDocuments().isEmpty() );
+    m_syncAction->setEnabled( KDevelop::ICore::self()->documentController()->activeDocument() );
 }
 
 ProjectManagerView::~ProjectManagerView()
@@ -162,26 +151,14 @@ QList<KDevelop::ProjectBaseItem*> ProjectManagerView::selectedItems() const
     return items;
 }
 
-void ProjectManagerView::switchDetailView()
-{
-    KFadeWidgetEffect* animation = new KFadeWidgetEffect( this );
-    if( m_ui->buildSetView->isHidden() )
-    {
-        m_ui->hideDetailsButton->setIcon( KIcon( "arrow-down-double" ) );
-        m_ui->buildSetView->show();
-    }
-    else
-    {
-        m_ui->hideDetailsButton->setIcon( KIcon( "arrow-up-double" ) );
-        m_ui->buildSetView->hide();
-    }
-    animation->start();
-}
-
 void ProjectManagerView::locateCurrentDocument()
 {
     KDevelop::IDocument *doc = ICore::self()->documentController()->activeDocument();
 
+    // We should _never_ get a null pointer for the document, as
+    // the action is only enabled when there is an active document.
+    Q_ASSERT(doc); 
+   
     foreach (IProject* proj, ICore::self()->projectController()->projects()) {
         foreach (KDevelop::ProjectFileItem* item, proj->filesForUrl(doc->url())) {
             QModelIndex index = m_modelFilter->proxyIndexFromItem(item);
