@@ -29,8 +29,8 @@
 #include <kmessagebox.h>
 #include <kio/netaccess.h>
 #include <kparts/mainwindow.h>
-#include <ksavefile.h>
 #include <kparts/componentfactory.h>
+#include <KTemporaryFile>
 
 #include <project/projectmodel.h>
 #include <interfaces/icore.h>
@@ -42,9 +42,9 @@
 #include <interfaces/idocumentcontroller.h>
 #include <project/interfaces/iprojectbuilder.h>
 #include <interfaces/iprojectcontroller.h>
-#include <interfaces/irun.h>
 #include <interfaces/context.h>
 #include <interfaces/contextmenuextension.h>
+#include <interfaces/iselectioncontroller.h>
 
 #include "projectmanagerview.h"
 #include "projectbuildsetmodel.h"
@@ -103,6 +103,8 @@ ProjectManagerViewPlugin::ProjectManagerViewPlugin( QObject *parent, const QVari
 
     connect( core()->projectController(), SIGNAL( projectClosing( KDevelop::IProject* ) ),
              d->buildSet, SLOT( saveToProject( KDevelop::IProject* ) ) );
+    connect( core()->projectController(), SIGNAL( projectClosed( KDevelop::IProject* ) ),
+             d->buildSet, SLOT( projectClosed( KDevelop::IProject* ) ) );
 
     d->m_buildAll = new KAction( i18n("Build all Projects"), this );
     d->m_buildAll->setIcon(KIcon("run-build"));
@@ -111,25 +113,56 @@ ProjectManagerViewPlugin::ProjectManagerViewPlugin( QObject *parent, const QVari
     d->m_build = new KAction( i18n("Build"), this );
     d->m_build->setShortcut( Qt::Key_F8 );
     d->m_build->setIcon(KIcon("run-build"));
+    d->m_build->setEnabled( false );
     connect( d->m_build, SIGNAL(triggered()), this, SLOT(buildProjectItems()) );
     actionCollection()->addAction( "project_build", d->m_build );
     d->m_install = new KAction( i18n("Install"), this );
+    d->m_install->setEnabled( false );
     connect( d->m_install, SIGNAL(triggered()), this, SLOT(installProjectItems()) );
     actionCollection()->addAction( "project_install", d->m_install );
     d->m_clean = new KAction( i18n("Clean"), this );
+    d->m_clean->setEnabled( false );
     connect( d->m_clean, SIGNAL(triggered()), this, SLOT(cleanProjectItems()) );
     actionCollection()->addAction( "project_clean", d->m_clean );
     d->m_configure = new KAction( i18n("Configure"), this );
     d->m_configure->setIcon(KIcon("configure"));
+    d->m_configure->setEnabled( false );
     connect( d->m_configure, SIGNAL(triggered()), this, SLOT(configureProjectItems()) );
     actionCollection()->addAction( "project_configure", d->m_configure );
     d->m_prune = new KAction( i18n("Prune"), this );
+    d->m_prune->setEnabled( false );
     connect( d->m_prune, SIGNAL(triggered()), this, SLOT(pruneProjectItems()) );
     actionCollection()->addAction( "project_prune", d->m_prune );
     setXMLFile( "kdevprojectmanagerview.rc" );
     d->factory = new KDevProjectManagerViewFactory( this );
     core()->uiController()->addToolView( i18n("Projects"), d->factory );
+    connect( core()->selectionController(), SIGNAL(selectionChanged(KDevelop::Context*)),
+             SLOT(updateActionState(KDevelop::Context*)));
+    connect( d->buildSet, SIGNAL(rowsInserted(const QModelIndex&, int, int)),
+             SLOT(updateFromBuildSetChange()));
+    connect( d->buildSet, SIGNAL(rowsRemoved(const QModelIndex&, int, int)),
+             SLOT(updateFromBuildSetChange()));
+    connect( d->buildSet, SIGNAL(modelReset()),
+             SLOT(updateFromBuildSetChange()));
+}
 
+void ProjectManagerViewPlugin::updateFromBuildSetChange()
+{
+    updateActionState( core()->selectionController()->currentSelection() );
+}
+
+void ProjectManagerViewPlugin::updateActionState( KDevelop::Context* ctx )
+{
+    bool isEmpty = d->buildSet->items().isEmpty();
+    if( isEmpty ) 
+    {
+        isEmpty = !ctx || ctx->type() != Context::ProjectItemContext || dynamic_cast<ProjectItemContext*>( ctx )->items().isEmpty();
+    }
+    d->m_build->setEnabled( !isEmpty );
+    d->m_install->setEnabled( !isEmpty );
+    d->m_clean->setEnabled( !isEmpty );
+    d->m_configure->setEnabled( !isEmpty );
+    d->m_prune->setEnabled( !isEmpty );
 }
 
 ProjectManagerViewPlugin::~ProjectManagerViewPlugin()
@@ -158,19 +191,26 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
     ContextMenuExtension menuExt;
     bool closeProjectsAdded = false;
     bool buildItemsAdded = false;
-    bool hasTargets=false;
+    bool hasTargets = false;
+    bool folderItemsAdded = false;
+    bool fileItemsAdded = false;
+    bool folderWithParentAdded = false;
+    bool targetAdded = false;
     foreach( ProjectBaseItem* item, items )
     {
         d->ctxProjectItemList << item;
         if ( !buildItemsAdded && ( item->target() || item->type() == ProjectBaseItem::BuildFolder ) )
         {
             KAction* action = new KAction( i18n( "Build" ), this );
+            action->setIcon(KIcon("run-build"));
             connect( action, SIGNAL( triggered() ), this, SLOT(buildItemsFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::BuildGroup, action );
             action = new KAction( i18n( "Install" ), this );
+            action->setIcon(KIcon("run-install"));
             connect( action, SIGNAL( triggered() ), this, SLOT(installItemsFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::BuildGroup, action );
             action = new KAction( i18n( "Clean" ), this );
+            action->setIcon(KIcon("run-clean"));
             connect( action, SIGNAL( triggered() ), this, SLOT(cleanItemsFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::BuildGroup, action );
             action = new KAction( i18n( "Add to Buildset" ), this );
@@ -182,52 +222,66 @@ ContextMenuExtension ProjectManagerViewPlugin::contextMenuExtension( KDevelop::C
         KDevelop::ProjectFolderItem *prjitem = item->folder();
         if ( !closeProjectsAdded && prjitem && prjitem->isProjectRoot() )
         {
-            KAction* close = new KAction( i18n( "Close Project(s)" ), this );
+            KAction* close = new KAction( i18np( "Close Project", "Close Projects", items.count() ), this );
+            close->setIcon(KIcon("project-close"));
             connect( close, SIGNAL(triggered()), this, SLOT(closeProjects()) );
             menuExt.addAction( ContextMenuExtension::ProjectGroup, close );
             closeProjectsAdded = true;
         }
 
-        if ( item->folder() )
+        if ( !folderItemsAdded && item->folder() )
         {
+            folderItemsAdded = true;
             KAction* action = new KAction( i18n( "Create File" ), this );
+            action->setIcon(KIcon("document-new"));
             connect( action, SIGNAL(triggered()), this, SLOT(createFileFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
 
             action = new KAction( i18n( "Create Folder" ), this );
+            action->setIcon(KIcon("folder-new"));
             connect( action, SIGNAL(triggered()), this, SLOT(createFolderFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
 
             action = new KAction( i18n( "Reload" ), this );
+            action->setIcon(KIcon("view-refresh"));
             connect( action, SIGNAL(triggered()), this, SLOT(reloadFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
         }
         
-        if ( item->folder() && item->parent() )
+        if ( !folderWithParentAdded && item->folder() && item->parent() )
         {
+            folderWithParentAdded = true;
             KAction* action = new KAction( i18n( "Remove Folder" ), this );
+            action->setIcon(KIcon("user-trash"));
             connect( action, SIGNAL(triggered()), this, SLOT(removeFolderFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
         }
         
-        if ( item->file() )
+        if ( !fileItemsAdded && item->file() )
         {
+            fileItemsAdded = true;
             KAction* action = new KAction( i18n( "Remove File" ), this );
+            action->setIcon(KIcon("user-trash"));
             connect( action, SIGNAL(triggered()), this, SLOT(removeFileFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
         }
-        else if ( item->target() )
+        else if ( !targetAdded && item->target() )
         {
+            targetAdded = true;
             KAction* action = new KAction( i18n( "Create File" ), this );
+            action->setIcon(KIcon("document-new"));
             connect( action, SIGNAL(triggered()), this, SLOT(createFileInTargetFromContextMenu()) );
             menuExt.addAction( ContextMenuExtension::FileGroup, action );
         }
-        else if(!hasTargets && item->executable())
-        {
-            KAction* action = new KAction( i18n("Run"), this );
-            connect( action, SIGNAL( triggered() ), this, SLOT( runTargetsFromContextMenu() ) );
-            menuExt.addAction( ContextMenuExtension::ProjectGroup, action );
-        }
+        
+        //TODO: Port to launch framework
+//         if(!hasTargets && item->executable())
+//         {
+//             KAction* action = new KAction( i18n("Run"), this );
+//             action->setIcon(KIcon("system-run"));
+//             connect( action, SIGNAL( triggered() ), this, SLOT( runTargetsFromContextMenu() ) );
+//             menuExt.addAction( ContextMenuExtension::ProjectGroup, action );
+//         }
 
     }
     
@@ -299,29 +353,42 @@ void ProjectManagerViewPlugin::buildAllProjects()
     ICore::self()->runController()->registerJob( new BuilderJob( BuilderJob::Build, items ) );
 }
 
+void ProjectManagerViewPlugin::runBuilderJob( BuilderJob::BuildType t )
+{
+    QList<ProjectBaseItem*> items;
+    if( !d->buildSet->items().isEmpty() )
+    {
+        ICore::self()->runController()->registerJob( new BuilderJob( t, d->buildSet->items() ) );
+    } else
+    {
+        KDevelop::ProjectItemContext* ctx = dynamic_cast<KDevelop::ProjectItemContext*>(ICore::self()->selectionController()->currentSelection());
+        ICore::self()->runController()->registerJob( new BuilderJob( t, ctx->items() ) );
+    }
+}
+
 void ProjectManagerViewPlugin::installProjectItems()
 {
-     ICore::self()->runController()->registerJob( new BuilderJob( BuilderJob::Install, d->buildSet->items() ) );
+    runBuilderJob( BuilderJob::Install );
 }
 
 void ProjectManagerViewPlugin::pruneProjectItems()
 {
-    ICore::self()->runController()->registerJob( new BuilderJob( BuilderJob::Prune, d->buildSet->items() ) );
+    runBuilderJob( BuilderJob::Prune );
 }
 
 void ProjectManagerViewPlugin::configureProjectItems()
 {
-    ICore::self()->runController()->registerJob( new BuilderJob( BuilderJob::Configure, d->buildSet->items() ) );
+    runBuilderJob( BuilderJob::Configure );
 }
 
 void ProjectManagerViewPlugin::cleanProjectItems()
 {
-     ICore::self()->runController()->registerJob( new BuilderJob( BuilderJob::Clean, d->buildSet->items() ) );
+    runBuilderJob( BuilderJob::Clean );
 }
 
 void ProjectManagerViewPlugin::buildProjectItems()
 {
-     ICore::self()->runController()->registerJob( new BuilderJob( BuilderJob::Build, d->buildSet->items() ) );
+    runBuilderJob( BuilderJob::Build );
 }
 
 ProjectBuildSetModel* ProjectManagerViewPlugin::buildSet()
@@ -345,11 +412,6 @@ void ProjectManagerViewPlugin::runTargetsFromContextMenu( )
         if(t)
         {
             kDebug() << "Running target: " << t->text() << t->builtUrl();
-            IRun r;
-            r.setExecutable(t->builtUrl().toLocalFile());
-            r.setInstrumentor("default");
-            
-            ICore::self()->runController()->execute(r);
         }
     }
 }
@@ -382,7 +444,7 @@ void ProjectManagerViewPlugin::createFolderFromContextMenu( )
                 KUrl url = item->folder()->url();
                 url.addPath( name );
                 if ( !KIO::NetAccess::mkdir( url, window ) ) {
-                    KMessageBox::error( window, i18n( "Can't create folder." ) );
+                    KMessageBox::error( window, i18n( "Cannot create folder." ) );
                     continue;
                 }
                 item->project()->projectFileManager()->addFolder( url, item->folder() );
@@ -447,13 +509,17 @@ ProjectFileItem* createFile(const ProjectFolderItem* item)
         return 0;
     }
 
-    KSaveFile file(url.path());
-    if ( ! file.open() ) {
-        KMessageBox::error( window, i18n( "Can't create file." ) );
-        return 0;
+    {
+        KTemporaryFile temp;
+        if ( !temp.open() || temp.write("\n") == -1 ) {
+            KMessageBox::error( window, i18n( "Cannot create temporary file." ) );
+            return 0;
+        }
+        if ( !KIO::NetAccess::upload( temp.fileName(), url, window ) ) {
+            KMessageBox::error( window, i18n( "Cannot create file." ) );
+            return 0;
+        }
     }
-    file.finalize();
-    file.close();
 
     ProjectFileItem* ret=item->project()->projectFileManager()->addFile( url, item->folder() );
     ICore::self()->documentController()->openDocument( url );
