@@ -68,6 +68,7 @@
 #include "declarationlistquickopen.h"
 #include "customlistquickopen.h"
 #include <language/duchain/functiondefinition.h>
+#include <qmenu.h>
 
 using namespace KDevelop;
 
@@ -87,7 +88,7 @@ public:
 };
 
 K_PLUGIN_FACTORY(KDevQuickOpenFactory, registerPlugin<QuickOpenPlugin>(); )
-K_EXPORT_PLUGIN(KDevQuickOpenFactory(KAboutData("kdevquickopen","kdevquickopen", ki18n("Quick Open"), "0.1", ki18n("Quickly open resources like files, classes, methods"), KAboutData::License_GPL)))
+K_EXPORT_PLUGIN(KDevQuickOpenFactory(KAboutData("kdevquickopen","kdevquickopen", ki18n("Quick Open"), "0.1", ki18n("Quickly open resources such as files, classes and methods."), KAboutData::License_GPL)))
 
 Declaration* cursorDeclaration() {
   IDocument* doc = ICore::self()->documentController()->activeDocument();
@@ -107,7 +108,7 @@ Declaration* cursorDeclaration() {
   return DUChainUtils::declarationForDefinition( DUChainUtils::itemUnderCursor( doc->url(), SimpleCursor(view->cursorPosition()) ) );
 }
 
-///The first declaration(or definition's declaration) that belongs to a context that surrounds the current cursor
+///The first definition that belongs to a context that surrounds the current cursor
 Declaration* cursorContextDeclaration() {
   IDocument* doc = ICore::self()->documentController()->activeDocument();
   if(!doc)
@@ -144,7 +145,7 @@ Declaration* cursorContextDeclaration() {
   if(!definition)
     return 0;
 
-  return DUChainUtils::declarationForDefinition(definition);
+  return definition;
 }
 
 //Returns only the name, no template-parameters or scope
@@ -177,63 +178,59 @@ QString cursorItemText() {
   return QString();
 }
 
-QuickOpenWidgetHandler::QuickOpenWidgetHandler( QuickOpenModel* model, const QStringList& initialItems, const QStringList& initialScopes, bool listOnly, bool noSearchField ) : m_model(model) {
-  m_dialog = new QDialog( ICore::self()->uiController()->activeMainWindow() );
+QuickOpenWidgetHandler::QuickOpenWidgetHandler( QString title, QuickOpenModel* model, const QStringList& initialItems, const QStringList& initialScopes, bool listOnly, bool noSearchField ) : m_model(model), m_expandedTemporary(false) {
+  m_dialog = new KDialog( ICore::self()->uiController()->activeMainWindow() );
+  m_dialog->setInitialSize(QSize(800, 400));
+  m_dialog->setButtons( KDialog::Ok | KDialog::Cancel );
 
-  o.setupUi( m_dialog );
+  o.setupUi( m_dialog->mainWidget() );
   o.list->header()->hide();
   o.list->setRootIsDecorated( false );
   o.list->setVerticalScrollMode( QAbstractItemView::ScrollPerItem );
   connect(o.list->verticalScrollBar(), SIGNAL(valueChanged(int)), m_model, SLOT(placeExpandingWidgets()));
 
+  m_dialog->setWindowTitle(title);
+  o.searchLine->setFocus();
+
   o.list->setItemDelegate( new QuickOpenDelegate( m_model, o.list ) );
 
   if(!listOnly) {
     QStringList allTypes = m_model->allTypes();
     QStringList allScopes = m_model->allScopes();
 
-    QVBoxLayout *itemsLayout = new QVBoxLayout;
-    itemsLayout->setAlignment(Qt::AlignTop);
-
-    foreach( QString type, allTypes )
+    QMenu* itemsMenu = new QMenu;
+    
+    foreach( const QString &type, allTypes )
     {
-      QCheckBox* check = new QCheckBox( type );
-      itemsLayout->addWidget( check );
-
-      if( initialItems.isEmpty() || initialItems.contains( type ) )
-        check->setCheckState( Qt::Checked );
-      else
-        check->setCheckState( Qt::Unchecked );
-
-      connect( check, SIGNAL(stateChanged(int)), this, SLOT(updateProviders()) );
+      QAction* action = new QAction(type, itemsMenu);
+      action->setCheckable(true);
+      action->setChecked(initialItems.isEmpty() || initialItems.contains( type ));
+      connect( action, SIGNAL(toggled(bool)), this, SLOT(updateProviders()), Qt::QueuedConnection );
+      itemsMenu->addAction(action);
     }
 
-    itemsLayout->addStretch( 1 );
-    o.itemsGroup->setLayout( itemsLayout );
+    o.itemsButton->setMenu(itemsMenu);
 
-    QVBoxLayout *scopesLayout = new QVBoxLayout;
-    scopesLayout->setAlignment(Qt::AlignTop);
+    QMenu* scopesMenu = new QMenu;
 
-
-    foreach( QString scope, allScopes )
+    foreach( const QString &scope, allScopes )
     {
-      QCheckBox* check = new QCheckBox( scope );
-      scopesLayout->addWidget( check );
+      QAction* action = new QAction(scope, scopesMenu);
+      action->setCheckable(true);
+      action->setChecked(initialScopes.isEmpty() || initialScopes.contains( scope ) );
 
-      if( initialScopes.isEmpty() || initialScopes.contains( scope ) )
-        check->setCheckState( Qt::Checked );
-      else
-        check->setCheckState( Qt::Unchecked );
-
-      connect( check, SIGNAL(stateChanged(int)), this, SLOT(updateProviders()) );
+      connect( action, SIGNAL(toggled(bool)), this, SLOT(updateProviders()), Qt::QueuedConnection );
+      scopesMenu->addAction(action);
     }
+    
+    o.scopesButton->setMenu(scopesMenu);
 
-    scopesLayout->addStretch( 1 );
-    o.scopeGroup->setLayout( scopesLayout );
   }else{
     o.list->setFocusPolicy(Qt::StrongFocus);
-    o.scopeGroup->hide();
-    o.itemsGroup->hide();
+    o.scopesButton->hide();
+    o.itemsButton->hide();
+    o.label->hide();
+    o.label_2->hide();
   }
 
   if( noSearchField ) {
@@ -242,7 +239,9 @@ QuickOpenWidgetHandler::QuickOpenWidgetHandler( QuickOpenModel* model, const QSt
   }
   o.searchLine->installEventFilter( this );
   o.list->installEventFilter( this );
-  o.buttonBox->installEventFilter( this );
+  o.list->setFocusPolicy(Qt::NoFocus);
+  o.scopesButton->setFocusPolicy(Qt::NoFocus);
+  o.itemsButton->setFocusPolicy(Qt::NoFocus);
 
   connect( o.searchLine, SIGNAL(textChanged( const QString& )), this, SLOT(textChanged( const QString& )) );
   connect( m_dialog, SIGNAL(accepted()), this, SLOT(accept()) );
@@ -264,8 +263,13 @@ void QuickOpenWidgetHandler::run() {
   o.list->setVerticalScrollMode(QAbstractItemView::ScrollPerItem);
   m_model->setTreeView( o.list );
   o.list->setModel( m_model );
+  if (!m_preselectedText.isEmpty())
+  {
+    o.searchLine->setText(m_preselectedText);
+    o.searchLine->selectAll();
+  }
   connect( o.list->selectionModel(), SIGNAL(currentRowChanged( const QModelIndex&, const QModelIndex& )), this, SLOT(currentChanged( const QModelIndex&, const QModelIndex& )) );
-  connect( o.list->selectionModel(), SIGNAL(selectionChanged( const QModelIndex&, const QModelIndex& )), this, SLOT(currentChanged( const QModelIndex&, const QModelIndex& )) );
+  connect( o.list->selectionModel(), SIGNAL(selectionChanged( const QItemSelection&, const QItemSelection& )), this, SLOT(currentChanged( const QItemSelection&, const QItemSelection& )) );
 }
 
 QuickOpenWidgetHandler::~QuickOpenWidgetHandler() {
@@ -274,24 +278,47 @@ QuickOpenWidgetHandler::~QuickOpenWidgetHandler() {
   delete m_dialog;
 }
 
-void QuickOpenWidgetHandler::updateProviders() {
-  QStringList checkedItems;
-  QStringList checkedScopes;
+void QuickOpenWidgetHandler::setPreselectedText(const QString& text)
+{
+  m_preselectedText = text;
+}
 
-  foreach( QObject* obj, o.itemsGroup->children() ) {
-    QCheckBox* box = qobject_cast<QCheckBox*>( obj );
-    if( box ) {
-      if( box->checkState() == Qt::Checked )
-        checkedItems << box->text().remove('&');
+void QuickOpenWidgetHandler::updateProviders() {
+  if(QAction* action = qobject_cast<QAction*>(sender())) {
+    QMenu* menu = qobject_cast<QMenu*>(action->parentWidget());
+    if(menu) {
+      menu->show();
+      menu->setActiveAction(action);
     }
   }
-
-  foreach( QObject* obj, o.scopeGroup->children() ) {
-    QCheckBox* box = qobject_cast<QCheckBox*>( obj );
-    if( box ) {
-      if( box->checkState() == Qt::Checked )
-        checkedScopes << box->text().remove('&');
+  
+  QStringList checkedItems;
+  
+  if(o.itemsButton->menu()) {
+  
+    foreach( QObject* obj, o.itemsButton->menu()->children() ) {
+      QAction* box = qobject_cast<QAction*>( obj );
+      if( box ) {
+        if( box->isChecked() )
+          checkedItems << box->text().remove('&');
+      }
     }
+    o.itemsButton->setText(checkedItems.join(", "));
+  }
+
+  QStringList checkedScopes;
+  
+  if(o.scopesButton->menu()) {
+    
+    foreach( QObject* obj, o.scopesButton->menu()->children() ) {
+      QAction* box = qobject_cast<QAction*>( obj );
+      if( box ) {
+        if( box->isChecked() )
+          checkedScopes << box->text().remove('&');
+      }
+    }
+    
+    o.scopesButton->setText(checkedScopes.join(", "));
   }
 
   emit scopesChanged( checkedScopes );
@@ -320,6 +347,10 @@ void QuickOpenWidgetHandler::currentChanged( const QModelIndex& /*current*/, con
   callRowSelected();
 }
 
+void QuickOpenWidgetHandler::currentChanged( const QItemSelection& /*current*/, const QItemSelection& /*previous */) {
+  callRowSelected();
+}
+
 void QuickOpenWidgetHandler::accept() {
   QString filterText = o.searchLine->text();
   m_model->execute( o.list->currentIndex(), filterText );
@@ -338,24 +369,42 @@ bool QuickOpenWidgetHandler::eventFilter ( QObject * watched, QEvent * event )
 {
   QKeyEvent *keyEvent = dynamic_cast<QKeyEvent*>(event);
   
-  if(keyEvent && m_shiftDetector.checkKeyEvent(keyEvent)) {
-    //Toggle the expanded state
-    QModelIndex row = o.list->selectionModel()->currentIndex();
-    if( row.isValid() ) {
-      row = row.sibling( row.row(), 0 );
-
-      m_model->setExpanded( row, !m_model->isExpanded( row ) );
-      return true;
-    }      
+  if( event->type() == QEvent::KeyRelease ) {
+    if(keyEvent->key() == Qt::Key_Alt) {
+      if((m_expandedTemporary && m_altDownTime.msecsTo( QTime::currentTime() ) > 300) || (!m_expandedTemporary && m_altDownTime.msecsTo( QTime::currentTime() ) < 300 && m_hadNoCommandSinceAlt)) {
+        //Unexpand the item
+        QModelIndex row = o.list->selectionModel()->currentIndex();
+        if( row.isValid() ) {
+          row = row.sibling( row.row(), 0 );
+          if(m_model->isExpanded( row ))
+            m_model->setExpanded( row, false );
+        }      
+      }
+      m_expandedTemporary = false;
+    }
   }
   
   if( event->type() == QEvent::KeyPress  ) {
+    m_hadNoCommandSinceAlt = false;
+    if(keyEvent->key() == Qt::Key_Alt) {
+      m_hadNoCommandSinceAlt = true;
+      //Expand
+      QModelIndex row = o.list->selectionModel()->currentIndex();
+      if( row.isValid() ) {
+        row = row.sibling( row.row(), 0 );
+        m_altDownTime = QTime::currentTime();
+        if(!m_model->isExpanded( row )) {
+          m_expandedTemporary = true;
+          m_model->setExpanded( row, true );
+        }
+      }      
+    }
 
     switch( keyEvent->key() ) {
       case Qt::Key_Down:
       case Qt::Key_Up:
       {
-        if( keyEvent->modifiers() == Qt::ShiftModifier ) {
+        if( keyEvent->modifiers() == Qt::AltModifier ) {
           QWidget* w = m_model->expandingWidget(o.list->selectionModel()->currentIndex());
           if( KDevelop::QuickOpenEmbeddedWidgetInterface* interface =
               dynamic_cast<KDevelop::QuickOpenEmbeddedWidgetInterface*>( w ) ){
@@ -378,7 +427,7 @@ bool QuickOpenWidgetHandler::eventFilter ( QObject * watched, QEvent * event )
 
       case Qt::Key_Left: {
         //Expand/unexpand
-        if( keyEvent->modifiers() == Qt::ShiftModifier ) {
+        if( keyEvent->modifiers() == Qt::AltModifier ) {
           //Eventually Send action to the widget
           QWidget* w = m_model->expandingWidget(o.list->selectionModel()->currentIndex());
           if( KDevelop::QuickOpenEmbeddedWidgetInterface* interface =
@@ -401,7 +450,7 @@ bool QuickOpenWidgetHandler::eventFilter ( QObject * watched, QEvent * event )
       }
       case Qt::Key_Right: {
         //Expand/unexpand
-        if( keyEvent->modifiers() == Qt::ShiftModifier ) {
+        if( keyEvent->modifiers() == Qt::AltModifier ) {
           //Eventually Send action to the widget
           QWidget* w = m_model->expandingWidget(o.list->selectionModel()->currentIndex());
           if( KDevelop::QuickOpenEmbeddedWidgetInterface* interface =
@@ -424,7 +473,7 @@ bool QuickOpenWidgetHandler::eventFilter ( QObject * watched, QEvent * event )
       }
       case Qt::Key_Return:
       case Qt::Key_Enter: {
-        if( keyEvent->modifiers() == Qt::ShiftModifier ) {
+        if( keyEvent->modifiers() == Qt::AltModifier ) {
           //Eventually Send action to the widget
           QWidget* w = m_model->expandingWidget(o.list->selectionModel()->currentIndex());
           if( KDevelop::QuickOpenEmbeddedWidgetInterface* interface =
@@ -476,11 +525,6 @@ QuickOpenPlugin::QuickOpenPlugin(QObject *parent,
     quickOpenClass->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_C );
     connect(quickOpenClass, SIGNAL(triggered(bool)), this, SLOT(quickOpenClass()));
 
-    KAction* quickOpenOpenFile = actions->addAction("quick_open_open_files");
-    quickOpenOpenFile->setText( i18n("Quick Open &Open Files") );
-    quickOpenOpenFile->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_T );
-    connect(quickOpenOpenFile, SIGNAL(triggered(bool)), this, SLOT(quickOpenOpenFiles()));
-
     KAction* quickOpenFunction = actions->addAction("quick_open_function");
     quickOpenFunction->setText( i18n("Quick Open &Function") );
     quickOpenFunction->setShortcut( Qt::CTRL | Qt::ALT | Qt::Key_M );
@@ -496,10 +540,10 @@ QuickOpenPlugin::QuickOpenPlugin(QObject *parent,
     quickOpenDefinition->setShortcut( Qt::CTRL | Qt::Key_Comma );
     connect(quickOpenDefinition, SIGNAL(triggered(bool)), this, SLOT(quickOpenDefinition()));
 
-    KAction* quickOpenNavigate = actions->addAction("quick_open_navigate");
-    quickOpenNavigate->setText( i18n("Navigate Declaration") );
-    quickOpenNavigate->setShortcut( Qt::ALT | Qt::Key_Space );
-    connect(quickOpenNavigate, SIGNAL(triggered(bool)), this, SLOT(quickOpenNavigate()));
+//     KAction* quickOpenNavigate = actions->addAction("quick_open_navigate");
+//     quickOpenNavigate->setText( i18n("Navigate Declaration") );
+//     quickOpenNavigate->setShortcut( Qt::ALT | Qt::Key_Space );
+//     connect(quickOpenNavigate, SIGNAL(triggered(bool)), this, SLOT(quickOpenNavigate()));
 
     KAction* quickOpenNavigateFunctions = actions->addAction("quick_open_outline");
     quickOpenNavigateFunctions->setText( i18n("Outline") );
@@ -566,7 +610,17 @@ void QuickOpenPlugin::showQuickOpen( ModelTypes modes )
   if((modes & OpenFiles) && !useScopes.contains(i18n("Currently Open")))
     useScopes << i18n("Currently Open");
 
-  m_currentWidgetHandler = new QuickOpenWidgetHandler( m_model, initialItems, useScopes );
+  m_currentWidgetHandler = new QuickOpenWidgetHandler( i18n("Quick Open"), m_model, initialItems, useScopes );
+  if (!(modes & Files) || modes == QuickOpenPlugin::All)
+  {
+    KDevelop::IDocument *currentDoc = core()->documentController()->activeDocument();
+    if (currentDoc && currentDoc->isTextDocument())
+    {
+      QString preselected = currentDoc->textSelection().isEmpty() ? currentDoc->textWord() : currentDoc->textDocument()->text(currentDoc->textSelection());
+      m_currentWidgetHandler->setPreselectedText(preselected);
+    }
+  }
+  
   connect( m_currentWidgetHandler, SIGNAL( scopesChanged( const QStringList& ) ), this, SLOT( storeScopes( const QStringList& ) ) );
   m_currentWidgetHandler->run();
 }
@@ -599,11 +653,6 @@ void QuickOpenPlugin::quickOpenClass()
   showQuickOpen( Classes );
 }
 
-void QuickOpenPlugin::quickOpenOpenFiles()
-{
-  showQuickOpen( OpenFiles );
-}
-
 QSet<KDevelop::IndexedString> QuickOpenPlugin::fileSet() const {
   return m_model->fileSet();
 }
@@ -779,7 +828,7 @@ void QuickOpenPlugin::quickOpenNavigate()
     }
 
     //Change the parent so there are no conflicts in destruction order
-    m_currentWidgetHandler = new QuickOpenWidgetHandler( model, QStringList(), QStringList(), true, true );
+    m_currentWidgetHandler = new QuickOpenWidgetHandler( i18n("Navigate"), model, QStringList(), QStringList(), true, true );
     model->setParent(m_currentWidgetHandler);
     model->setExpanded(model->index(0,0, QModelIndex()), true);
 
@@ -844,7 +893,7 @@ void QuickOpenPlugin::quickOpenNavigateFunctions()
         return false;
     }
     virtual bool accept(DUContext* ctx) {
-      if(ctx->type() == DUContext::Class || ctx->type() == DUContext::Namespace || ctx->type() == DUContext::Global )
+      if(ctx->type() == DUContext::Class || ctx->type() == DUContext::Namespace || ctx->type() == DUContext::Global || ctx->type() == DUContext::Other || ctx->type() == DUContext::Helper )
         return true;
       else
         return false;
@@ -863,7 +912,11 @@ void QuickOpenPlugin::quickOpenNavigateFunctions()
 
   model->registerProvider( QStringList(), QStringList(), new DeclarationListDataProvider(this, items, true) );
 
-  m_currentWidgetHandler = new QuickOpenWidgetHandler( model, QStringList(), QStringList(), true );
+  m_currentWidgetHandler = new QuickOpenWidgetHandler( i18n("Outline"), model, QStringList(), QStringList(), true );
+  
+  model->setParent(m_currentWidgetHandler);
+  m_currentWidgetHandler->run();
+
   //Select the declaration that contains the cursor
   if(cursorDecl) {
     int num = 0;
@@ -875,8 +928,6 @@ void QuickOpenPlugin::quickOpenNavigateFunctions()
       ++num;
     }
   }
-  model->setParent(m_currentWidgetHandler);
-  m_currentWidgetHandler->run();
 }
 
 #include "quickopenplugin.moc"
