@@ -447,6 +447,30 @@ int ProjectLibraryTargetItem::type() const
     return ProjectBaseItem::LibraryTarget;
 }
 
+QModelIndex ProjectModel::pathToIndex(const QStringList& tofetch_) const
+{
+    if(tofetch_.isEmpty())
+        return QModelIndex();
+    QStringList tofetch(tofetch_);
+    if(tofetch.last().isEmpty())
+        tofetch.takeLast();
+
+    QModelIndex current=index(0,0, QModelIndex());
+
+    QModelIndex ret;
+    foreach(const QString& currentName, tofetch)
+    {
+        QModelIndexList l = match(current, Qt::EditRole, currentName, 1, Qt::MatchExactly);
+        if(l.count()==1) {
+            ret=l.first();
+            current = index(0,0, ret);
+        } else
+            ret = current = QModelIndex();
+    }
+    Q_ASSERT(!ret.isValid() || data(ret).toString()==tofetch.last());
+    return ret;
+}
+
 
 }
 #include "projectmodel.moc"
