@@ -27,7 +27,11 @@
 
 class CompletionSettings : public KDevelop::ICompletionSettings {
 public:
-    CompletionSettings() : m_level(AlwaysFull), m_automatic(true), m_enableSemanticHighlighting(true), m_highlightSemanticProblems(true), m_localVariableColorizationLevel(120) {
+    CompletionSettings()
+        : m_level(MinimalWhenAutomatic), m_automatic(true),
+          m_highlightSemanticProblems(true), m_showMultiLineInformation(true),
+          m_localColorizationLevel(170), m_globalColorizationLevel(255)
+    {
     }
 
     bool readBoolConfig(QString name, bool _default = false) const {
@@ -60,23 +64,28 @@ public:
         emit settingsChanged(this);
     }
     
-    virtual int localVariableColorizationLevel() const {
-        return readIntConfig("localVariableColorization", m_localVariableColorizationLevel);
+    virtual int localColorizationLevel() const {
+        return readIntConfig("localColorization", m_localColorizationLevel);
     }
     
-    virtual bool semanticHighlightingEnabled() const {
-        return readBoolConfig("enableSemanticHighlighting", m_enableSemanticHighlighting);
+    virtual int globalColorizationLevel() const {
+        return readIntConfig("globalColorization", m_globalColorizationLevel);
     }
     
     virtual bool highlightSemanticProblems() const {
         return readBoolConfig("highlightSemanticProblems", m_highlightSemanticProblems);
     }
     
+    virtual bool showMultiLineSelectionInformation() const {
+        return readBoolConfig("showMultiLineSelectionInformation", m_showMultiLineInformation);
+    }
+    
     static CompletionSettings& self();
     
     CompletionLevel m_level;
-    bool m_automatic, m_enableSemanticHighlighting, m_highlightSemanticProblems;
-    int m_localVariableColorizationLevel;
+    bool m_automatic, m_highlightSemanticProblems, m_showMultiLineInformation;
+    int m_localColorizationLevel;
+    int m_globalColorizationLevel;
 };
 
 #endif
