@@ -18,12 +18,14 @@
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 #include "core.h"
+#include "core_p.h"
 
-#include <QApplication>
-#include <QPointer>
+#include <QtGui/QApplication>
+#include <QtCore/QPointer>
 
 #include <kdebug.h>
 #include <kglobal.h>
+#include <klocale.h>
 
 #include <sublime/area.h>
 #include <sublime/tooldocument.h>
@@ -43,15 +45,40 @@
 #include "languagecontroller.h"
 #include "documentcontroller.h"
 #include "runcontroller.h"
+#include "documentationcontroller.h"
 #include "sourceformattercontroller.h"
-#include "core_p.h"
+#include "selectioncontroller.h"
+#include "debugcontroller.h"
+#include "kdevplatformversion.h"
 
 namespace KDevelop {
 
 Core *Core::m_self = 0;
+KAboutData aboutData()
+{
+    KAboutData aboutData( "kdevplatform", "kdevplatform", 
+                          ki18n("KDevelop Platform"), KDEVPLATFORM_VERSION_STR, 
+                          ki18n("Development Platform for IDE-like Applications"), 
+                          KAboutData::License_LGPL_V2, ki18n( "Copyright 2004-2009, The KDevelop developers" ), 
+                          KLocalizedString(), "http://www.kdevelop.org" );
+    aboutData.addAuthor( ki18n("Andreas Pakulat"), ki18n( "Maintainer, Architecture, VCS Support, Project Management Support, QMake Projectmanager" ), "apaku@gmx.de" );
+    aboutData.addAuthor( ki18n("Alexander Dymo"), ki18n( "Architecture, Sublime UI, Ruby support" ), "adymo@kdevelop.org" );
+    aboutData.addAuthor( ki18n("David Nolden"), ki18n( "Definition-Use Chain, C++ Support" ), "david.nolden.kdevelop@art-master.de" );
+    aboutData.addAuthor( ki18n("Aleix Pol Gonzalez"), ki18n( "CMake Support, Run Support, Kross Support" ), "aleixpol@kde.org" );
+    aboutData.addAuthor( ki18n("Vladimir Prus"), ki18n( "GDB integration" ), "ghost@cs.msu.su" );
+    aboutData.addAuthor( ki18n("Hamish Rodda"), ki18n( "Text editor integration, definition-use chain" ), "rodda@kde.org" );
+    
+    aboutData.addCredit( ki18n("Matt Rogers"), KLocalizedString(), "mattr@kde.org");
+    aboutData.addCredit( ki18n("Cédric Pasteur"), ki18n("astyle and ident support"), "cedric.pasteur@free.fr" );
+    aboutData.addCredit( ki18n("Evgeniy Ivanov"), ki18n("Distributed VCS, Git, Mercurial"), "powerfox@kde.ru" );
+    aboutData.addCredit( ki18n("Manuel Breugelmanns"), ki18n( "Veritas, QTest integraton"), "mbr.nxi@gmail.com" );
+    aboutData.addCredit( ki18n("Robert Gruber") , ki18n( "SnippetPart, debugger and usability patches" ), "rgruber@users.sourceforge.net" );
+    aboutData.addCredit( ki18n("Dukju Ahn"), ki18n( "Subversion plugin, Custom Make Manager, Overall improvements" ), "dukjuahn@gmail.com" );
+    return aboutData;
+}
 
 CorePrivate::CorePrivate(Core *core):
-    m_componentData( KAboutData( "kdevplatform", "kdevplatform", ki18n("KDevelop Platform"), "1.0", ki18n("Development Platform for IDE-like Applications"), KAboutData::License_LGPL_V2 ) ), m_core(core), m_cleanedUp(false)
+    m_componentData( aboutData() ), m_core(core), m_cleanedUp(false)
 {
 }
 
@@ -103,6 +130,21 @@ void CorePrivate::initialize(Core::Setup mode)
         sourceFormatterController = new SourceFormatterController(m_core);
     }
 
+    if( !selectionController )
+    {
+        selectionController = new SelectionController(m_core);
+    }
+    
+    if( !documentationController && !(mode & Core::NoUi) )
+    {
+        documentationController = new DocumentationController(m_core);
+    }
+
+    if( !debugController )
+    {
+        debugController = new DebugController(m_core);
+    }
+
     kDebug() << "initializing ui controller";
     sessionController->initialize();
     if(!(mode & Core::NoUi)) uiController->initialize();
@@ -130,10 +172,15 @@ void CorePrivate::initialize(Core::Setup mode)
         uiController->loadAllAreas(KGlobal::config());
         uiController->defaultMainWindow()->show();
     }
+    runController->initialize();
     sourceFormatterController->initialize();
+    selectionController->initialize();
+    documentationController->initialize();
+    debugController->initialize();
 }
 CorePrivate::~CorePrivate()
 {
+    delete selectionController;
     delete projectController;
     delete languageController;
     delete pluginController;
@@ -143,6 +190,8 @@ CorePrivate::~CorePrivate()
     delete runController;
     delete sessionController;
     delete sourceFormatterController;
+    delete documentationController;
+    delete debugController;
 }
 
 
@@ -186,12 +235,14 @@ Core::Setup Core::setupFlags() const
 void Core::cleanup()
 {
     if (!d->m_cleanedUp) {
+        d->selectionController->cleanup();
         // Save the layout of the ui here, so run it first
         d->uiController->cleanup();
 
         /* Must be called before projectController->cleanup(). */
         // Closes all documents (discards, as already saved if the user wished earlier)
         d->documentController->cleanup();
+        d->runController->cleanup();
 
         d->projectController->cleanup();
         d->sourceFormatterController->cleanup();
@@ -292,4 +343,29 @@ ISourceFormatterController* Core::sourceFormatterController()
     return d->sourceFormatterController;
 }
 
+ISelectionController* Core::selectionController()
+{
+    return d->selectionController;
+}
+
+IDocumentationController* Core::documentationController()
+{
+    return d->documentationController;
+}
+
+IDebugController* Core::debugController()
+{
+    return d->debugController;
+}
+
+DebugController* Core::debugControllerInternal()
+{
+    return d->debugController;
+}
+
+QString Core::version()
+{
+    return KDEVPLATFORM_VERSION_STR;
+}
+
 }
