@@ -36,11 +36,15 @@ class SessionController;
 class CorePrivate;
 class SourceFormatterController;
 class SelectionController;
+class DocumentationController;
+class DebugController;
 
 class KDEVPLATFORMSHELL_EXPORT Core: public ICore {
 public:
     enum Setup { Default=0, NoUi=1 };
 
+    static QString version();
+
     /** Initialize the core of the kdevplatform application */
     static void initialize(Setup mode=Default);
 
@@ -77,6 +81,12 @@ public:
 
     /** @copydoc ICore::selectionController() */
     virtual ISelectionController* selectionController();
+    
+    /** @copydoc ICore::documentationController() */
+    virtual IDocumentationController* documentationController();
+
+    /** @copydoc ICore::debugController() */
+    virtual IDebugController* debugController();
 
     /** @copydoc ICore::activeSession() */
     virtual ISession *activeSession();
@@ -106,6 +116,9 @@ public:
     /** @return run controller */
     RunController *runControllerInternal();
 
+    /** @return debug controller */
+    DebugController *debugControllerInternal();
+
     /**
      * @return session controller
      * @internal
