@@ -43,6 +43,7 @@ Boston, MA 02110-1301, USA.
 #include <sublime/area.h>
 #include <sublime/view.h>
 #include <interfaces/iplugincontroller.h>
+#include <interfaces/iprojectcontroller.h>
 
 #include "core.h"
 #include "mainwindow.h"
@@ -57,7 +58,8 @@ namespace KDevelop
 
 struct DocumentControllerPrivate {
     DocumentControllerPrivate(DocumentController* c)
-    : controller(c)
+        : controller(c)
+        , fileOpenRecent(0)
     {
     }
 
@@ -165,7 +167,8 @@ void KDevelop::DocumentController::initialize()
 
 void DocumentController::cleanup()
 {
-    d->fileOpenRecent->saveEntries( KConfigGroup(KGlobal::config(), "Recent Files" ) );
+    if (d->fileOpenRecent)
+        d->fileOpenRecent->saveEntries( KConfigGroup(KGlobal::config(), "Recent Files" ) );
 
     // Close all documents without checking if they should be saved.
     // This is because the user gets a chance to save them during MainWindow::queryClose.
@@ -234,6 +237,7 @@ void DocumentController::setupActions()
 
     action = d->closeAllOthers = ac->addAction( "file_closeother" );
     action->setIcon(KIcon("window-close"));
+    action->setShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_W );
     action->setText(i18n( "Close All Ot&hers" ) );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllOtherDocuments() ) );
     action->setToolTip( i18n( "Close all other documents" ) );
@@ -284,19 +288,13 @@ IDocument* DocumentController::openDocument( const KUrl & inputUrl,
 
     if ( url.isEmpty() && (!activationParams.testFlag(IDocumentController::DoNotCreateView)) )
     {
-        KSharedConfig * config = KGlobal::config().data();
-        KConfigGroup group = config->group( "General Options" );
-        QString dir;
-        if( group.hasKey( "DefaultProjectsDirectory" ) )
+        KUrl dir;
+        if( activeDocument() )
         {
-            dir = group.readEntry( "DefaultProjectsDirectory",
-                                             QDir::homePath() );
-        }else if( activeDocument() )
-        {
-            dir = activeDocument()->url().directory();
+            dir = KUrl( activeDocument()->url().directory() );
         }else
         {
-            dir = QDir::homePath();
+            dir = KGlobal::config()->group("Open File").readEntry( "Last Open File Directory", Core::self()->projectController()->projectsBaseDirectory() );
         }
 
         url = KFileDialog::getOpenUrl( dir, i18n( "*.*|Text File\n" ),
@@ -307,6 +305,8 @@ IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         //still no url
         return 0;
 
+    KGlobal::config()->group("Open File").writeEntry( "Last Open File Directory", url.upUrl() );
+
     bool emitOpened = false;
 
     //get a part document
@@ -379,7 +379,7 @@ IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         Sublime::View *partView = 0;
         foreach (Sublime::View *view, sdoc->views())
         {
-            if (area->views().contains(view))
+            if (area->views().contains(view) && (activationParams.testFlag(DoNotForceCurrentView) || area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView())))
             {
                 partView = view;
                 break;
@@ -397,9 +397,9 @@ IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         if(textView && textView->textView()) {
             applyRange = false;
             if (range.isEmpty())
-                textView->textView()->setCursorPosition( range.start() );
+                doc->setCursorPosition( range.start() );
             else
-                textView->textView()->setSelection( range );
+                doc->setTextSelection( range );
         }else if(textView) {
             textView->setInitialRange(range);
         }
@@ -429,7 +429,7 @@ IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         emit documentOpened( d->documents[url] );
     }
 
-    if (!activationParams.testFlag(IDocumentController::DoNotActivate))
+    if (!activationParams.testFlag(IDocumentController::DoNotActivate) && doc != activeDocument())
         emit documentActivated( doc );
 
     d->saveAll->setEnabled(true);
@@ -438,7 +438,7 @@ IDocument* DocumentController::openDocument( const KUrl & inputUrl,
     d->closeAll->setEnabled(true);
     d->closeAllOthers->setEnabled(true);
 
-    if(doc != previousActiveDocument) {
+    if(doc) {
         KTextEditor::Cursor activePosition;
         if(doc->textDocument() && doc->textDocument()->activeView())
             activePosition = doc->textDocument()->activeView()->cursorPosition();
@@ -689,6 +689,7 @@ void DocumentController::closeAllOtherDocuments()
                 }
             }
         }
+        activeView->widget()->setFocus();
     }
 }
 
