@@ -24,7 +24,6 @@ Boston, MA 02110-1301, USA.
 
 #include <QtCore/QHash>
 #include <QtGui/QDockWidget>
-#include <QtGui/QStackedWidget>
 
 #include <KDE/KApplication>
 #include <KDE/KActionCollection>
@@ -36,12 +35,15 @@ Boston, MA 02110-1301, USA.
 #include <kdebug.h>
 #include <ktexteditor/view.h>
 #include <kxmlguifactory.h>
+#include <ktoggleaction.h>
 
+#include <sublime/area.h>
 #include "shellextension.h"
 #include "partcontroller.h"
 #include "plugincontroller.h"
 #include "uicontroller.h"
 #include "documentcontroller.h"
+#include "debugcontroller.h"
 
 namespace KDevelop
 {
@@ -77,8 +79,8 @@ MainWindow::MainWindow( Sublime::Controller *parent, Qt::WFlags flags )
         setXMLFile( ShellExtension::getInstance() ->xmlFile() );
     }
 
-    connect(this->guiFactory(), SIGNAL(clientAdded(KXMLGUIClient*)),
-            d, SLOT(fixToolbar()));
+//    connect(this->guiFactory(), SIGNAL(clientAdded(KXMLGUIClient*)),
+//            d, SLOT(fixToolbar()));
 }
 
 MainWindow::~ MainWindow()
@@ -115,6 +117,8 @@ void MainWindow::loadSettings()
     else if( bottomright == 1 )
         setCorner( Qt::BottomRightCorner, Qt::BottomDockWidgetArea );
 
+    d->setupAreaSelectorActions();
+
     Sublime::MainWindow::loadSettings();
 }
 
@@ -137,10 +141,7 @@ void MainWindow::initialize()
     connect(Core::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), SLOT(documentActivated(KDevelop::IDocument*)));
     connect(Core::self()->documentController(), SIGNAL(documentStateChanged(KDevelop::IDocument*)), SLOT(documentStateChanged(KDevelop::IDocument*)));
     connect(Core::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), SLOT(documentClosed(KDevelop::IDocument*)));
-    /*connect( Core::projectController(), SIGNAL( projectOpened() ),
-             d, SLOT( projectOpened() ) );
-    connect( Core::projectController(), SIGNAL( projectClosed() ),
-             d, SLOT( projectClosed() ) );*/
+    guiFactory()->addClient(Core::self()->debugControllerInternal());
     d->setupGui();
 }
 
@@ -148,18 +149,6 @@ void MainWindow::cleanup()
 {
 }
 
-void MainWindow::fillContextMenu( KMenu *menu, const Context *context )
-{
-    //Perhaps we get rid of this framework and instead have every Context contains
-    //a kactioncollection.  Plugins could add their actions directly to the context
-    //object retrieved from Core... ??
-    emit contextMenu( menu, context );
-
-    //Put this in every context menu so that plugins will be encouraged to allow shortcuts
-    QAction * action = actionCollection() ->action( "settings_configure_shortcuts" );
-    menu->addAction( action );
-}
-
 void MainWindow::setVisible( bool visible )
 {
     KXmlGuiWindow::setVisible( visible );
@@ -198,6 +187,11 @@ void MainWindow::registerStatus(QObject* status)
     d->registerStatus(status);
 }
 
+void MainWindow::initializeStatusBar()
+{
+    d->setupStatusBar();
+}
+
 }
 
 #include "mainwindow.moc"
