@@ -22,7 +22,8 @@ Boston, MA 02110-1301, USA.
 #include "mainwindow_p.h"
 
 #include <QApplication>
-#include <QStackedWidget>
+#include <QBoxLayout>
+#include <QLabel>
 
 #include <kmenu.h>
 #include <kdebug.h>
@@ -41,6 +42,7 @@ Boston, MA 02110-1301, USA.
 #include <ktogglefullscreenaction.h>
 #include <kactioncollection.h>
 #include <ktoolbarpopupaction.h>
+#include <knotifyconfigwidget.h>
 
 #include <sublime/area.h>
 #include <sublime/view.h>
@@ -68,7 +70,14 @@ MainWindowPrivate::MainWindowPrivate(MainWindow *mainWindow)
 void MainWindowPrivate::setupGui()
 {
     m_statusBar = new KDevelop::StatusBar(m_mainWindow);
-    m_mainWindow->setStatusBar(m_statusBar);
+    setupStatusBar();
+}
+
+void MainWindowPrivate::setupStatusBar()
+{
+    QWidget *location = m_mainWindow->statusBarLocation();
+    if (m_statusBar)
+        location->layout()->addWidget(m_statusBar);
 }
 
 void MainWindowPrivate::addPlugin( IPlugin *plugin )
@@ -77,6 +86,7 @@ void MainWindowPrivate::addPlugin( IPlugin *plugin )
     Q_ASSERT( plugin );
 
     m_mainWindow->guiFactory()->addClient( plugin );
+    setupAreaSelectorActions();
 }
 
 void MainWindowPrivate::removePlugin( IPlugin *plugin )
@@ -84,6 +94,7 @@ void MainWindowPrivate::removePlugin( IPlugin *plugin )
     Q_ASSERT( plugin );
 
     m_mainWindow->guiFactory()->removeClient( plugin );
+    setupAreaSelectorActions();
 }
 
 void MainWindowPrivate::activePartChanged(KParts::Part *part)
@@ -107,8 +118,6 @@ void MainWindowPrivate::changeActiveView(Sublime::View *view)
         lastXMLGUIClientView = NULL;
     }
 
-    m_statusBar->viewChanged(view);
-
     if (!view)
         return;
 
@@ -141,6 +150,7 @@ void MainWindowPrivate::changeActiveView(Sublime::View *view)
         connect(viewWidget, SIGNAL(destroyed(QObject*)),
                 this, SLOT(xmlguiclientDestroyed(QObject*)));
     }
+    setupAreaSelectorActions();
 }
 
 void MainWindowPrivate::xmlguiclientDestroyed(QObject* obj)
@@ -168,19 +178,31 @@ void MainWindowPrivate::setupActions()
     action = KStandardAction::preferences( this, SLOT( settingsDialog() ),
                                       actionCollection());
     action->setToolTip( text );
-    action->setWhatsThis( QString( "<b>%1</b><p>%2<p>" ).arg( text ).arg(
+    action->setWhatsThis( QString( "<b>%1</b><p>%2</p>" ).arg( text ).arg(
                               i18n( "Lets you customize %1.", app ) ) );
 
     action = KStandardAction::showStatusbar( this, SLOT( toggleStatusbar() ),
                                         actionCollection());
-    action->setText( i18n( "Show &Statusbar" ) );
-    action->setToolTip( i18n( "Show statusbar" ) );
-    action->setWhatsThis( i18n( "<b>Show statusbar</b><p>Hides or shows the statusbar.</p>" ) );
+    action->setText( i18n( "Show &Status Bar" ) );
+    action->setToolTip( i18n( "Show status bar" ) );
+    action->setWhatsThis( i18n( "<b>Show status bar</b><p>Hides or shows the status bar.</p>" ) );
 
     action = actionCollection()->addAction( "configure_sessions", this, SLOT( configureSessions() ) );
     action->setText( i18n("Configure Sessions...") );
     action->setToolTip( i18n("Create/Delete/Activate Sessions") );
-    action->setWhatsThis( i18n( "<b>Configure Sessions</b><p>Shows a dialog to Create/Delete Sessions and set a new active session" ) );
+    action->setWhatsThis( i18n( "<b>Configure Sessions</b><p>Shows a dialog to Create/Delete Sessions and set a new active session.</p>" ) );
+
+    
+    action =  KStandardAction::configureNotifications(this, SLOT(configureNotifications()), actionCollection());
+    action->setText( i18n("Configure Notifications...") );
+    action->setToolTip( i18n("Configure Notifications") );
+    action->setWhatsThis( i18n( "<b>Configure Notifications</b><p>Shows a dialog that lets you configure notifications.</p>" ) );
+
+    action = actionCollection()->addAction( "about_platform", this, SLOT( showAboutPlatform() ) );
+    action->setText( i18n("About KDevelop Platform") );
+    action->setStatusTip( i18n("Show Information about KDevelop Platform") );
+    action->setWhatsThis( i18n( "<b>About KDevelop Platform</b><p>Shows a dialog with information about KDevelop Platform.</p>" ) );
+
 
 //     KToolBarPopupAction *popupAction;
 //     popupAction = new KToolBarPopupAction( KIcon( "process-stop" ),
@@ -191,7 +213,7 @@ void MainWindowPrivate::setupActions()
 
 //     popupAction->setShortcut( Qt::Key_Escape );
 //     popupAction->setToolTip( i18n( "Stop" ) );
-//     popupAction->setWhatsThis( i18n( "<b>Stop</b><p>Stops all running processes." ) );
+//     popupAction->setWhatsThis( i18n( "<b>Stop</b><p>Stops all running processes.</p>" ) );
 //     popupAction->setEnabled( false );
 //     connect( popupAction, SIGNAL( triggered() ),
 //              this, SLOT( stopButtonPressed() ) );
@@ -204,7 +226,7 @@ void MainWindowPrivate::setupActions()
                  this, SLOT( showMenuBar() ),
                  actionCollection());
     action->setToolTip( action->text() );
-    action->setWhatsThis( QString( "<b>%1</b><p>%2<p>" ).arg( action->text() ).arg( i18n( "Lets you toggle the menubar on/off." ) ) );
+    action->setWhatsThis( QString( "<b>%1</b><p>%2</p>" ).arg( action->text() ).arg( i18n( "Lets you toggle the menubar on/off." ) ) );
 
     action = actionCollection()->addAction( "view_next_window" );
     action->setText( i18n( "&Next Window" ) );
@@ -222,27 +244,13 @@ void MainWindowPrivate::setupActions()
     action->setWhatsThis( i18n( "<b>Previous window</b><p>Switches to the previous window.</p>" ) );
     action->setIcon(KIcon("go-previous"));
 
-    action = actionCollection()->addAction( "view_last_window" );
-    action->setText( i18n( "&Last Accessed Window" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( gotoLastWindow() ) );
-    action->setShortcut( Qt::ALT + Qt::SHIFT + Qt::Key_Up );
-    action->setToolTip( i18n( "Last accessed window" ) );
-    action->setWhatsThis( i18n( "<b>Last accessed window</b><p>Switches to the last viewed window (Hold the Alt key pressed and walk on by repeating the Up key).</p>" ) );
-
-    action = actionCollection()->addAction( "view_first_window" );
-    action->setText( i18n( "&First Accessed Window" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( gotoFirstWindow() ) );
-    action->setShortcut( Qt::ALT + Qt::SHIFT + Qt::Key_Down );
-    action->setToolTip( i18n( "First accessed window" ) );
-    action->setWhatsThis( i18n( "<b>First accessed window</b><p>Switches to the first accessed window (Hold the Alt key pressed and walk on by repeating the Down key).</p>" ) );
-
     action = actionCollection()->addAction( "new_window" );
     action->setIcon(KIcon( "window-new" ));
     action->setText( i18n( "&New Window" ) );
     action->setShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_N );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( newWindow() ) );
     action->setToolTip( i18n( "New Window" ) );
-    action->setWhatsThis( i18n( "<b>New Window</b><p>Creates a new window with a duplicate of current area.</p>" ) );
+    action->setWhatsThis( i18n( "<b>New Window</b><p>Creates a new window with a duplicate of the current area.</p>" ) );
 
     action = actionCollection()->addAction( "split_horizontal" );
     action->setIcon(KIcon( "view-split-top-bottom" ));
@@ -282,6 +290,32 @@ void MainWindowPrivate::setupActions()
     action->setWhatsThis( i18n( "<b>Add Tool View</b><p>Adds a new tool view to this window.</p>" ) );
 }
 
+void MainWindowPrivate::setupAreaSelectorActions()
+{
+    m_mainWindow->unplugActionList("area_selector");
+    QActionGroup *group = new QActionGroup(this);
+    QList<QAction*> areaActions;
+    foreach (Sublime::Area *a, m_mainWindow->controller()->defaultAreas())
+    {
+        KToggleAction *t = new KToggleAction(KIcon(a->iconName()), i18n("%1 Area", a->title()), this);
+        t->setData(a->objectName());
+        if (a->objectName() == m_mainWindow->area()->objectName())
+            t->setChecked(true);
+        connect (t, SIGNAL(toggled(bool)), this, SLOT(toggleArea(bool)));
+        areaActions << t;
+        group->addAction(t);
+    }
+    m_mainWindow->plugActionList("area_selector", areaActions);
+}
+
+void MainWindowPrivate::toggleArea(bool b)
+{
+    if (!b) return;
+    KAction *action = qobject_cast<KAction*>(sender());
+    if (!action) return;
+    m_mainWindow->controller()->showArea(action->data().toString(), m_mainWindow);
+}
+
 KActionCollection * MainWindowPrivate::actionCollection()
 {
     return m_mainWindow->actionCollection();
@@ -291,19 +325,21 @@ KActionCollection * MainWindowPrivate::actionCollection()
    and not from anywhere else.  Presently, when new XMLGUI client is added,
    it's actions are always merged, and there's no clear way to stop this.
    So, we execute the below code whenever a new client is added.  */
-void MainWindowPrivate::fixToolbar()
+/*
+ * Only re-enable if you manage to also remove the relevant actions from the 
+ * configure toolbar dialog. If not, we stay with the actions, as anything else
+ * confuses users.
+ * void MainWindowPrivate::fixToolbar()
 {
     QWidget* w = m_mainWindow->guiFactory()->container(
         "mainToolBar", m_mainWindow);
     if (w)
         foreach (QAction *a, w->actions())
         {
-            if (m_mainWindow->actionCollection()->action(a->objectName()) != a)
-            {
+            if ( !a->isSeparator() && (a != m_mainWindow->actionCollection()->action(a->objectName())) )
                 w->removeAction(a);
-            }
         }
-}
+}*/
 
 bool MainWindowPrivate::applicationQuitRequested() const
 {
