@@ -33,10 +33,10 @@
 namespace KDevelop
 {
 
-OpenProjectDialog::OpenProjectDialog( QWidget* parent )
+OpenProjectDialog::OpenProjectDialog( const KUrl& startUrl, QWidget* parent )
     : KAssistantDialog( parent )
 {
-    QWidget* page = new OpenProjectPage( this );
+    QWidget* page = new OpenProjectPage( startUrl, this );
     connect( page, SIGNAL( urlSelected( const KUrl& ) ), this, SLOT( validateOpenUrl( const KUrl& ) ) );
     openPage = addPage( page, "Select Directory/Project File" );
     page = new ProjectInfoPage( this );
@@ -88,30 +88,38 @@ void OpenProjectDialog::validateOpenUrl( const KUrl& url )
             OpenProjectPage* page2 = qobject_cast<OpenProjectPage*>( openPage->widget() );
             if( page2 )
             {
-                if( !isDir )
+                // Default manager
+                page->setProjectManager( "Generic Project Manager" );
+                // clear the filelist
+                m_fileList.clear();
+
+                if( isDir ) {
+                    // If a dir was selected fetch all files in it
+                    KIO::ListJob* job = KIO::listDir( m_url );
+                    connect( job, SIGNAL(entries(KIO::Job*, const KIO::UDSEntryList&)), 
+                                  SLOT(storeFileList(KIO::Job*, const KIO::UDSEntryList&)));
+                    KIO::NetAccess::synchronousRun( job, Core::self()->uiController()->activeMainWindow() );
+                } else {
+                    // Else we'lll just take the given file
+                    m_fileList << url.fileName();
+                }
+                // Now find a manager for the file(s) in our filelist.
+                bool managerFound = false;
+                foreach( const QString& manager, page2->projectFilters().keys() )
                 {
-                    bool managerFound = false;
-                    foreach( const QString& manager, page2->projectFilters().keys() )
+                    foreach( const QString& filterexp, page2->projectFilters()[manager] )
                     {
-                        foreach( const QString& filter, page2->projectFilters()[manager] )
-                        {
-                            QRegExp r( filter );
-                            r.setPatternSyntax( QRegExp::Wildcard );
-                            if( r.exactMatch( url.fileName() ) )
-                            {
-                                page->setProjectManager( manager );
-                                managerFound = true;
-                                break;
-                            }
-                        }
-                        if( managerFound ) 
+                        if( !m_fileList.filter( QRegExp( filterexp, Qt::CaseSensitive, QRegExp::Wildcard ) ).isEmpty() ) 
                         {
+                            managerFound = true;
                             break;
                         }
                     }
-                } else
-                {
-                    page->setProjectManager( "Generic Project Manager" );
+                    if( managerFound )
+                    {
+                        page->setProjectManager( manager );
+                        break;
+                    }
                 }
             }
         }
@@ -157,6 +165,18 @@ QString OpenProjectDialog::projectManager()
     return m_projectManager;
 }
 
+void OpenProjectDialog::storeFileList(KIO::Job*, const KIO::UDSEntryList& list)
+{
+    foreach( const KIO::UDSEntry& entry, list )
+    {
+        QString name = entry.stringValue( KIO::UDSEntry::UDS_NAME );
+        if( name != "." && name != ".." && !entry.isDir() )
+        {
+            m_fileList << name;
+        }
+    }
+}
+
 }
 
 #include "openprojectdialog.moc"
