@@ -12,12 +12,11 @@
 #define OPENPROJECTPAGE_H
 
 #include <QtGui/QWidget>
+#include <QtCore/QMap>
 
-class KFilePlacesView;
-class KFileTreeView;
-class KHistoryComboBox;
 class KUrl;
-class QModelIndex;
+class KFileItem;
+class KFileWidget;
 
 namespace KDevelop
 {
@@ -26,18 +25,18 @@ class OpenProjectPage : public QWidget
 {
 Q_OBJECT
 public:
-    OpenProjectPage( QWidget* parent = 0 );
+    OpenProjectPage( const KUrl& startUrl, QWidget* parent = 0 );
+    QMap<QString,QStringList> projectFilters() const;
 signals:
     void urlSelected( const KUrl& );
 private slots:
-    void changeUrl( const KUrl& );
-    void expandTreeView( const QModelIndex& idx );
-    void changeUrl( const QString& );
-    void activateUrl( const QString& );
+    void highlightFile(const QString&);
+    void opsEntered(const KUrl& item );
+    void comboTextChanged(const QString& );
 private:
-    KFilePlacesView* filePlacesView;
-    KFileTreeView* fileTreeView;
-    KHistoryComboBox* historyCombo;
+    KUrl getAbsoluteUrl( const QString& ) const;
+    KFileWidget* fileWidget;
+    QMap<QString,QStringList> m_projectFilters;
 };
 
 }
