@@ -48,10 +48,14 @@ Boston, MA 02110-1301, USA.
 #include <interfaces/contextmenuextension.h>
 #include <interfaces/iplugin.h>
 #include <interfaces/isession.h>
+#include <interfaces/idebugcontroller.h>
+
+#include <kross/krossplugin.h>
 
 #include "mainwindow.h"
 #include "core.h"
 #include "shellextension.h"
+#include <interfaces/idocumentationcontroller.h>
 
 namespace KDevelop
 {
@@ -136,14 +140,9 @@ PluginController::PluginController(Core *core)
 {
     setObjectName("PluginController");
     d->core = core;
-    kDebug() << "Fetching plugin info which matches:" << QString( "[X-KDevelop-Version] == %1" ).arg(KDEVELOP_PLUGIN_VERSION);
+    //kDebug() << "Fetching plugin info which matches:" << QString( "[X-KDevelop-Version] == %1" ).arg(KDEVELOP_PLUGIN_VERSION);
     d->plugins = KPluginInfo::fromServices( KServiceTypeTrader::self()->query( QLatin1String( "KDevelop/Plugin" ),
         QString( "[X-KDevelop-Version] == %1" ).arg(KDEVELOP_PLUGIN_VERSION) ) );
-    foreach( KPluginInfo p, d->plugins )
-    {
-        if( p.pluginName().contains("nongui") )
-                kDebug() << "FOUND test plugin" << p.pluginName();
-    }
     d->cleanupMode = PluginControllerPrivate::Running;
     d->m_manager = new QExtensionManager();
     // Register the KDevelop::IPlugin* metatype so we can properly unload it
@@ -175,7 +174,7 @@ void PluginController::cleanup()
 {
     if(d->cleanupMode != PluginControllerPrivate::Running)
     {
-        kDebug() << "called when not running. state =" << d->cleanupMode;
+        //kDebug() << "called when not running. state =" << d->cleanupMode;
         return;
     }
 
@@ -196,13 +195,30 @@ IPlugin* PluginController::loadPlugin( const QString& pluginName )
     return loadPluginInternal( pluginName );
 }
 
+bool PluginController::isEnabled( const KPluginInfo& info ) 
+{
+    KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
+    bool isEnabled = grp.readEntry( info.pluginName()+"Enabled", ShellExtension::getInstance()->defaultPlugins().isEmpty() || ShellExtension::getInstance()->defaultPlugins().contains( info.pluginName() ) );
+    //kDebug() << "read config:" << isEnabled << "is global plugin:" << isGlobalPlugin( info ) << "default:" << ShellExtension::getInstance()->defaultPlugins().isEmpty()  << ShellExtension::getInstance()->defaultPlugins().contains( info.pluginName() );
+    return !isGlobalPlugin( info ) || isEnabled;
+}
+
 void PluginController::initialize()
 {
     QMap<QString, bool> pluginMap;
-    // Get the default from the ShellExtension
-    foreach( const QString& s, ShellExtension::getInstance()->defaultPlugins() )
+    if( ShellExtension::getInstance()->defaultPlugins().isEmpty() ) 
     {
-        pluginMap.insert( s, true );
+        foreach( const KPluginInfo& pi, d->plugins )
+        {
+            pluginMap.insert( pi.pluginName(), true );
+        }
+    } else
+    {
+        // Get the default from the ShellExtension
+        foreach( const QString& s, ShellExtension::getInstance()->defaultPlugins() )
+        {
+            pluginMap.insert( s, true );
+        }
     }
 
     KConfigGroup grp = Core::self()->activeSession()->config()->group( pluginControllerGrp );
@@ -232,11 +248,13 @@ void PluginController::initialize()
         if( isGlobalPlugin( pi ) )
         {
             QMap<QString, bool>::const_iterator it = pluginMap.constFind( pi.pluginName() );
-            if( pluginMap.isEmpty() || ( it != pluginMap.constEnd() && it.value() ) )
+            if( it != pluginMap.constEnd() && it.value() )
             { 
+                // Plugin is mentioned in pluginmap and the value is true, so try to load it
                 loadPluginInternal( pi.pluginName() );
-                if( pluginMap.isEmpty() || it == pluginMap.constEnd() )
+                if( !grp.hasKey( pi.pluginName() + "Enabled" ) )
                 {
+                    // If plugin isn't listed yet, add it with true now
                     grp.writeEntry( pi.pluginName()+"Enabled", true );
                 }
             }
@@ -310,21 +328,16 @@ IPlugin *PluginController::loadPluginInternal( const QString &pluginId )
         return 0L;
     }
 
-    // Do not load KDevKrossManager directly, its indirectly loaded when loading
-    // any plugins written in a supported scripting language
-    // The kross-manager plugin needs the name+interfaces of the script-plugin
-    // as argument
-    // At a later point in time, we should try to move the plugin's code directly
-    // into shell
-    if( info.pluginName() == "KDevKrossManager" )
-    {
-        kDebug() << "tried to load KDevKrossManager, ignoring";
-        return 0;
-    }
-
     if ( d->loadedPlugins.contains( info ) )
         return d->loadedPlugins[ info ];
 
+    if( !isEnabled( info ) )
+    {
+        // Do not load disabled plugins
+        kWarning() << "Not loading plugin named" << pluginId << "because its been disabled!";
+        return 0;
+    }
+
     if( !hasMandatoryProperties( info ) ) {
         kWarning() << "Unable to load plugin named " << pluginId << "! Doesn't have all mandatory properties set";
         return 0;
@@ -333,9 +346,22 @@ IPlugin *PluginController::loadPluginInternal( const QString &pluginId )
     if( info.property("X-KDevelop-Mode") == "GUI"
         && Core::self()->setupFlags() == Core::NoUi )
     {
-        kWarning() << "Unable to load plugin named" << pluginId << ". Running in No-Ui mode, but the plugin says it needs a GUI";
+        kDebug() << "Unable to load plugin named" << pluginId << ". Running in No-Ui mode, but the plugin says it needs a GUI";
         return 0;
     }
+    
+    bool isKrossPlugin = false;
+    QString krossScriptFile;
+    if( info.property("X-KDevelop-PluginType").toString() == "Kross" ) 
+    {
+        isKrossPlugin = true;
+        krossScriptFile = KStandardDirs::locate( "appdata", info.service()->library(), KComponentData("kdevkrossplugins"));
+        if( krossScriptFile.isEmpty() || !QFileInfo( krossScriptFile ).exists() || !QFileInfo( krossScriptFile ).isReadable() )
+        {
+            kWarning() << "Unable to load kross plugin" << pluginId << ". Script file" << krossScriptFile << "not found or not readable";
+            return 0;
+        }
+    }
 
     kDebug() << "Attempting to load '" << pluginId << "'";
     emit loadingPlugin( info.pluginName() );
@@ -347,22 +373,41 @@ IPlugin *PluginController::loadPluginInternal( const QString &pluginId )
     {
         QVariant prop = info.property( "X-KDevelop-PluginType" );
         kDebug() << "Checked... starting to load:" << info.name() << "type:" << prop;
-        if(prop.toString()=="Kross")
+        
+        QString failedPlugin;
+        if( !loadDependencies( info, failedPlugin ) ) 
         {
+            kWarning() << "Could not load a required dependency:" << failedPlugin;
+            return 0;
+        }
+        loadOptionalDependencies( info );
+        
+        if( isKrossPlugin )
+        {
+            // Kross is special, we create always the same "plugin" which hooks up
+            // the script and makes the connection between C++ and script side
             kDebug() << "it is a kross plugin!!";
-            QStringList interfaces=info.property( "X-KDevelop-Interfaces" ).toStringList();
-            plugin = KServiceTypeTrader::createInstanceFromQuery<IPlugin>( QLatin1String( "KDevelop/Plugin" ),
-                            QString::fromLatin1( "[X-KDE-PluginInfo-Name]=='KDevKrossManager'" ),
-                            d->core, QVariantList() << interfaces << info.pluginName(), &str_error );
-            kDebug() << "kross plugin:" << plugin;
+            // Workaround for KAboutData constructor needing a KLocalizedString and 
+            // KLocalized string storing the char* for later usage
+            QString tmp = info.name();
+            int len = tmp.toUtf8().size();
+            char* name = new char[len+1];
+            memcpy( name, tmp.toUtf8().data(), len );
+            name[len] = '\0';
+            tmp = info.comment();
+            len = tmp.toUtf8().size();
+            char* comment = new char[len+1];
+            memcpy( comment, tmp.toUtf8().data(), len );
+            comment[len] = '\0';
+            // Create the kross plugin instance from the desktop file data.
+            plugin = new KrossPlugin( krossScriptFile, KAboutData( info.pluginName().toUtf8(), info.pluginName().toUtf8(),
+                              ki18n( name ), info.version().toUtf8(), ki18n( comment ), KAboutLicense::byKeyword( info.license() ).key() ), d->core );
         }
         else
         {
             plugin = KServiceTypeTrader::createInstanceFromQuery<IPlugin>( QLatin1String( "KDevelop/Plugin" ),
                     QString::fromLatin1( "[X-KDE-PluginInfo-Name]=='%1'" ).arg( pluginId ), d->core, QVariantList(), &str_error );
         }
-        loadDependencies( info );
-        loadOptionalDependencies( info );
     }
 
     if ( plugin )
@@ -379,7 +424,7 @@ IPlugin *PluginController::loadPluginInternal( const QString &pluginId )
         if( str_error.isEmpty() && !missingInterfaces.isEmpty() )
         {
             kWarning() << "Can't load plugin '" << pluginId
-                    << "' some of its required dependecies could not be fulfilled:" << endl
+                    << "' some of its required dependencies could not be fulfilled:" << endl
                     << missingInterfaces.join(",") << endl;
         }
         else
@@ -444,23 +489,34 @@ void PluginController::loadOptionalDependencies( const KPluginInfo& info )
     }
 }
 
-void PluginController::loadDependencies( const KPluginInfo& info )
+bool PluginController::loadDependencies( const KPluginInfo& info, QString& failedPlugin )
 {
     QVariant prop = info.property( "X-KDevelop-IRequired" );
+    QStringList loadedPlugins;
     if( prop.canConvert<QStringList>() )
     {
         QStringList deps = prop.toStringList();
         foreach( const QString &iface, deps )
         {
             KPluginInfo info = queryPlugins( QString("'%1' in [X-KDevelop-Interfaces]").arg(iface) ).first();
-            loadPluginInternal( info.pluginName() );
+            if( !loadPluginInternal( info.pluginName() ) )
+            {
+                foreach( const QString& name, loadedPlugins )
+                {
+                    unloadPlugin( name );
+                }
+                failedPlugin = info.pluginName();
+                return false;
+            }
+            loadedPlugins << info.pluginName();
         }
     }
+    return true;
 }
 
 IPlugin* PluginController::pluginForExtension( const QString& extension, const QString& pluginname)
 {
-    kDebug() << "Loading Plugin ("<< pluginname << ") for Extension:" << extension;
+    //kDebug() << "Loading Plugin ("<< pluginname << ") for Extension:" << extension;
     QStringList constraints;
     if (!pluginname.isEmpty())
         constraints << QString("[X-KDE-PluginInfo-Name]=='%1'").arg( pluginname );
@@ -470,7 +526,7 @@ IPlugin* PluginController::pluginForExtension( const QString& extension, const Q
 
 IPlugin *PluginController::pluginForExtension(const QString &extension, const QStringList &constraints)
 {
-    kDebug() << "Finding Plugin for Extension:" << extension << "|" << constraints;
+    //kDebug() << "Finding Plugin for Extension:" << extension << "|" << constraints;
     KPluginInfo::List infos = queryExtensionPlugins(extension, constraints);
 
     if( infos.isEmpty() )
@@ -483,7 +539,7 @@ IPlugin *PluginController::pluginForExtension(const QString &extension, const QS
 
 QList<IPlugin*> PluginController::allPluginsForExtension(const QString &extension, const QStringList &constraints)
 {
-    kDebug() << "Finding all Plugins for Extension:" << extension << "|" << constraints;
+    //kDebug() << "Finding all Plugins for Extension:" << extension << "|" << constraints;
     KPluginInfo::List infos = queryExtensionPlugins(extension, constraints);
     QList<IPlugin*> plugins;
     foreach (const KPluginInfo &info, infos)
@@ -491,7 +547,7 @@ QList<IPlugin*> PluginController::allPluginsForExtension(const QString &extensio
         IPlugin* plugin;
         if( d->loadedPlugins.contains( info ) )
             plugin = d->loadedPlugins[ info ];
-        else
+        else 
             plugin = loadPluginInternal( info.pluginName() );
 
         if (plugin)
@@ -525,6 +581,8 @@ QList<ContextMenuExtension> PluginController::queryPluginsForContextMenuExtensio
         IPlugin* plug = d->loadedPlugins[info];
         exts << plug->contextMenuExtension( context );
     }
+    exts << ICore::self()->debugController()->contextMenuExtension( context );
+    exts << ICore::self()->documentationController()->contextMenuExtension( context );
     return exts;
 }
 
