@@ -50,7 +50,6 @@ class PluginControllerPrivate;
  */
 class KDEVPLATFORMSHELL_EXPORT PluginController: public IPluginController
 {
-
     Q_OBJECT
 friend class Core;
 friend class CorePrivate;
@@ -167,11 +166,13 @@ private:
 
     bool checkForDependencies( const KPluginInfo& info, QStringList& missing ) const;
 
-    void loadDependencies( const KPluginInfo& );
+    bool loadDependencies( const KPluginInfo&, QString& failedPlugin );
     void loadOptionalDependencies( const KPluginInfo& info );
 
     void cleanup();
     void initialize();
+    
+    bool isEnabled( const KPluginInfo& info );
 
 private:
     class PluginControllerPrivate* const d;
