@@ -160,6 +160,9 @@ public:
     }
     QList<ProjectBaseItem*> itemsForUrl( const KUrl& url ) const
     {
+        if( !url.isValid() ) {
+            return QList<ProjectBaseItem*>();
+        }
         // TODO: This is moderately efficient, but could be much faster with a
         // QHash<QString, ProjectFolderItem> member. Would it be worth it?
         KUrl u = topItem->url();
@@ -200,15 +203,16 @@ public:
             return false;
         }
 
-
-        developerFileUrl = KUrl( projectFileUrl.directory( KUrl::AppendTrailingSlash ) );
-        developerFileUrl.addPath(".kdev4");
-        developerFileUrl.addPath( projectFileUrl.fileName() );
+        // developerfile == dirname(projectFileUrl) ."/.kdev4/". basename(projectfileUrl)
+        developerFileUrl = projectFileUrl.upUrl();
+        developerFileUrl.addPath( ".kdev4/" + projectFileUrl.fileName() );
 
         statJob = KIO::stat( developerFileUrl, KIO::HideProgressInfo );
         if( !statJob->exec() )
         {
-            KUrl dir = KUrl( projectFileUrl.directory( KUrl::AppendTrailingSlash ) + ".kdev4");
+            // the developerfile does not exist yet, check if its folder exists
+            // the developerfile itself will get created below
+            KUrl dir = developerFileUrl.upUrl();
             statJob = KIO::stat( dir, KIO::HideProgressInfo );
             if( !statJob->exec() )
             {
@@ -430,15 +434,18 @@ bool Project::open( const KUrl& projectFileUrl_ )
     KConfigGroup projectGroup = d->initKConfigObject();
     if (d->projectNameUsed(projectGroup))
         return false;
-    d->folder = d->projectFileUrl.directory( KUrl::AppendTrailingSlash );
+
+    d->folder = d->projectFileUrl.upUrl();
 
     IProjectFileManager* iface = d->fetchFileManager(projectGroup);
     if (!iface)
         return false;
 
     Q_ASSERT(d->manager);
-    if (!d->importTopItem(iface) )
+
+    if (!d->importTopItem(iface) ) {
         return false;
+    }
 
     d->loadVersionControlPlugin(projectGroup);
     ImportProjectJob* importJob = new ImportProjectJob( d->topItem, iface );
@@ -469,7 +476,7 @@ void Project::close()
 
 bool Project::inProject( const KUrl& url ) const
 {
-    if( url.isLocalFile() && QFileInfo( url.path() ).isFile() )
+    if( url.isLocalFile() && QFileInfo( url.toLocalFile() ).isFile() )
         return d->fileSet.contains( IndexedString( url ) );
     return ( !d->itemsForUrl( url ).isEmpty() );
 }
@@ -529,14 +536,6 @@ KUrl Project::relativeUrl( const KUrl& absolute ) const
     return KUrl::relativeUrl( folder(), absolute );
 }
 
-KUrl Project::urlRelativeToProject( const KUrl & relativeUrl ) const
-{
-    if ( KUrl::isRelativeUrl( relativeUrl.toLocalFile() ) )
-        return KUrl( folder(), relativeUrl.toLocalFile() );
-
-    return relativeUrl;
-}
-
 IProjectFileManager* Project::projectFileManager() const
 {
     return d->manager->extension<IProjectFileManager>();
