@@ -30,6 +30,7 @@
 #include <kurl.h>
 
 #include <interfaces/iproject.h>
+#include <interfaces/istatus.h>
 
 #include "shellexport.h"
 
@@ -150,16 +151,6 @@ public Q_SLOTS:
      **/
     KUrl relativeUrl(const KUrl& absoluteUrl) const;
 
-    /**
-     * Returns the absolute url corresponding to the given \a relativeUrl and
-     * the project directory.
-     *
-     * @param relativeUrl Relative url to convert
-     *
-     * @returns the absolute URL relative to projectDirectory()
-     **/
-    KUrl urlRelativeToProject(const KUrl& relativeUrl) const;
-
     /**
      * Check if the url specified by @a url is part of the project.
      * @a url can be either a relative url (to the project directory) or
@@ -177,6 +168,6 @@ private:
 
     class ProjectPrivate* const d;
 };
-
+    
 } // namespace KDevelop
 #endif
