@@ -1,6 +1,6 @@
 /* This file is part of KDevelop
 Copyright 2006 Adam Treat <treat@kde.org>
-Copyright 2007 Anreas Pakulat <apaku@gmx.de>
+Copyright 2007 Andreas Pakulat <apaku@gmx.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -31,6 +31,7 @@ Boston, MA 02110-1301, USA.
 #include <kaction.h>
 #include <kconfig.h>
 #include <klocale.h>
+#include <kpassivepopup.h>
 #include <kservice.h>
 #include <kstandardaction.h>
 #include <kmessagebox.h>
@@ -46,7 +47,7 @@ Boston, MA 02110-1301, USA.
 #include <ksettings/dialog.h>
 #include <kio/netaccess.h>
 #include <kstandarddirs.h>
-#include <kpassivepopup.h>
+#include <knotification.h>
 
 #include <ksettings/dispatcher.h>
 
@@ -68,6 +69,9 @@ Boston, MA 02110-1301, USA.
 #include "uicontroller.h"
 #include "documentcontroller.h"
 #include "openprojectdialog.h"
+#include <interfaces/iruncontroller.h>
+#include <language/backgroundparser/parseprojectjob.h>
+#include <kio/job.h>
 
 namespace KDevelop
 {
@@ -102,9 +106,10 @@ public:
     QList<KUrl> m_currentlyOpening; // project-file urls that are being opened
     IProject* m_configuringProject;
     ProjectController* q;
+    bool m_foundProjectFile; //Temporary flag used while searching the hierarchy for a project file
 
     ProjectControllerPrivate( ProjectController* p )
-        : m_core(0), model(0), selectionModel(0), dialog(0), m_configuringProject(0), q(p)
+        : m_core(0), model(0), selectionModel(0), dialog(0), m_configuringProject(0), q(p), m_foundProjectFile(false)
     {
     }
 
@@ -127,8 +132,8 @@ public:
             //plugins implementing IProjectManager, removing from that the
             //plugin that manages this project. Set this as blacklist on the
             //dialog
-            //@FIXME: Currently its important to set a parentApp on the kcm's
-            //thats different from the component name of the application, else
+            //@FIXME: Currently it is important to set a parentApp on the kcms
+            //that's different from the component name of the application, else
             //the plugin will show up on all projects settings dialogs.
 
             QStringList pluginsForPrj = findPluginsForProject( proj );
@@ -146,6 +151,21 @@ public:
         m_cfgDlgs[proj]->exec();
         m_configuringProject = 0;
     }
+    void saveListOfOpenedProjects()
+    {
+        KSharedConfig::Ptr config = Core::self()->activeSession()->config();
+        KConfigGroup group = config->group( "General Options" );
+    
+        KUrl::List openProjects;
+    
+        foreach( IProject* project, m_projects ) {
+            openProjects.append(project->projectFileUrl());
+        }
+    
+        group.writeEntry( "Open Projects", openProjects.toStringList() );
+        group.sync();
+    }
+
     QStringList findPluginsForProject( IProject* project )
     {
         QList<IPlugin*> plugins = m_core->pluginController()->loadedPlugins();
@@ -237,10 +257,10 @@ bool projectFileExists( const KUrl& u )
     }
 }
 
-KUrl ProjectDialogProvider::askProjectConfigLocation()
+KUrl ProjectDialogProvider::askProjectConfigLocation(const KUrl& startUrl)
 {
     Q_ASSERT(d);
-    OpenProjectDialog dlg( Core::self()->uiController()->activeMainWindow() );
+    OpenProjectDialog dlg( startUrl, Core::self()->uiController()->activeMainWindow() );
     if(dlg.exec() == QDialog::Rejected)
         return KUrl();
     
@@ -256,9 +276,9 @@ KUrl ProjectDialogProvider::askProjectConfigLocation()
         } else
         {
             KTemporaryFile tmp;
+            ///TODO: do we really want to set setAutoRemove to false??
             tmp.setAutoRemove( false );
             tmp.open();
-            tmp.close();
             writeNewProjectFile( KSharedConfig::openConfig( tmp.fileName(), KConfig::SimpleConfig ),
                             dlg.projectName(),
                             dlg.projectManager() );
@@ -324,9 +344,9 @@ void ProjectController::setupActions()
     action->setText( i18n( "Close All Projects" ) );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllProjects() ) );
     action->setToolTip( i18n( "Close all currently open projects" ) );
-    action->setWhatsThis( i18n( "<b>Close all projects</b><p>Closes all of the currently open projects." ) );
+    action->setWhatsThis( i18n( "<b>Close all projects</b><p>Closes all of the currently open projects.</p>" ) );
     action->setEnabled( false );
-    action->setIcon(KIcon("window-close"));
+    action->setIcon(KIcon("project-close"));
 
     d->m_closeProject = action = ac->addAction( "project_close" );
     connect( action, SIGNAL( triggered( bool ) ), SLOT( closeSelectedProjects() ) );
@@ -351,6 +371,11 @@ void ProjectController::setupActions()
     d->m_recentAction->setWhatsThis(
         i18n( "<b>Open recent project</b><p>Opens recently opened project.</p>" ) );
     d->m_recentAction->loadEntries( KConfigGroup(config, "RecentProjects") );
+    
+    KAction* openProjectForFileAction = new KAction( this );
+    ac->addAction("project_open_for_file", openProjectForFileAction);
+    openProjectForFileAction->setText("Open Project for Current File");
+    connect( openProjectForFileAction, SIGNAL(triggered(bool)), SLOT(openProjectForUrlSlot(bool)));
 }
 
 ProjectController::~ProjectController()
@@ -418,24 +443,96 @@ QList<IProject*> ProjectController::projects() const
     return d->m_projects;
 }
 
-bool ProjectController::openProject( const KUrl &projectFile )
+void ProjectController::eventuallyOpenProjectFile(KIO::Job* _job, KIO::UDSEntryList entries ) {
+    KIO::SimpleJob* job(dynamic_cast<KIO::SimpleJob*>(_job));
+    Q_ASSERT(job);
+    foreach(KIO::UDSEntry entry, entries) {
+        if(d->m_foundProjectFile)
+            break;
+        if(!entry.isDir()) {
+            QString name = entry.stringValue( KIO::UDSEntry::UDS_NAME );
+            
+            if(name.endsWith(".kdev4")) {
+                //We have found a project-file, open it
+                KUrl u(job->url());
+                u.addPath(name);
+                openProject(u);
+                d->m_foundProjectFile = true;
+            }
+        }
+    }
+}
+
+void ProjectController::openProjectForUrlSlot(bool) {
+    if(ICore::self()->documentController()->activeDocument()) {
+        KUrl url = ICore::self()->documentController()->activeDocument()->url();
+        IProject* project = ICore::self()->projectController()->findProjectForUrl(url);
+        if(!project) {
+            openProjectForUrl(url);
+        }else{
+            KMessageBox::error(Core::self()->uiController()->activeMainWindow(), i18n("Project already open: %1", project->name()));
+        }
+    }else{
+        KMessageBox::error(Core::self()->uiController()->activeMainWindow(), i18n("No active document"));
+    }
+}
+
+
+void ProjectController::openProjectForUrl(const KUrl& sourceUrl) {
+    KUrl dirUrl = sourceUrl.upUrl();
+    KUrl testAt = dirUrl;
+    
+    d->m_foundProjectFile = false;
+    
+    while(!testAt.path().isEmpty()) {
+        KUrl testProjectFile(testAt);
+        KIO::ListJob* job = KIO::listDir(testAt);
+        
+        connect(job, SIGNAL(entries(KIO::Job*,KIO::UDSEntryList)), SLOT(eventuallyOpenProjectFile(KIO::Job*,KIO::UDSEntryList)));
+        
+        KIO::NetAccess::synchronousRun(job, ICore::self()->uiController()->activeMainWindow());
+        if(d->m_foundProjectFile) {
+            //Fine! We have directly opened the project
+            d->m_foundProjectFile = false;
+            return;
+        }
+        KUrl oldTest = testAt;
+        testAt = testAt.upUrl();
+        if(oldTest == testAt)
+            break;
+    }
+    
+    KUrl askForOpen = d->dialog->askProjectConfigLocation(dirUrl);
+    
+    if(askForOpen.isValid())
+        openProject(askForOpen);
+}
+
+void ProjectController::openProject( const KUrl &projectFile )
 {
     KUrl url = projectFile;
 
     if ( url.isEmpty() )
     {
         url = d->dialog->askProjectConfigLocation();
+        if ( url.isEmpty() )
+            return;
     }
 
     if ( !url.isValid() )
     {
-        KPassivePopup::message( i18n( "Invalid Location: %1", url.prettyUrl() ), Core::self()->uiControllerInternal()->activeMainWindow() );
-        return false;
+        KMessageBox::error(Core::self()->uiControllerInternal()->activeMainWindow(),
+                           i18n("Invalid Location: %1", url.prettyUrl()));
+        return;
     }
     if ( d->m_currentlyOpening.contains(url))
     {
         kDebug() << "Already opening " << url << ". Aborting.";
-        return false;
+        KPassivePopup::message( i18n( "Project already being opened"), 
+                                i18n( "Already opening %1, not opening again", 
+                                      url.prettyUrl() ), 
+                                Core::self()->uiController()->activeMainWindow() );
+        return;
     }
 
     foreach( IProject* project, d->m_projects )
@@ -446,12 +543,15 @@ bool ProjectController::openProject( const KUrl &projectFile )
             { // close first, then open again by falling through
                 closeProject(project);
             } else { // abort
-                return false;
+                return;
             }
         }
     }
 
-    KPassivePopup::message( i18n( "Loading Project: %1", url.prettyUrl() ), Core::self()->uiControllerInternal()->activeMainWindow() );
+    KNotification* ev=new KNotification("LoadingProject", Core::self()->uiControllerInternal()->activeMainWindow());
+    ev->setText(i18n( "Loading Project: %1", url.prettyUrl() ));
+    ev->setComponentData(KGlobal::mainComponent());
+    ev->sendEvent();
 
     //FIXME Create the hidden directory if it doesn't exist
     if ( loadProjectPart() )
@@ -460,30 +560,41 @@ bool ProjectController::openProject( const KUrl &projectFile )
         //Now we can load settings for all of the Core objects including this one!!
 //         Core::loadSettings();
         d->m_core->pluginControllerInternal()->loadProjectPlugins();
-    } else
-        return false;
+    } else {
+        KMessageBox::error(Core::self()->uiControllerInternal()->activeMainWindow(),
+                           i18n("Cannot Load Projects View plugin, aborting."));
+        return;
+    }
 
     Project* project = new Project();
     if ( !project->open( url ) )
     {
-        KPassivePopup::message( i18n( "Project could not be opened: %1", url.prettyUrl() ), Core::self()->uiControllerInternal()->activeMainWindow() );
+        KMessageBox::error(Core::self()->uiControllerInternal()->activeMainWindow(),
+                           i18n( "Project could not be opened: %1", url.prettyUrl() ));
         delete project;
-        return false;
+        return;
     }
 
     d->m_currentlyOpening << url;
     d->m_closeAllProjects->setEnabled(true);
-    return true;
+    return;
 }
 
-bool ProjectController::projectImportingFinished( IProject* project )
+void ProjectController::projectImportingFinished( IProject* project )
 {
+    if( !project )
+    {
+        kWarning() << "OOOPS: 0-pointer project";
+        return;
+    }
     IPlugin *managerPlugin = project->managerPlugin();
     QList<IPlugin*> pluglist;
     pluglist.append( managerPlugin );
     d->m_projectPlugins.insert( project, pluglist );
     d->m_projects.append( project );
 
+    d->saveListOfOpenedProjects();
+    
 //     KActionCollection * ac = d->m_core->uiController()->defaultMainWindow()->actionCollection();
 //     QAction * action;
 
@@ -503,14 +614,8 @@ bool ProjectController::projectImportingFinished( IProject* project )
     KUrl::List parseList;
     if (parseAllProjectSources())
     {
-        // Add the project files to the background parser to be parsed.
-        QList<ProjectFileItem*> files = project->files();
-        foreach ( ProjectFileItem* file, files )
-        {
-            parseList.append( file->url() );
-        }
-        //Add low-priority parse jobs, with only the minimum parsed information
-        Core::self()->languageController()->backgroundParser()->addDocumentList( parseList, KDevelop::TopDUContext::VisibleDeclarationsAndContexts, 10000 );
+        KJob* parseProjectJob = new KDevelop::ParseProjectJob(project);
+        ICore::self()->runController()->registerJob(parseProjectJob);
     } else
     {
         // Add all currently open files that belong to the project to the background-parser,
@@ -524,9 +629,10 @@ bool ProjectController::projectImportingFinished( IProject* project )
         }
         Core::self()->languageController()->backgroundParser()->addDocumentList( parseList, KDevelop::TopDUContext::AllDeclarationsContextsAndUses, 10 );
     }
-    KPassivePopup::message( i18n( "Project loaded: %1", project->name() ), Core::self()->uiControllerInternal()->activeMainWindow() );
-
-    return true;
+    KNotification* ev=new KNotification("ProjectLoadedSuccessfully", Core::self()->uiControllerInternal()->activeMainWindow());
+    ev->setText(i18n( "Project loaded: %1", project->name() ));
+    ev->setComponentData(KGlobal::mainComponent());
+    ev->sendEvent();
 }
 
 // helper method for closeProject()
@@ -579,12 +685,13 @@ void ProjectController::initializePluginCleanup(IProject* proj)
     }
 }
 
-bool ProjectController::closeProject(IProject* proj)
+void ProjectController::closeProject(IProject* proj)
 {
     if(!proj || d->m_projects.indexOf(proj) == -1)
     {
-        return false;
+        return;
     }
+    d->m_projects.removeAll(proj);
     emit projectClosing(proj);
     //Core::self()->saveSettings();     // The project file is being closed.
                                         // Now we can save settings for all of the Core
@@ -593,13 +700,13 @@ bool ProjectController::closeProject(IProject* proj)
     closeAllOpenedFiles(proj);
     proj->close();
     proj->deleteLater();                //be safe when deleting
-    d->m_projects.removeAll(proj);
     if (d->m_projects.isEmpty())
     {
         initializePluginCleanup(proj);
     }
+    d->saveListOfOpenedProjects();
     emit projectClosed(proj);
-    return true;
+    return;
 }
 
 bool ProjectController::loadProjectPart()
@@ -636,10 +743,9 @@ IProject* ProjectController::findProjectByName( const QString& name )
 }
 
 
-bool ProjectController::configureProject( IProject* project )
+void ProjectController::configureProject( IProject* project )
 {
     d->projectConfig( project );
-    return true;
 }
 
 void ProjectController::addProject(IProject* project)
