@@ -1,6 +1,6 @@
 /* This file is part of KDevelop
 Copyright 2006 Adam Treat <treat@kde.org>
-Copyright 2007 Anreas Pakulat <apaku@gmx.de>
+Copyright 2007 Andreas Pakulat <apaku@gmx.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -22,10 +22,14 @@ Boston, MA 02110-1301, USA.
 #define KDEVPROJECTCONTROLLER_H
 
 #include <interfaces/iprojectcontroller.h>
+#include <kio/udsentry.h>
 
 #include "shellexport.h"
 
 class QModelIndex;
+namespace KIO {
+    class Job;
+}
 
 namespace KDevelop
 {
@@ -42,7 +46,7 @@ public:
     virtual ~IProjectDialogProvider();
 
 public Q_SLOTS:
-    virtual KUrl askProjectConfigLocation() = 0;
+    virtual KUrl askProjectConfigLocation(const KUrl& startUrl = KUrl()) = 0;
     virtual bool userWantsReopen() = 0;
 };
 
@@ -74,11 +78,14 @@ public:
     KUrl projectsBaseDirectory() const;
 
 public Q_SLOTS:
-    virtual bool openProject( const KUrl &KDev4ProjectFile = KUrl() );
-    virtual bool projectImportingFinished( IProject* );
-    virtual bool closeProject( IProject* );
+    virtual void openProjectForUrl( const KUrl &sourceUrl );
+    virtual void openProject( const KUrl &KDev4ProjectFile = KUrl() );
+    void projectImportingFinished( IProject* );
+    virtual void closeProject( IProject* );
     virtual void closeAllProjects();
-    virtual bool configureProject( IProject* );
+    virtual void configureProject( IProject* );
+    void eventuallyOpenProjectFile(KIO::Job*,KIO::UDSEntryList);
+    void openProjectForUrlSlot(bool);
 //     void changeCurrentProject( ProjectBaseItem* );
 
 protected:
@@ -120,7 +127,7 @@ public:
     ProjectControllerPrivate* const d;
 
 public Q_SLOTS:
-    virtual KUrl askProjectConfigLocation();
+    virtual KUrl askProjectConfigLocation(const KUrl& sta);
     virtual bool userWantsReopen();
 };
 
