@@ -53,6 +53,8 @@
 #include "partcontroller.h"
 #include "plugincontroller.h"
 #include "documentcontroller.h"
+#include <interfaces/ilanguagecontroller.h>
+#include <interfaces/icompletionsettings.h>
 
 namespace KDevelop {
 
@@ -150,6 +152,14 @@ struct TextDocumentPrivate {
         EditorIntegrator::addDocument( m_textDocument->textDocument() );
         m_textDocument->notifyLoaded();
     }
+    
+    void documentSaved(KTextEditor::Document* document, bool saveAs)
+    {
+        Q_UNUSED(document);
+        Q_UNUSED(saveAs);
+        m_textDocument->notifySaved();
+        m_textDocument->notifyStateChanged();
+    }
 
 private:
     TextDocument *m_textDocument;
@@ -235,6 +245,8 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
                  this, SLOT(textChanged(KTextEditor::Document*)));
         connect(d->document, SIGNAL(documentUrlChanged(KTextEditor::Document*)),
                  this, SLOT(documentUrlChanged(KTextEditor::Document*)));
+        connect(d->document, SIGNAL(documentSavedOrUploaded(KTextEditor::Document*,bool)),
+                 this, SLOT(documentSaved(KTextEditor::Document*,bool)));
 
         KTextEditor::ModificationInterface *iface = qobject_cast<KTextEditor::ModificationInterface*>(d->document);
         if (iface)
@@ -254,12 +266,8 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
         connect(view, SIGNAL(contextMenuAboutToShow(KTextEditor::View*,QMenu*)), this, SLOT(populateContextMenu(KTextEditor::View*,QMenu*)));
     }
 
-    if (KTextEditor::CodeCompletionInterface* cc = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view)) {
-        KConfigGroup group(KGlobal::config(), "Language Support");
-        bool automaticInvocation = group.readEntry( "Automatic Invocation", false );
-
-        cc->setAutomaticInvocationEnabled(automaticInvocation);
-    }
+    if (KTextEditor::CodeCompletionInterface* cc = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view))
+        cc->setAutomaticInvocationEnabled(core()->languageController()->completionSettings()->automaticCompletionEnabled());
 
     return view;
 }
@@ -349,8 +357,6 @@ bool TextDocument::save(DocumentSaveMode mode)
 
     if (d->document->save())
     {
-        notifyStateChanged();
-        notifySaved();
         return true;
     }
     return false;
@@ -455,6 +461,7 @@ bool TextDocument::close(DocumentSaveMode mode)
     if (!PartDocument::close(mode))
         return false;
 
+    KDevelop::EditorIntegrator::removeDocument(d->document);
     d->document->deleteLater();
 
     return true;
@@ -526,8 +533,12 @@ void KDevelop::TextView::setInitialRange(KTextEditor::Range range) {
 void KDevelop::TextView::setState(const QString & state)
 {
     static QRegExp re("Cursor=([\\d]+),([\\d]+)");
-    if (d->editor && d->editor->editorView() && re.exactMatch(state))
-        d->editor->editorView()->setCursorPosition(KTextEditor::Cursor(re.cap(1).toInt(), re.cap(2).toInt()));
+    if (re.exactMatch(state)) {
+        if (d->editor && d->editor->editorView() && re.exactMatch(state))
+            d->editor->editorView()->setCursorPosition(KTextEditor::Cursor(re.cap(1).toInt(), re.cap(2).toInt()));
+        else
+            setInitialRange(KTextEditor::Range(KTextEditor::Cursor(re.cap(1).toInt(), re.cap(2).toInt()), 0));
+    }
 }
 
 QString KDevelop::TextDocument::documentType() const
@@ -564,11 +575,6 @@ KDevelop::TextEditorWidget::TextEditorWidget(QWidget* parent)
     d->widgetLayout->setSpacing(0);
 
     setLayout(d->widgetLayout);
-    
-    d->statusBar = new KStatusBar(this);
-    d->statusLabel = new QLabel(d->statusBar);
-    d->statusBar->addPermanentWidget(d->statusLabel);
-    d->widgetLayout->addWidget(d->statusBar);
 }
 
 KDevelop::TextEditorWidget::~TextEditorWidget()
@@ -579,7 +585,6 @@ KDevelop::TextEditorWidget::~TextEditorWidget()
 void KDevelop::TextEditorWidget::viewStatusChanged(KTextEditor::View*, const KTextEditor::Cursor& newPosition)
 {
     d->status = i18n(" Line: %1 Col: %2 ", KGlobal::locale()->formatNumber(newPosition.line() + 1, 0), KGlobal::locale()->formatNumber(newPosition.column() + 1, 0));
-    d->statusLabel->setText(d->status);
     emit statusChanged();
 }
 
