@@ -46,6 +46,9 @@
 #include "partdocument.h"
 #include "textdocument.h"
 #include "documentcontroller.h"
+#include "assistantpopup.h"
+#include <kactioncollection.h>
+#include <ktexteditor/view.h>
 
 namespace KDevelop {
 
@@ -63,9 +66,11 @@ public:
         desired["org.kdevelop.ProblemReporterView"] = Sublime::Bottom;
         desired["org.kdevelop.OutputView"] = Sublime::Bottom;
         desired["org.kdevelop.ContextBrowser"] = Sublime::Bottom;
+        desired["org.kdevelop.debugger.ConsoleView"] = Sublime::Bottom;
         Sublime::Area* a =
             new Sublime::Area(m_controller, "code", i18n("Code"));
         a->setDesiredToolViews(desired);
+        a->setIconName("document-edit");
         controller->addDefaultArea(a);
 
         desired.clear();
@@ -75,6 +80,7 @@ public:
         desired["org.kdevelop.debugger.ConsoleView"] = Sublime::Bottom;
         a = new Sublime::Area(m_controller, "debug", i18n("Debug"));
         a->setDesiredToolViews(desired);
+        a->setIconName("tools-report-bug");
         controller->addDefaultArea(a);
 
         desired.clear();
@@ -82,7 +88,8 @@ public:
         desired["org.kdevelop.CoverageReport"] = Sublime::Right;
         a = new Sublime::Area(m_controller, "test", i18n("Test"));
         a->setDesiredToolViews(desired);
-        controller->addDefaultArea(a);        
+        a->setIconName("applications-engineering");
+        controller->addDefaultArea(a);
 
         if(!(Core::self()->setupFlags() & Core::NoUi)) 
         {
@@ -118,6 +125,9 @@ public:
     QList<Sublime::MainWindow*> sublimeWindows;
     bool areasRestored;
 
+    //Currently shown assistant popup.
+    QPointer<AssistantPopup> currentShownAssistant;
+
 private:
     UiController *m_controller;
 };
@@ -157,17 +167,41 @@ UiController::UiController(Core *core)
     :Sublime::Controller(0), IUiController(), d(new UiControllerPrivate(this))
 {
     setObjectName("UiController");
-    // FIXME: restore.
-#if 0
-    KSettings::Dispatcher::registerComponent( KGlobal::mainComponent(),
-                                    defaultMainWindow(), "loadSettings" );
-    KSettings::Dispatcher::registerComponent( KComponentData("kdevplatform"),
-                                    defaultMainWindow(), "loadSettings" );
-#endif
     d->core = core;
+
+    if (!defaultMainWindow())
+        return;
+
     connect( QApplication::instance(),
              SIGNAL( focusChanged( QWidget*, QWidget* ) ),
             this, SLOT( widgetChanged( QWidget*, QWidget* ) ) );
+
+    KActionCollection * actions = defaultMainWindow()->actionCollection();
+
+    KAction* assistantaction1 = actions->addAction("assistant_action_1");
+    assistantaction1->setText( i18n("&Assistant Action 1") );
+    assistantaction1->setShortcut( Qt::ALT | Qt::Key_1 );
+    connect(assistantaction1, SIGNAL(triggered(bool)), this, SLOT(assistantAction1()));
+
+    KAction* assistantaction2 = actions->addAction("assistant_action_2");
+    assistantaction2->setText( i18n("&Assistant Action 2") );
+    assistantaction2->setShortcut( Qt::ALT | Qt::Key_2 );
+    connect(assistantaction2, SIGNAL(triggered(bool)), this, SLOT(assistantAction2(bool)));
+            
+    KAction* assistantaction3 = actions->addAction("assistant_action_3");
+    assistantaction3->setText( i18n("&Assistant Action 3") );
+    assistantaction3->setShortcut( Qt::ALT | Qt::Key_3 );
+    connect(assistantaction3, SIGNAL(triggered(bool)), this, SLOT(assistantAction3(bool)));
+
+    KAction* assistantaction4 = actions->addAction("assistant_action_4");
+    assistantaction4->setText( i18n("&Assistant Action 4") );
+    assistantaction4->setShortcut( Qt::ALT | Qt::Key_4 );
+    connect(assistantaction4, SIGNAL(triggered(bool)), this, SLOT(assistantAction4(bool)));
+
+    KAction* assistantactionhide = actions->addAction("assistant_action_hide");
+    assistantactionhide->setText( i18n("&Hide Assistant") );
+    assistantactionhide->setShortcut( Qt::ALT | Qt::Key_0 );
+    connect(assistantactionhide, SIGNAL(triggered(bool)), this, SLOT(assistantHide()));
 }
 
 UiController::~UiController()
@@ -218,6 +252,37 @@ void UiController::switchToArea(const QString &areaName, SwitchMode switchMode)
 }
 
 
+QWidget* UiController::findToolView(const QString& name, IToolViewFactory *factory, FindFlags flags)
+{
+    if(!d->areasRestored)
+        return 0;
+
+    QList< Sublime::View* > views = activeArea()->toolViews();
+    foreach(Sublime::View* view, views) {
+        Sublime::ToolDocument *doc = dynamic_cast<Sublime::ToolDocument*>(view->document());
+        if(doc && doc->title() == name && view->widget()) {
+            if(flags & Raise)
+                view->requestRaise();
+            return view->widget();
+        }
+    }
+    
+    if(flags & Create)
+    {
+        if(!d->factoryDocuments.contains(factory))
+            d->factoryDocuments[factory] = new Sublime::ToolDocument(name, this, new UiToolViewFactory(factory));
+        
+        Sublime::ToolDocument *doc = d->factoryDocuments[factory];
+
+        addToolViewToArea(factory, doc, activeArea());
+        
+        if(flags & Raise)
+            findToolView(name, factory, Raise);
+    }
+
+    return 0;
+}
+
 void UiController::addToolView(const QString & name, IToolViewFactory *factory)
 {
     kDebug() ;
@@ -592,6 +657,70 @@ void UiController::registerStatus(QObject* status)
     mw->registerStatus(status);
 }
 
+void UiController::hideAssistant(const KDevelop::IAssistant::Ptr& assistant)
+{
+    if(d->currentShownAssistant && d->currentShownAssistant->assistant() == assistant) {
+        disconnect(d->currentShownAssistant->assistant().data(), SIGNAL(hide()), this, SLOT(assistantHide()));
+        disconnect(d->currentShownAssistant->assistant().data(), SIGNAL(actionsChanged()), this, SLOT(assistantActionsChanged()));
+        
+        AssistantPopup* oldPopup = d->currentShownAssistant;
+        d->currentShownAssistant = 0;
+        oldPopup->hide();
+        oldPopup->deleteLater(); //We have to do deleteLater, so we don't get problems when an assistant hides itself
+    }
 }
 
+void UiController::popUpAssistant(const KDevelop::IAssistant::Ptr& assistant)
+{
+    assistantHide();
+    
+    Sublime::View* view = d->activeSublimeWindow->activeView();
+    TextEditorWidget* textWidget = dynamic_cast<TextEditorWidget*>(view->widget());
+    if(textWidget && textWidget->editorView()) {
+
+        d->currentShownAssistant = new AssistantPopup(textWidget->editorView(), assistant);
+        if(assistant->actions().count())
+            d->currentShownAssistant->show();
+
+        connect(assistant.data(), SIGNAL(hide()), SLOT(assistantHide()), Qt::DirectConnection);
+        connect(assistant.data(), SIGNAL(actionsChanged()), SLOT(assistantActionsChanged()), Qt::DirectConnection);
+    }
+}
+
+void UiController::assistantAction1() {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction1();
+}
+
+void UiController::assistantAction2(bool) {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction2();
+}
+
+void UiController::assistantAction3(bool) {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction3();
+}
+void UiController::assistantAction4(bool) {
+    if(d->currentShownAssistant)
+        d->currentShownAssistant->executeAction4();
+}
+
+void UiController::assistantHide() {
+    if(d->currentShownAssistant)
+        hideAssistant(d->currentShownAssistant->assistant());
+}
+
+void UiController::assistantActionsChanged() {
+    if(d->currentShownAssistant)
+        popUpAssistant(d->currentShownAssistant->assistant());
+}
+
+
+}
+
+
+
+
+
 #include "uicontroller.moc"
