@@ -26,7 +26,7 @@
 #include "areaindex.h"
 #include "sublimedefs.h"
 
-#include <KConfigGroup>
+#include <KDE/KConfigGroup>
 
 namespace Sublime {
 
@@ -72,6 +72,9 @@ public:
     QString title() const;
     void setTitle(const QString &title);
 
+    QString iconName() const;
+    void setIconName(const QString &iconName);
+
     /**Adds the @p view to the list of views at the given area index.*/
     void addView(View *view, AreaIndex *index);
 
@@ -194,6 +197,9 @@ Q_SIGNALS:
     /**Emitted when a toolview is moved to a different position.*/
     void toolViewMoved(Sublime::View*, Sublime::Position);
 
+private Q_SLOTS:
+    void positionChanged(Sublime::View*, int);
+
 private:
     template <typename Operator>
     WalkerMode walkViewsInternal(Operator &op, AreaIndex *index);
