@@ -36,6 +36,7 @@
 #include "view.h"
 #include "controller.h"
 #include "container.h"
+#include "ideal.h"
 
 namespace Sublime {
 
@@ -70,12 +71,19 @@ void MainWindow::setArea(Area *area)
     d->area = area;
     d->reconstruct();
     d->activateFirstVisibleView();
+    initializeStatusBar();
     emit areaChanged(area);
     d->ignoreDockShown = false;
     
     loadSettings();
 }
 
+void MainWindow::initializeStatusBar()
+{
+    //nothing here, reimplement in the subclasses if you want to have status bar
+    //inside the bottom toolview buttons row
+}
+
 void MainWindow::resizeEvent(QResizeEvent* event)
 {
     return KParts::MainWindow::resizeEvent(event);
@@ -211,7 +219,10 @@ void MainWindow::loadSettings()
     
     KConfigGroup uiGroup = KGlobal::config()->group("UiSettings");
     foreach (Container *container, findChildren<Container*>())
+    {
         container->setTabBarHidden(uiGroup.readEntry("TabBarVisibility", 1) == 0);
+        container->setOpenAfterCurrent(uiGroup.readEntry("TabBarOpenAfterCurrent", 1) == 1);
+    }
 
     cg.sync();
 
@@ -273,6 +284,11 @@ void MainWindow::enableAreaSettingsSave()
     d->autoAreaSettingsSave = true;
 }
 
+QWidget *MainWindow::statusBarLocation()
+{
+    return d->idealMainWidget->statusBarLocation();
+}
+
 }
 
 #include "mainwindow.moc"
