@@ -84,8 +84,17 @@ Q_SIGNALS:
     void activeToolViewChanged(Sublime::View*);
     /**Emitted when the user interface settings have changed.*/
     void settingsLoaded();
+    
+    /**Emitted when a new view is added to the mainwindow.*/
+    void viewAdded(Sublime::View*);
+    /**Emitted when a view is going to be removed from the mainwindow.*/
+    void aboutToRemoveView(Sublime::View*);
+
 
 protected:
+    QWidget *statusBarLocation();
+    virtual void initializeStatusBar();
+
 public: // FIXME?
     /**Saves size/toolbar/menu/statusbar settings to the global configuration file.
     Reimplement in subclasses to save more and don't forget to call inherited method.*/
