@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright 2006-2007 Alexander Dymo  <adymo@kdevelop.org>       *
+ *   Copyright 2006-2009 Alexander Dymo  <adymo@kdevelop.org>              *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -18,18 +18,19 @@
  ***************************************************************************/
 #include "mainwindow_p.h"
 
-#include <QMenu>
-#include <QLayout>
-#include <QSplitter>
-#include <QDockWidget>
-#include <QWidgetAction>
-#include <QComboBox>
-#include <QHBoxLayout>
-#include <QToolButton>
-#include <QMenu>
-#include <QCommonStyle>
+#include <QtGui/QMenu>
+#include <QtGui/QLayout>
+#include <QtGui/QSplitter>
+#include <QtGui/QDockWidget>
+#include <QtGui/QWidgetAction>
+#include <QtGui/QComboBox>
+#include <QtGui/QHBoxLayout>
+#include <QtGui/QToolButton>
+#include <QtGui/QCommonStyle>
 
 #include <kdebug.h>
+#include <klocale.h>
+#include <kactionmenu.h>
 #include <kacceleratormanager.h>
 #include <kactioncollection.h>
 
@@ -41,74 +42,138 @@
 #include "controller.h"
 #include "mainwindow.h"
 #include "ideal.h"
+#include <KToolBar>
+#include <KSelectAction>
+#include <ktoggleaction.h>
 
 namespace Sublime {
 
-AreaSelectorWidget::AreaSelectorWidget(QWidget *parent,
-                                       MainWindow* window,
-                                       Controller* controller)
-: QWidget(parent), window_(window), controller_(controller)
-{
-    QHBoxLayout* layout = new QHBoxLayout(this);
-    combo_ = new QComboBox(this);
-    layout->addWidget(combo_);
-    QToolButton* button = new QToolButton(this);
-    button->setIcon(KIcon("configure"));
-    button->setPopupMode(QToolButton::InstantPopup);
-    layout->addWidget(button);
-
-    QMenu *menu = new QMenu(button);
-    menu->addAction("Reset Current Area", this,
-                    SLOT(resetCurrentArea()));
-    menu->addAction("New Area");
-    menu->addAction("Delete Area");
-    button->setMenu(menu);
-
-    foreach (Area *a, controller_->defaultAreas())
-    {
-        areaIds_.push_back(a->objectName());
-        combo_->addItem(a->title());
-        if (a->title() == window->area()->title())
-            combo_->setCurrentIndex(combo_->count()-1);
-    }
-    connect (combo_, SIGNAL(activated(int)), this, SLOT(activateArea(int)));
-    connect (window_, SIGNAL(areaChanged(Sublime::Area*)),
-             this, SLOT(areaChanged(Sublime::Area*)));
-}
-
-void AreaSelectorWidget::resetCurrentArea()
-{
-    controller_->resetCurrentArea(window_);
-}
-
-void AreaSelectorWidget::activateArea(int index)
-{
-    controller_->showArea(areaIds_[index], window_);
-}
-
-void AreaSelectorWidget::areaChanged(Sublime::Area* area)
-{
-    for (int i = 0; i < combo_->count(); ++i)
-        if (combo_->itemText(i) == area->title())
-        {
-            combo_->setCurrentIndex(i);
-            break;
-        }
-}
-
-QWidget* AreaSelectionAction::createWidget(QWidget* parent)
-{
-    return new AreaSelectorWidget(parent, window_, controller_);
-}
-
 MainWindowPrivate::MainWindowPrivate(MainWindow *w, Controller* controller)
 :controller(controller), area(0), activeView(0), activeToolView(0), centralWidget(0),
  ignoreDockShown(false), autoAreaSettingsSave(false), m_mainWindow(w)
 {
-    recreateCentralWidget();
+    KActionCollection *ac = m_mainWindow->actionCollection();
 
-    KAction* action = new AreaSelectionAction(this, controller);
-    m_mainWindow->actionCollection()->addAction("switch_area", action);
+    KAction* action = new KAction(i18n("Show Left Dock"), this);
+    action->setCheckable(true);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_L);
+    connect(action, SIGNAL(toggled(bool)), SLOT(showLeftDock(bool)));
+    ac->addAction("show_left_dock", action);
+
+    action = new KAction(i18n("Show Right Dock"), this);
+    action->setCheckable(true);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_R);
+    connect(action, SIGNAL(toggled(bool)), SLOT(showRightDock(bool)));
+    ac->addAction("show_right_dock", action);
+
+    action = new KAction(i18n("Show Bottom Dock"), this);
+    action->setCheckable(true);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_B);
+    connect(action, SIGNAL(toggled(bool)), SLOT(showBottomDock(bool)));
+    ac->addAction("show_bottom_dock", action);
+
+    action = new KAction(i18n("Show Top Dock"), this);
+    action->setCheckable(true);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_T);
+    connect(action, SIGNAL(toggled(bool)), SLOT(showTopDock(bool)));
+    ac->addAction("show_top_dock", action);
+
+    action = new KAction(i18n("Focus Editor"), this);
+    action->setShortcuts(QList<QKeySequence>() << (Qt::META | Qt::CTRL | Qt::Key_E) << Qt::META + Qt::Key_C);
+    connect(action, SIGNAL(triggered(bool)), this, SLOT(focusEditor()));
+    m_mainWindow->actionCollection()->addAction("focus_editor", action);
+
+    action = new KAction(i18n("Hide/Restore Docks"), this);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_H);
+    connect(action, SIGNAL(triggered(bool)), SLOT(toggleDocksShown()));
+    ac->addAction("hide_all_docks", action);
+
+    action = new KAction(i18n("Next Tool View"), this);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_N);
+    connect(action, SIGNAL(triggered(bool)), SLOT(selectNextDock()));
+    ac->addAction("select_next_dock", action);
+
+    action = new KAction(i18n("Previous Tool View"), this);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_P);
+    connect(action, SIGNAL(triggered(bool)), SLOT(selectPreviousDock()));
+    ac->addAction("select_previous_dock", action);
+
+    action = new KAction(i18n("Remove view"), this);
+    connect(action, SIGNAL(triggered(bool)), SLOT(removeView()));
+    ac->addAction("remove_view", action);
+
+    action = new KAction(i18n("Anchor Current Dock"), this);
+    action->setCheckable(true);
+    action->setEnabled(false);
+    action->setShortcut(Qt::META | Qt::CTRL | Qt::Key_A);
+    connect(action, SIGNAL(toggled(bool)), SLOT(anchorCurrentDock(bool)));
+    ac->addAction("anchor_current_dock", action);
+
+    action = new KAction(i18n("Maximize Current Dock"), this);
+    action->setCheckable(true);
+    action->setEnabled(false);
+    connect(action, SIGNAL(toggled(bool)), SLOT(maximizeCurrentDock(bool)));
+    ac->addAction("maximize_current_dock", action);
+
+    action = new KActionMenu(i18n("Tool Views"), this);
+    ac->addAction("docks_submenu", action);
+
+    recreateCentralWidget();
+}
+
+void MainWindowPrivate::showLeftDock(bool b)
+{
+    idealMainWidget->showLeftDock(b);
+}
+
+void MainWindowPrivate::showBottomDock(bool b)
+{
+    idealMainWidget->showBottomDock(b);
+}
+
+void MainWindowPrivate::showRightDock(bool b)
+{
+    idealMainWidget->showRightDock(b);
+}
+
+void MainWindowPrivate::showTopDock(bool b)
+{
+    idealMainWidget->showTopDock(b);
+}
+
+void MainWindowPrivate::anchorCurrentDock(bool b)
+{
+    idealMainWidget->anchorCurrentDock(b);
+}
+
+void MainWindowPrivate::maximizeCurrentDock(bool b)
+{
+    idealMainWidget->maximizeCurrentDock(b);
+}
+
+void MainWindowPrivate::focusEditor()
+{
+    idealMainWidget->focusEditor();
+}
+
+void MainWindowPrivate::toggleDocksShown()
+{
+    idealMainWidget->toggleDocksShown();
+}
+
+void MainWindowPrivate::removeView()
+{
+    idealMainWidget->removeView();
+}
+
+void MainWindowPrivate::selectNextDock()
+{
+    idealMainWidget->selectNextDock();
+}
+
+void MainWindowPrivate::selectPreviousDock()
+{
+    idealMainWidget->selectPreviousDock();
 }
 
 Area::WalkerMode MainWindowPrivate::IdealToolViewCreator::operator() (View *view, Sublime::Position position)
@@ -276,7 +341,7 @@ slotDockShown(Sublime::View* view, Sublime::Position pos, bool shown)
     area->setShownToolView(pos, id);
 }
 
-void MainWindowPrivate::viewAdded(Sublime::AreaIndex *index, Sublime::View */*view*/)
+void MainWindowPrivate::viewAdded(Sublime::AreaIndex *index, Sublime::View *view)
 {
     ViewCreator viewCreator(this);
     QSplitter *splitter = m_indexSplitters[index];
@@ -294,6 +359,7 @@ void MainWindowPrivate::viewAdded(Sublime::AreaIndex *index, Sublime::View */*vi
         delete container;
     }
     area->walkViews(viewCreator, index);
+    emit m_mainWindow->viewAdded( view );
 }
 
 void Sublime::MainWindowPrivate::raiseToolView(Sublime::View * view)
@@ -315,6 +381,8 @@ void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::Vi
         kWarning() << "Splitter does not have a left widget!";
         return;
     }
+    
+    emit m_mainWindow->aboutToRemoveView( view );
 
     if (view->widget())
         widgetToView.remove(view->widget());
@@ -325,11 +393,13 @@ void MainWindowPrivate::aboutToRemoveView(Sublime::AreaIndex *index, Sublime::Vi
         bool wasActive = m_mainWindow->activeView() == view;
         //container is not empty or this is a root index
         //just remove a widget
-        container->removeWidget(view->widget());
-        view->widget()->setParent(0);
-        //activate what is visible currently in the container if the removed view was active
-        if (wasActive)
-            return m_mainWindow->setActiveView(container->viewForWidget(container->currentWidget()));
+		if( view->widget() ) {
+			container->removeWidget(view->widget());
+			view->widget()->setParent(0);
+			//activate what is visible currently in the container if the removed view was active
+			if (wasActive)
+				return m_mainWindow->setActiveView(container->viewForWidget(container->currentWidget()));
+		}
     }
     else
     {
